@@ -61,23 +61,39 @@ class CopyProcess(object):
   def __init__(self):
     self.__process = client.CopyProcess()
 
+  def parallel(self, parallel):
+    """ Add a config job to the copy process in order to set the number of
+        parallel copy jobs.
+
+    :param parallel: number of parallel copy jobs
+    :type  parallel: integer
+    """
+    self.__process.parallel(parallel)
+
   def add_job(self,
               source,
               target,
-              sourcelimit    = 1,
-              force          = False,
-              posc           = False,
-              coerce         = False,
-              mkdir          = False,
-              thirdparty     = 'none',
-              checksummode   = 'none',
-              checksumtype   = '',
-              checksumpreset = '',
-              dynamicsource  = False,
-              chunksize      = 4194304,
-              parallelchunks = 8,
-              inittimeout    = 600,
-              tpctimeout     = 1800):
+              sourcelimit     = 1,
+              force           = False,
+              posc            = False,
+              coerce          = False,
+              mkdir           = False,
+              thirdparty      = 'none',
+              checksummode    = 'none',
+              checksumtype    = '',
+              checksumpreset  = '',
+              dynamicsource   = False,
+              chunksize       = 8388608,
+              parallelchunks  = 4,
+              inittimeout     = 600,
+              tpctimeout      = 1800,
+              rmBadCksum      = False,
+              cptimeout       = 0,
+              xrateThreashold = 0,
+              xrate           = 0,
+              retry           = 0,
+              cont            = False,
+              rtrplc          = "force" ):
     """Add a job to the copy process.
 
     :param         source: original source URL
@@ -114,11 +130,26 @@ class CopyProcess(object):
     :type     inittimeout: integer
     :param     tpctimeout: timeout for a third-party copy to finish
     :type      tpctimeout: integer
+    :param     rmBadCksum: remove target file on bad checksum
+    :type      rmBadCksum: boolean
+    :param      cptimeout: timeout for classic cp operation
+    :type       cptimeout: integer
+    :param xrateThreashold: data transfer rate threshold
+    :type  xrateThreashold: integer
+    :param           xrate: data transfer rate limit
+    :type            xrate: integer
+    :param     retry: number of retries
+    :type      retry: integer
+    :param     cont: continue copying a file from the point where the previous copy was interrupted
+    :type      cont: boolean
+    :param     rtrplc: the retry polic (force or continue)
+    :type      rtrplc: string
     """
-    self.__process.add_job(source, target, sourcelimit, force, posc, coerce, mkdir,
-                           thirdparty, checksummode, checksumtype, checksumpreset,
-                           dynamicsource, chunksize, parallelchunks, inittimeout,
-                           tpctimeout)
+    self.__process.add_job(source, target, sourcelimit, force, posc,
+                           coerce, mkdir, thirdparty, checksummode, checksumtype,
+                           checksumpreset, dynamicsource, chunksize, parallelchunks, inittimeout,
+                           tpctimeout, rmBadCksum, cptimeout, retry, xrateThreashold,
+                           xrate, cont, rtrplc )
 
   def prepare(self):
     """Prepare the copy jobs. **Must be called before** ``run()``."""
