@@ -63,10 +63,11 @@ namespace PyXRootD
     XrdCl::Env *env = XrdCl::DefaultEnv::GetEnv();
 
     static const char *kwlist[]
-      = { "source", "target", "sourcelimit", "force", "posc", "coerce",
-          "mkdir", "thirdparty", "checksummode", "checksumtype",
+      = { "source", "target", "sourcelimit", "force", "posc",
+          "coerce", "mkdir", "thirdparty", "checksummode", "checksumtype",
           "checksumpreset", "dynamicsource", "chunksize", "parallelchunks", "inittimeout",
-          "tpctimeout", "rmBadCksum", NULL };
+          "tpctimeout", "rmBadCksum", "cptimeout", "xratethreshold", "xrate",
+          "retry", "cont", "rtrplc", NULL };
 
     const char  *source;
     const char  *target;
@@ -81,6 +82,11 @@ namespace PyXRootD
     const char  *checkSumPreset    = "";
     bool         dynamicSource     = false;
     bool         rmBadCksum        = false;
+    long long    xRateThreashold   = 0;
+    long long    xRate             = 0;
+    long long    retry             = 0;
+    bool         cont              = false;
+    const char  *rtrplc            = "force";
 
 
     int val = XrdCl::DefaultCPChunkSize;
@@ -99,33 +105,45 @@ namespace PyXRootD
     env->GetInt( "CPTPCTimeout", val );
     uint16_t tpcTimeout = val;
 
+    val = XrdCl::DefaultCPTimeout;
+    env->GetInt( "CPTimeout", val );
+    uint16_t cpTimeout = val;
 
-    if ( !PyArg_ParseTupleAndKeywords( args, kwds, "ss|HbbbbssssbIHHHb:add_job",
-         (char**) kwlist, &source, &target, &sourceLimit, &force, &posc,
+    if ( !PyArg_ParseTupleAndKeywords( args, kwds, "ss|HbbbbssssbIHHHbHLLLbs:add_job",
+         (char**) kwlist,
+         &source, &target, &sourceLimit, &force, &posc,
          &coerce, &mkdir, &thirdParty, &checkSumMode, &checkSumType,
-         &checkSumPreset, &dynamicSource, &chunkSize, &parallelChunks,
-         &initTimeout, &tpcTimeout, &rmBadCksum ) )
+         &checkSumPreset, &dynamicSource, &chunkSize, &parallelChunks, &initTimeout,
+         &tpcTimeout, &rmBadCksum, &cpTimeout, &xRateThreashold, &xRate,
+         &retry, &cont, &rtrplc ) )
       return NULL;
 
     XrdCl::PropertyList properties;
     self->results->push_back(XrdCl::PropertyList());
 
-    properties.Set( "source",         source         );
-    properties.Set( "target",         target         );
-    properties.Set( "force",          force          );
-    properties.Set( "posc",           posc           );
-    properties.Set( "coerce",         coerce         );
-    properties.Set( "makeDir",        mkdir          );
-    properties.Set( "dynamicSource",  dynamicSource  );
-    properties.Set( "thirdParty",     thirdParty     );
-    properties.Set( "checkSumMode",   checkSumMode   );
-    properties.Set( "checkSumType",   checkSumType   );
-    properties.Set( "checkSumPreset", checkSumPreset );
-    properties.Set( "chunkSize",      chunkSize      );
-    properties.Set( "parallelChunks", parallelChunks );
-    properties.Set( "initTimeout",    initTimeout    );
-    properties.Set( "tpcTimeout",     tpcTimeout     );
-    properties.Set( "rmOnBadCksum",   rmBadCksum     );
+    properties.Set( "source",          source          );
+    properties.Set( "target",          target          );
+    properties.Set( "force",           force           );
+    properties.Set( "posc",            posc            );
+    properties.Set( "coerce",          coerce          );
+    properties.Set( "makeDir",         mkdir           );
+    properties.Set( "dynamicSource",   dynamicSource   );
+    properties.Set( "thirdParty",      thirdParty      );
+    properties.Set( "checkSumMode",    checkSumMode    );
+    properties.Set( "checkSumType",    checkSumType    );
+    properties.Set( "checkSumPreset",  checkSumPreset  );
+    properties.Set( "chunkSize",       chunkSize       );
+    properties.Set( "parallelChunks",  parallelChunks  );
+    properties.Set( "initTimeout",     initTimeout     );
+    properties.Set( "tpcTimeout",      tpcTimeout      );
+    properties.Set( "rmOnBadCksum",    rmBadCksum      );
+    properties.Set( "cpTimeout",       cpTimeout       );
+    properties.Set( "xrateThreashold", xRateThreashold );
+    properties.Set( "xrate",           xRate           );
+    properties.Set( "continue",        cont );
+
+    env->PutInt( "CpRetry", retry );
+    env->PutString( "CpRetryPolicy", rtrplc );
 
     if( sourceLimit > 1 )
     {
