@@ -761,4 +761,232 @@ namespace PyXRootD
     bool status = self->file->SetProperty( name, value );
     return status ? Py_True : Py_False;
   }
+
+  //----------------------------------------------------------------------------
+  //! Set Extended File Attributes
+  //----------------------------------------------------------------------------
+  PyObject* File::SetXAttr( File *self, PyObject *args, PyObject *kwds )
+  {
+    static const char  *kwlist[] = { "attrs", "timeout", "callback", NULL };
+
+    std::vector<XrdCl::xattr_t>  attrs;
+    uint16_t     timeout  = 0;
+
+    PyObject    *callback = NULL, *pystatus   = NULL;
+    PyObject    *pyattrs  = NULL, *pyresponse = NULL;
+    XrdCl::XRootDStatus status;
+
+    if ( !self->file->IsOpen() ) return FileClosedError();
+
+    if ( !PyArg_ParseTupleAndKeywords( args, kwds, "O|HO:set_xattr",
+         (char**) kwlist, &pyattrs, &timeout, &callback ) ) return NULL;
+
+    // it should be a list
+    if( !PyList_Check( pyattrs ) )
+      return NULL;
+
+    // now parse the input
+    Py_ssize_t size = PyList_Size( pyattrs );
+    attrs.reserve( size );
+    for( ssize_t i = 0; i < size; ++i )
+    {
+      // get the item at respective index
+      PyObject *item = PyList_GetItem( pyattrs, i );
+      // make sure the item is a tuple
+      if( !item || !PyTuple_Check( item ) )
+        return NULL;
+      // make sure the tuple size equals to 2
+      if( PyTuple_Size( item ) != 2 )
+        return NULL;
+      // extract the attribute name from the tuple
+      PyObject *py_name = PyTuple_GetItem( item, 0 );
+      if( !PyBytes_Check( py_name ) )
+        return NULL;
+      std::string name = PyBytes_AsString( py_name );
+      // extract the attribute value from the tuple
+      PyObject *py_value = PyTuple_GetItem( item, 1 );
+      if( !PyBytes_Check( py_value ) )
+        return NULL;
+      std::string value = PyBytes_AsString( py_value );
+      // update the C++ list of xattrs
+      attrs.push_back( XrdCl::xattr_t( name, value ) );
+    }
+
+    if ( callback && callback != Py_None ) {
+      XrdCl::ResponseHandler *handler = GetHandler<std::vector<XrdCl::XAttrStatus>>( callback );
+      if ( !handler ) return NULL;
+      async( status = self->file->SetXAttr( attrs, handler, timeout ) );
+    }
+
+    else {
+      std::vector<XrdCl::XAttrStatus>  result;
+      async( status = self->file->SetXAttr( attrs, result, timeout ) );
+      pyresponse = ConvertType( &result );
+    }
+
+    pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
+    PyObject *o = ( callback && callback != Py_None ) ?
+            Py_BuildValue( "O", pystatus ) :
+            Py_BuildValue( "OO", pystatus, pyresponse );
+    Py_DECREF( pystatus );
+    Py_XDECREF( pyresponse );
+    return o;
+  }
+
+  //----------------------------------------------------------------------------
+  //! Get Extended File Attributes
+  //----------------------------------------------------------------------------
+  PyObject* File::GetXAttr( File *self, PyObject *args, PyObject *kwds )
+  {
+    static const char  *kwlist[] = { "attrs", "timeout", "callback", NULL };
+
+    std::vector<std::string>  attrs;
+    uint16_t     timeout  = 0;
+
+    PyObject    *callback = NULL, *pystatus   = NULL;
+    PyObject    *pyattrs  = NULL, *pyresponse = NULL;
+    XrdCl::XRootDStatus status;
+
+    if ( !self->file->IsOpen() ) return FileClosedError();
+
+    if ( !PyArg_ParseTupleAndKeywords( args, kwds, "O|HO:set_xattr",
+         (char**) kwlist, &pyattrs, &timeout, &callback ) ) return NULL;
+
+    // it should be a list
+    if( !PyList_Check( pyattrs ) )
+      return NULL;
+
+    // now parse the input
+    Py_ssize_t size = PyList_Size( pyattrs );
+    attrs.reserve( size );
+    for( ssize_t i = 0; i < size; ++i )
+    {
+      // get the item at respective index
+      PyObject *item = PyList_GetItem( pyattrs, i );
+      // make sure the item is a string
+      if( !item || !PyBytes_Check( item ) )
+        return NULL;
+      std::string name = PyBytes_AsString( item );
+      // update the C++ list of xattrs
+      attrs.push_back( name );
+    }
+
+    if ( callback && callback != Py_None ) {
+      XrdCl::ResponseHandler *handler = GetHandler<std::vector<XrdCl::XAttr>>( callback );
+      if ( !handler ) return NULL;
+      async( status = self->file->GetXAttr( attrs, handler, timeout ) );
+    }
+
+    else {
+      std::vector<XrdCl::XAttr>  result;
+      async( status = self->file->GetXAttr( attrs, result, timeout ) );
+      pyresponse = ConvertType( &result );
+    }
+
+    pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
+    PyObject *o = ( callback && callback != Py_None ) ?
+            Py_BuildValue( "O", pystatus ) :
+            Py_BuildValue( "OO", pystatus, pyresponse );
+    Py_DECREF( pystatus );
+    Py_XDECREF( pyresponse );
+    return o;
+  }
+
+  //----------------------------------------------------------------------------
+  //! Delete Extended File Attributes
+  //----------------------------------------------------------------------------
+  PyObject* File::DelXAttr( File *self, PyObject *args, PyObject *kwds )
+  {
+    static const char  *kwlist[] = { "attrs", "timeout", "callback", NULL };
+
+    std::vector<std::string>  attrs;
+    uint16_t     timeout  = 0;
+
+    PyObject    *callback = NULL, *pystatus   = NULL;
+    PyObject    *pyattrs  = NULL, *pyresponse = NULL;
+    XrdCl::XRootDStatus status;
+
+    if ( !self->file->IsOpen() ) return FileClosedError();
+
+    if ( !PyArg_ParseTupleAndKeywords( args, kwds, "O|HO:set_xattr",
+         (char**) kwlist, &pyattrs, &timeout, &callback ) ) return NULL;
+
+    // it should be a list
+    if( !PyList_Check( pyattrs ) )
+      return NULL;
+
+    // now parse the input
+    Py_ssize_t size = PyList_Size( pyattrs );
+    attrs.reserve( size );
+    for( ssize_t i = 0; i < size; ++i )
+    {
+      // get the item at respective index
+      PyObject *item = PyList_GetItem( pyattrs, i );
+      // make sure the item is a string
+      if( !item || !PyBytes_Check( item ) )
+        return NULL;
+      std::string name = PyBytes_AsString( item );
+      // update the C++ list of xattrs
+      attrs.push_back( name );
+    }
+
+    if ( callback && callback != Py_None ) {
+      XrdCl::ResponseHandler *handler = GetHandler<std::vector<XrdCl::XAttrStatus>>( callback );
+      if ( !handler ) return NULL;
+      async( status = self->file->DelXAttr( attrs, handler, timeout ) );
+    }
+
+    else {
+      std::vector<XrdCl::XAttrStatus>  result;
+      async( status = self->file->DelXAttr( attrs, result, timeout ) );
+      pyresponse = ConvertType( &result );
+    }
+
+    pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
+    PyObject *o = ( callback && callback != Py_None ) ?
+            Py_BuildValue( "O", pystatus ) :
+            Py_BuildValue( "OO", pystatus, pyresponse );
+    Py_DECREF( pystatus );
+    Py_XDECREF( pyresponse );
+    return o;
+  }
+
+  //----------------------------------------------------------------------------
+  //! List Extended File Attributes
+  //----------------------------------------------------------------------------
+  PyObject* File::ListXAttr( File *self, PyObject *args, PyObject *kwds )
+  {
+    static const char  *kwlist[] = { "timeout", "callback", NULL };
+
+    uint16_t     timeout  = 0;
+
+    PyObject    *callback   = NULL, *pystatus = NULL;
+    PyObject    *pyresponse = NULL;
+    XrdCl::XRootDStatus status;
+
+    if ( !self->file->IsOpen() ) return FileClosedError();
+
+    if ( !PyArg_ParseTupleAndKeywords( args, kwds, "|HO:set_xattr",
+         (char**) kwlist, &timeout, &callback ) ) return NULL;
+
+    if ( callback && callback != Py_None ) {
+      XrdCl::ResponseHandler *handler = GetHandler<std::vector<XrdCl::XAttr>>( callback );
+      if ( !handler ) return NULL;
+      async( status = self->file->ListXAttr( handler, timeout ) );
+    }
+
+    else {
+      std::vector<XrdCl::XAttr>  result;
+      async( status = self->file->ListXAttr( result, timeout ) );
+      pyresponse = ConvertType( &result );
+    }
+
+    pystatus = ConvertType<XrdCl::XRootDStatus>( &status );
+    PyObject *o = ( callback && callback != Py_None ) ?
+            Py_BuildValue( "O", pystatus ) :
+            Py_BuildValue( "OO", pystatus, pyresponse );
+    Py_DECREF( pystatus );
+    Py_XDECREF( pyresponse );
+    return o;
+  }
 }
