@@ -35,13 +35,13 @@
 */
   
 #include <unistd.h>
-#include <ctype.h>
+#include <cctype>
 #include <fcntl.h>
 #include <pwd.h>
-#include <stdint.h>
+#include <cstdint>
 #include <string>
-#include <string.h>
-#include <stdio.h>
+#include <cstring>
+#include <cstdio>
 #include <sys/param.h>
 #include <sys/resource.h>
 #include <sys/types.h>
@@ -444,7 +444,7 @@ int XrdConfig::Configure(int argc, char **argv)
        case 'n': myInsName = (!strcmp(optarg,"anon")||!strcmp(optarg,"default")
                            ? 0 : optarg);
                  break;
-       case 'p': if ((clPort = yport(&Log, "tcp", optarg)) < 0) Usage(1);
+       case 'p': if ((clPort = XrdOuca2x::a2p(Log,"tcp",optarg)) < 0) Usage(1);
                  break;
        case 'P': if (dfltProt) free(dfltProt);
                  dfltProt = strdup(optarg);
@@ -896,7 +896,7 @@ int XrdConfig::ConfigProc()
         ||  !strcmp (var, "all.sitename" ))
            if (ConfigXeq(var+4, Config)) {Config.Echo(); NoGo = 1;}
 
-// Now check if any errors occured during file i/o
+// Now check if any errors occurred during file i/o
 //
    if ((retc = Config.LastError()))
       NoGo = Log.Emsg("Config", retc, "read config file", ConfigFN);
@@ -1586,7 +1586,7 @@ int XrdConfig::xhpath(XrdSysError *eDest, XrdOucStream &Config)
 // If the command line specified he home, it cannot be undone
 //
    if (Specs & hpSpec)
-      {eDest->Say("Config warning: command line homepath cannot be overidden.");
+      {eDest->Say("Config warning: command line homepath cannot be overridden.");
        Config.GetWord();
        return 0;
       }
@@ -1663,7 +1663,7 @@ int XrdConfig::xbuf(XrdSysError *eDest, XrdOucStream &Config)
 
              tls       parameters apply only to the tls port
              keepalive do [not] set the socket keepalive option.
-             kaparms   keepalive paramters as specfied by parms.
+             kaparms   keepalive paramters as specified by parms.
              <blen>    is the socket's send/rcv buffer size.
              <ct>      Seconds to cache address to name resolutions.
              [no]dnr   do [not] perform a reverse DNS lookup if not needed.
@@ -1865,7 +1865,7 @@ int XrdConfig::xpidf(XrdSysError *eDest, XrdOucStream &Config)
                                                [if [<hlst>] [named <nlst>]]
 
              tls        apply this to the tls port
-             <tcpnum>   number of the tcp port for incomming requests
+             <tcpnum>   number of the tcp port for incoming requests
              <hlst>     list of applicable host patterns
              <nlst>     list of applicable instance names.
 
@@ -1888,32 +1888,13 @@ int XrdConfig::xport(XrdSysError *eDest, XrdOucStream &Config)
                               ProtInfo.myInst, myProg)) <= 0)
           {if (!rc) Config.noEcho(); return (rc < 0);}
 
-    if ((pnum = yport(eDest, "tcp", cport)) < 0) return 1;
+    if ((pnum = XrdOuca2x::a2p(*eDest, "tcp", cport)) < 0) return 1;
     if (istls) PortTLS = pnum;
        else PortTCP = PortUDP = pnum;
 
     return 0;
 }
 
-/******************************************************************************/
-
-int XrdConfig::yport(XrdSysError *eDest, const char *ptype, const char *val)
-{
-    int pnum;
-    if (!strcmp("any", val)) return 0;
-
-    const char *invp = (*ptype == 't' ? "tcp port" : "udp port" );
-    const char *invs = (*ptype == 't' ? "Unable to find tcp service" :
-                                        "Unable to find udp service" );
-
-    if (isdigit(*val))
-       {if (XrdOuca2x::a2i(*eDest,invp,val,&pnum,1,65535)) return 0;}
-       else if (!(pnum = XrdNetUtils::ServPort(val, (*ptype != 't'))))
-               {eDest->Emsg("Config", invs, val);
-                return -1;
-               }
-    return pnum;
-}
   
 /******************************************************************************/
 /*                                 x p r o t                                  */
@@ -1951,7 +1932,7 @@ int XrdConfig::xprot(XrdSysError *eDest, XrdOucStream &Config)
     strcpy(proname, val);
 
     if ((val = index(proname, ':')))
-       {if ((portnum = yport(&Log, "tcp", val+1)) < 0) return 1;
+       {if ((portnum = XrdOuca2x::a2p(*eDest, "tcp", val+1)) < 0) return 1;
            else *val = '\0';
        }
 
