@@ -30,8 +30,8 @@
 #include <limits.h>
 #include <poll.h>
 #include <signal.h>
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/uio.h>
@@ -72,6 +72,22 @@
 /*                               G l o b a l s                                */
 /******************************************************************************/
 
+namespace
+{
+int getIovMax()
+{
+int maxiov;
+#ifdef _SC_IOV_MAX
+    if ((maxiov = sysconf(_SC_IOV_MAX)) > 0) return maxiov;
+#endif
+#ifdef IOV_MAX
+    return IOV_MAX;
+#else
+    return 1024;
+#endif
+}
+};
+
 namespace XrdGlobal
 {
 extern XrdSysError    Log;
@@ -79,6 +95,7 @@ extern XrdScheduler   Sched;
 extern XrdTlsContext *tlsCtx;
        XrdTcpMonPin  *TcpMonPin = 0;
 extern int            devNull;
+       int            maxIOV = getIovMax();
 };
 
 using namespace XrdGlobal;
@@ -188,7 +205,7 @@ int XrdLinkXeq::Close(bool defer)
 //
    if (defer)
       {if (!sendQ) Shutdown(false);
-          else {TRACEI(DEBUG, "Shutdown FD only via SendQ");
+          else {TRACEI(DEBUG, "Shutdown FD " <<LinkInfo.FD<<" only via SendQ");
                 LinkInfo.InUse++;
                 LinkInfo.FD = -LinkInfo.FD; // Leave poll version untouched!
                 wrMutex.Lock();
@@ -199,7 +216,7 @@ int XrdLinkXeq::Close(bool defer)
        return 0;
       }
 
-// If we got here then this is not a defered close so we just need to check
+// If we got here then this is not a deferred close so we just need to check
 // if there is a sendq appendage we need to get rid of.
 //
    if (sendQ)
@@ -214,7 +231,8 @@ int XrdLinkXeq::Close(bool defer)
 //
    while(LinkInfo.InUse > 1)
       {opHelper.UnLock();
-       TRACEI(DEBUG, "Close defered, use count=" <<LinkInfo.InUse);
+       TRACEI(DEBUG, "Close FD "<<LinkInfo.FD <<" deferred, use count="
+                     <<LinkInfo.InUse);
        Serialize();
        opHelper.Lock(&LinkInfo.opMutex);
       }
@@ -441,6 +459,64 @@ int XrdLinkXeq::Recv(char *Buff, int Blen, int timeout)
    return int(totlen);
 }
 
+/******************************************************************************/
+  
+int XrdLinkXeq::Recv(const struct iovec *iov, int iocnt, int timeout)
+{
+   XrdSysMutexHelper theMutex;
+   struct pollfd polltab = {PollInfo.FD, POLLIN|POLLRDNORM, 0};
+   int retc, rlen;
+
+// Lock the read mutex if we need to, the helper will unlock it upon exit
+//
+   if (LockReads) theMutex.Lock(&rdMutex);
+
+// Wait up to timeout milliseconds for data to arrive
+//
+   isIdle = 0;
+   do {retc = poll(&polltab,1,timeout);} while(retc < 0 && errno == EINTR);
+   if (retc != 1)
+      {if (retc == 0)
+          {tardyCnt++;
+           return 0;
+          }
+       return (LinkInfo.FD >= 0 ? Log.Emsg("Link",-errno,"poll",ID) : -1);
+      }
+
+// Verify it is safe to read now
+//
+   if (!(polltab.revents & (POLLIN|POLLRDNORM)))
+      {Log.Emsg("Link", XrdPoll::Poll2Text(polltab.revents), "polling", ID);
+       return -1;
+      }
+
+// If the iocnt is within limits then just go ahead and read once.
+//
+   if (iocnt <= maxIOV)
+      {rlen = RecvIOV(iov, iocnt);
+       if (rlen > 0) {AtomicAdd(BytesIn, rlen);}
+       return rlen;
+      }
+
+// We will have to break this up into allowable segments and we need to add up
+// the bytes in each segment so that we know when to stop reading.
+//
+   int seglen, segcnt = maxIOV, totlen = 0;
+   do {seglen = 0;
+       for (int i = 0; i < segcnt; i++) seglen += iov[i].iov_len;
+       if ((rlen = RecvIOV(iov, segcnt)) < 0) return rlen;
+       totlen += rlen;
+       if (rlen < seglen) break;
+       iov   += segcnt;
+       iocnt -= segcnt;
+       if (iocnt <= maxIOV) segcnt = iocnt;
+      } while(iocnt > 0);
+
+// All done
+//
+   AtomicAdd(BytesIn, totlen);
+   return totlen;
+}
 
 /******************************************************************************/
 /*                               R e c v A l l                                */
@@ -480,9 +556,31 @@ int XrdLinkXeq::RecvAll(char *Buff, int Blen, int timeout)
    if (int(rlen) == Blen) return Blen;
         if (!rlen) {TRACEI(DEBUG, "No RecvAll() data; errno=" <<errno);}
    else if (rlen > 0) Log.Emsg("RecvAll", "Premature end from", ID);
-   else if (LinkInfo.FD >= 0) Log.Emsg("Link", errno, "recieve from", ID);
+   else if (LinkInfo.FD >= 0) Log.Emsg("Link", errno, "receive from", ID);
    return -1;
 }
+  
+/******************************************************************************/
+/* Protected:                    R e c v I O V                                */
+/******************************************************************************/
+  
+int XrdLinkXeq::RecvIOV(const struct iovec *iov, int iocnt)
+{
+   ssize_t retc = 0;
+
+// Read the data in. On some version of Unix (e.g., Linux) a readv() may
+// end at any time without reading all the bytes when directed to a socket.
+// We always return the number bytes read (or an error). The caller needs to
+// restart the read at the appropriate place in the iovec when more data arrives.
+//
+   do {retc = readv(LinkInfo.FD, iov, iocnt);}
+      while(retc < 0 && errno == EINTR);
+
+// Check how we completed
+//
+   if (retc < 0) Log.Emsg("Link", errno, "receive from", ID);
+   return retc;
+}
 
 /******************************************************************************/
 /*                              R e g i s t e r                               */
@@ -1146,7 +1244,7 @@ int XrdLinkXeq::TLS_Recv(char *Buff, int Blen)
 
 /******************************************************************************/
 
-int XrdLinkXeq::TLS_Recv(char *Buff, int Blen, int timeout)
+int XrdLinkXeq::TLS_Recv(char *Buff, int Blen, int timeout, bool havelock)
 {
    XrdSysMutexHelper theMutex;
    XrdTls::RC retc;
@@ -1154,7 +1252,7 @@ int XrdLinkXeq::TLS_Recv(char *Buff, int Blen, int timeout)
 
 // Lock the read mutex if we need to, the helper will unlock it upon exit
 //
-   if (LockReads) theMutex.Lock(&rdMutex);
+   if (LockReads && !havelock) theMutex.Lock(&rdMutex);
 
 // Wait up to timeout milliseconds for data to arrive
 //
@@ -1191,6 +1289,34 @@ int XrdLinkXeq::TLS_Recv(char *Buff, int Blen, int timeout)
    return totlen;
 }
 
+/******************************************************************************/
+
+int XrdLinkXeq::TLS_Recv(const struct iovec *iov, int iocnt, int timeout)
+{
+   XrdSysMutexHelper theMutex;
+   char *Buff;
+   int Blen, rlen, totlen = 0;
+
+// Lock the read mutex if we need to, the helper will unlock it upon exit
+//
+   if (LockReads) theMutex.Lock(&rdMutex);
+
+// Individually process each element until we can't read any more
+//
+   isIdle = 0;
+   for (int i = 0; i < iocnt; i++)
+       {Buff = (char *)iov[i].iov_base;
+        Blen =         iov[i].iov_len;
+        rlen = TLS_Recv(Buff, Blen, timeout, true);
+        if (rlen <= 0) break;
+        totlen += rlen;
+        if (rlen < Blen) break;
+       }
+
+   if (totlen) {AtomicAdd(BytesIn, totlen);}
+   return totlen;
+}
+
 /******************************************************************************/
 /*                           T L S _ R e c v A l l                            */
 /******************************************************************************/
@@ -1210,7 +1336,7 @@ int XrdLinkXeq::TLS_RecvAll(char *Buff, int Blen, int timeout)
 
 // Note that we will block until we receive all the bytes.
 //
-   return Recv(Buff, Blen, -1);
+   return TLS_Recv(Buff, Blen, -1);
 }
 
 /******************************************************************************/
