@@ -27,10 +27,10 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
 #include <signal.h>
-#include <stdio.h>
+#include <cstdio>
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -41,6 +41,7 @@
 
 #include "Xrd/XrdJob.hh"
 #include "Xrd/XrdScheduler.hh"
+#include "XrdOuc/XrdOucTrace.hh"    // For ABI compatibility only!
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysLogger.hh"
 
@@ -93,12 +94,70 @@ void *XrdStartWorking(void *carg)
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
   
+XrdScheduler::XrdScheduler(XrdSysError *eP, XrdSysTrace *tP,
+                           int minw, int maxw, int maxi)
+              : XrdJob("underused thread monitor"),
+                 XrdTraceOld(0), WorkAvail(0, "sched work")
+{
+   Boot(eP, tP, minw, maxw, maxi);
+}
+ 
+/******************************************************************************/
+
+  
 XrdScheduler::XrdScheduler(XrdSysError *eP, XrdOucTrace *tP,
                            int minw, int maxw, int maxi)
               : XrdJob("underused thread monitor"),
-                WorkAvail(0, "sched work")
+                XrdTraceOld(tP), WorkAvail(0, "sched work")
 {
 
+// Invoke the main initialization function with a new style trace object
+//
+   Boot(eP, new XrdSysTrace("Xrd", eP->logger()), minw, maxw, maxi);
+}
+
+/******************************************************************************/
+
+// This constructor creates a self contained scheduler.
+//
+XrdScheduler::XrdScheduler(int minw, int maxw, int maxi)
+              : XrdJob("underused thread monitor"),
+                WorkAvail(0, "sched work")
+{
+   XrdSysLogger *Logger;
+   int eFD;
+
+// Get a file descriptor mirroring standard error
+//
+#if ( defined(__linux__) || defined(__GNU__) ) && defined(F_DUPFD_CLOEXEC)
+   eFD = fcntl(STDERR_FILENO, F_DUPFD_CLOEXEC, 0);
+#else
+   eFD = dup(STDERR_FILENO);
+   fcntl(eFD, F_SETFD, FD_CLOEXEC);
+#endif
+
+// Now we need to get a logger object. We make this a real dumb one.
+//
+   Logger = new XrdSysLogger(eFD, 0);
+   XrdLog = new XrdSysError(Logger);
+
+// Now get a trace object
+//
+   XrdTrace = new XrdSysTrace("Xrd", Logger);
+   if (getenv("XRDDEBUG") != 0) XrdTrace->What = TRACE_SCHED;
+
+// Set remaining values. We do no use maximum possible threads here.
+//
+   Init(minw, maxw, maxi);
+}
+
+/******************************************************************************/
+/* Private:                         B o o t                                   */
+/******************************************************************************/
+
+void XrdScheduler::Boot(XrdSysError *eP, XrdSysTrace *tP,
+                        int minw, int maxw, int maxi)
+{
 // Perform common initialization
 //
    XrdLog      =  eP;
@@ -107,7 +166,7 @@ XrdScheduler::XrdScheduler(XrdSysError *eP, XrdOucTrace *tP,
 
 // Make sure we are using the maximum number of threads allowed (Linux only)
 //
-#if defined(__linux__) && defined(RLIMIT_NPROC)
+#if ( defined(__linux__) || defined(__GNU__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__)) ) && defined(RLIMIT_NPROC)
 
    struct rlimit rlim;
 
@@ -151,41 +210,6 @@ XrdScheduler::XrdScheduler(XrdSysError *eP, XrdOucTrace *tP,
 #endif
 
 }
- 
-/******************************************************************************/
-
-// This constructor creates a self contained scheduler.
-//
-XrdScheduler::XrdScheduler(int minw, int maxw, int maxi)
-              : XrdJob("underused thread monitor"),
-                WorkAvail(0, "sched work")
-{
-   XrdSysLogger *Logger;
-   int eFD;
-
-// Get a file descriptor mirroring standard error
-//
-#if defined(__linux__) && defined(O_CLOEXEC)
-   eFD = fcntl(STDERR_FILENO, F_DUPFD_CLOEXEC, 0);
-#else
-   eFD = dup(STDERR_FILENO);
-   fcntl(eFD, F_SETFD, FD_CLOEXEC);
-#endif
-
-// Now we need to get a logger object. We make this a real dumb one.
-//
-   Logger = new XrdSysLogger(eFD, 0);
-   XrdLog = new XrdSysError(Logger);
-
-// Now get a trace object
-//
-   XrdTrace = new XrdOucTrace(XrdLog);
-   if (getenv("XRDDEBUG") != 0) XrdTrace->What = TRACE_SCHED;
-
-// Set remaining values. We do no use maximum possible threads here.
-//
-   Init(minw, maxw, maxi);
-}
 
 /******************************************************************************/
 /*                            D e s t r u c t o r                             */
@@ -536,8 +560,13 @@ void XrdScheduler::setParms(int minw, int maxw, int avlw, int maxi, int once)
   
 void XrdScheduler::Start() // Serialized one time call!
 {
-    int retc, numw;
-    pthread_t tid;
+   int retc, numw;
+   pthread_t tid;
+
+// Provide ABI compatibility for XrdOucTrace which is deprecated!
+//
+   if (getenv("XRDDEBUG") != 0) XrdTrace->What = TRACE_SCHED;
+      else if (XrdTraceOld) XrdTrace->What |= XrdTraceOld->What;
 
 // Start a time based scheduler
 //
