@@ -28,9 +28,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <ctype.h>
-#include <stdio.h>
-#include <time.h>
+#include <cctype>
+#include <cstdio>
+#include <ctime>
 #include <sys/param.h>
 
 #include "XrdVersion.hh"
@@ -68,7 +68,7 @@ XrdAccAuthorize *XrdAccDefaultAuthorizeObject(XrdSysLogger *lp,
    static XrdVERSIONINFODEF(myVer, XrdAcc, XrdVNUMBER, XrdVERSION);
    static XrdSysError Eroute(lp, "acc_");
 
-// Verify version compatability
+// Verify version compatibility
 //
    if (urVer.vNum != myVer.vNum && !XrdSysPlugin::VerCmp(urVer,myVer))
       return 0;
@@ -353,7 +353,7 @@ void XrdAccAccess::SwapTabs(struct XrdAccAccess_Tables &newtab)
 // Determine if we need to resolve the hostname at all.
 //
    if (!hRefX)
-      {if (Atab.D_List || Atab.H_Hash || Atab.N_Hash) hRefY = true;
+      {if (newtab.D_List || newtab.H_Hash || newtab.N_Hash) hRefY = true;
           else {XrdAccAccess_ID *ylP = newtab.SYList;
                 while (ylP)
                       {if (ylP->host) {hRefY = true; break;}
