@@ -30,9 +30,9 @@
   
 #include <fcntl.h>
 #include <getopt.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <sys/stat.h>
 #include <sys/types.h>
 
@@ -608,6 +608,17 @@ int XrdCpConfig::a2z(const char *item, long long *val,
   
 int XrdCpConfig::defCks(const char *opval)
 {
+  if( CksVal )
+  {
+    std::string cksum( opval );
+    size_t pos = cksum.find( ':' );
+    std::string mode = cksum.substr( pos + 1 );
+    if( mode != "source" )
+      FMSG("Additional checksum must be of mode 'source'.", 13);
+    AddCksVal.push_back( cksum.substr( 0, pos ) );
+    return 1;
+  }
+
    static XrdVERSIONINFODEF(myVer, xrdcp, XrdVNUMBER, XrdVERSION);
    const char *Colon = index(opval, ':');
    char  csName[XrdCksData::NameSize];
@@ -980,7 +991,7 @@ void XrdCpConfig::Usage(int rc)
    "-V | --version                prints the version number\n"
    "-X | --xrate <rate>           limits the transfer to the specified rate. You can\n"
    "                              suffix the value with 'k', 'm', or 'g'\n"
-   "     --xrate-threshold <rate> If the transfer rate drops bellow given threshold force\n"
+   "     --xrate-threshold <rate> If the transfer rate drops below given threshold force\n"
    "                              the client to use different source or if no more sources\n"
    "                              are available fail the transfer. You can suffix the value\n"
    "                              with 'k', 'm', or 'g'\n"
