@@ -28,9 +28,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <string.h>
+#include <cstring>
 #include <strings.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -85,7 +85,7 @@ XrdCpFile::XrdCpFile(const char *FSpec, int &badURL)
        return;
       }
 
-// Dtermine protocol of the incomming spec
+// Dtermine protocol of the incoming spec
 //
    for (i = 0; i < pTnum; i++)
        {if (!strncmp(FSpec, pTab[i].pHdr, pTab[i].pHsz))
