@@ -36,27 +36,20 @@ namespace XrdCl
                                           Poller           *poller,
                                           TransportHandler *transport,
                                           AnyObject        *channelData,
-                                          uint16_t          subStreamNum ):
+                                          uint16_t          subStreamNum,
+                                          Stream           *strm ):
     pPoller( poller ),
     pTransport( transport ),
     pChannelData( channelData ),
     pSubStreamNum( subStreamNum ),
-    pStream( 0 ),
-    pSocket( 0 ),
-    pIncoming( 0 ),
-    pHSIncoming( 0 ),
-    pOutgoing( 0 ),
-    pSignature( 0 ),
-    pHSOutgoing( 0 ),
-    pHandShakeData( 0 ),
+    pStream( strm ),
+    pStreamName( ToStreamName( url, subStreamNum ) ),
+    pSocket( new Socket() ),
     pHandShakeDone( false ),
     pConnectionStarted( 0 ),
     pConnectionTimeout( 0 ),
-    pHeaderDone( false ),
-    pOutMsgDone( false ),
-    pOutHandler( 0 ),
-    pIncMsgSize( 0 ),
-    pOutMsgSize( 0 ),
+    pHSWaitStarted( 0 ),
+    pHSWaitSeconds( 0 ),
     pUrl( url ),
     pTlsHandShakeOngoing( false )
   {
@@ -66,9 +59,7 @@ namespace XrdCl
     env->GetInt( "TimeoutResolution", timeoutResolution );
     pTimeoutResolution = timeoutResolution;
 
-    pSocket = new Socket();
     pSocket->SetChannelID( pChannelData );
-    pIncHandler = std::make_pair( (IncomingMsgHandler*)0, false );
     pLastActivity = time(0);
   }
 
@@ -79,7 +70,6 @@ namespace XrdCl
   {
     Close();
     delete pSocket;
-    delete pSignature;
   }
 
   //----------------------------------------------------------------------------
@@ -119,7 +109,7 @@ namespace XrdCl
         log->Error( AsyncSockMsg, "[%s] Unable to turn on keepalive: %s",
                     st.ToString().c_str() );
 
-#if defined(__linux__) && defined( TCP_KEEPIDLE ) && \
+#if ( defined(__linux__) || defined(__GNU__) ) && defined( TCP_KEEPIDLE ) && \
     defined( TCP_KEEPINTVL ) && defined( TCP_KEEPCNT )
 
       param = DefaultTCPKeepAliveTime;
@@ -200,24 +190,15 @@ namespace XrdCl
 
     pPoller->RemoveSocket( pSocket );
     pSocket->Close();
-
-    if( !pIncHandler.second )
-      delete pIncoming;
-
-    pIncoming = 0;
     return XRootDStatus();
   }
 
-  //----------------------------------------------------------------------------
-  // Set a stream object to be notified about the status of the operations
-  //----------------------------------------------------------------------------
-  void AsyncSocketHandler::SetStream( Stream *stream )
+  std::string AsyncSocketHandler::ToStreamName( const URL &url, uint16_t strmnb )
   {
-    pStream    = stream;
     std::ostringstream o;
-    o << pStream->GetURL()->GetHostId();
-    o << "." << pSubStreamNum;
-    pStreamName = o.str();
+    o << url.GetHostId();
+    o << "." << strmnb;
+    return o.str();
   }
 
   //----------------------------------------------------------------------------
@@ -251,6 +232,9 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     else if( type & ReadTimeOut )
     {
+      if( pHSWaitSeconds )
+        CheckHSWait();
+
       if( likely( pHandShakeDone ) )
         OnReadTimeout();
       else
@@ -265,12 +249,18 @@ namespace XrdCl
       pLastActivity = time(0);
       if( unlikely( pSocket->GetStatus() == Socket::Connecting ) )
         OnConnectionReturn();
-      else if( unlikely( pTlsHandShakeOngoing ) )
-        OnTLSHandShake();
-      else if( likely( pHandShakeDone ) )
-        OnWrite();
-      else
-        OnWriteWhileHandshaking();
+      //------------------------------------------------------------------------
+      // Make sure we are not writing anything if we have been told to wait.
+      //------------------------------------------------------------------------
+      else if( pHSWaitSeconds == 0 )
+      {
+        if( unlikely( pTlsHandShakeOngoing ) )
+          OnTLSHandShake();
+        else if( likely( pHandShakeDone ) )
+          OnWrite();
+        else
+          OnWriteWhileHandshaking();
+      }
     }
 
     //--------------------------------------------------------------------------
@@ -342,15 +332,13 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Initialize the handshake
     //--------------------------------------------------------------------------
-    pHandShakeData = new HandShakeData( pStream->GetURL(),
-                                        pSubStreamNum );
-    pHandShakeData->serverAddr = &pSocket->GetServerAddress();
+    pHandShakeData.reset( new HandShakeData( pStream->GetURL(),
+                                        pSubStreamNum ) );
+    pHandShakeData->serverAddr = pSocket->GetServerAddress();
     pHandShakeData->clientName = pSocket->GetSockName();
     pHandShakeData->streamName = pStreamName;
 
-    st = pTransport->HandShake( pHandShakeData, *pChannelData );
-    ++pHandShakeData->step;
-
+    st = pTransport->HandShake( pHandShakeData.get(), *pChannelData );
     if( !st.IsOK() )
     {
       log->Error( AsyncSockMsg, "[%s] Connection negotiation failed",
@@ -359,13 +347,22 @@ namespace XrdCl
       return;
     }
 
+    if( st.code != suRetry )
+      ++pHandShakeData->step;
+
+    //--------------------------------------------------------------------------
+    // Initialize the hand-shake reader and writer
+    //--------------------------------------------------------------------------
+    hswriter.reset( new AsyncHSWriter( *pSocket, pStreamName ) );
+    hsreader.reset( new AsyncHSReader( *pTransport, *pSocket, pStreamName, *pStream, pSubStreamNum ) );
+
     //--------------------------------------------------------------------------
     // Transport has given us something to send
     //--------------------------------------------------------------------------
     if( pHandShakeData->out )
     {
-      pHSOutgoing = pHandShakeData->out;
-      pHandShakeData->out = 0;
+      hswriter->Reset( pHandShakeData->out );
+      pHandShakeData->out = nullptr;
     }
 
     //--------------------------------------------------------------------------
@@ -373,6 +370,8 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( !pPoller->EnableReadNotification( pSocket, true, pTimeoutResolution ) )
     {
+      hswriter.reset();
+      hsreader.reset();
       pStream->OnConnectError( pSubStreamNum,
                                XRootDStatus( stFatal, errPollerError ) );
       return;
@@ -384,61 +383,31 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void AsyncSocketHandler::OnWrite()
   {
-    //--------------------------------------------------------------------------
-    // Pick up a message if we're not in process of writing something
-    //--------------------------------------------------------------------------
-    if( !pOutgoing )
+    if( !reqwriter )
     {
-      pOutMsgDone = false;
-      std::pair<Message *, OutgoingMsgHandler *> toBeSent;
-      toBeSent = pStream->OnReadyToWrite( pSubStreamNum );
-      pOutgoing = toBeSent.first; pOutHandler = toBeSent.second;
-
-      if( !pOutgoing )
-        return;
-
-      pOutgoing->SetCursor( 0 );
-      pOutMsgSize = pOutgoing->GetSize();
-
-      //------------------------------------------------------------------------
-      // Secure the message if necessary
-      //------------------------------------------------------------------------
-      delete pSignature; pSignature = 0;
-      XRootDStatus st = pTransport->GetSignature( pOutgoing, pSignature, *pChannelData );
-      if( !st.IsOK() )
-      {
-        OnFault( st );
-        return;
-      }
-
-      if( pSignature )
-        pOutMsgSize += pSignature->GetSize();
+      OnFault( XRootDStatus( stError, errInternal, 0, "Request writer is null." ) );
+      return;
     }
-
     //--------------------------------------------------------------------------
-    // Write everything at once: signature, request and raw data
+    // Let's do the writing ...
     //--------------------------------------------------------------------------
-    XRootDStatus st = WriteMessageAndRaw( pOutgoing, pSignature );
+    XRootDStatus st = reqwriter->Write();
     if( !st.IsOK() )
     {
+      //------------------------------------------------------------------------
+      // We failed
+      //------------------------------------------------------------------------
       OnFault( st );
       return;
     }
-
-    if( st.code == suRetry )
-      return;
-
-    Log *log = DefaultEnv::GetLog();
-    log->Dump( AsyncSockMsg, "[%s] Successfully sent message: %s (0x%x).",
-               pStreamName.c_str(), pOutgoing->GetDescription().c_str(),
-               pOutgoing );
-
-    pStream->OnMessageSent( pSubStreamNum, pOutgoing, pOutMsgSize );
-    pOutgoing = 0;
-
+    //--------------------------------------------------------------------------
+    // We are not done yet
+    //--------------------------------------------------------------------------
+    if( st.code == suRetry) return;
     //--------------------------------------------------------------------------
     // Disable the respective substream if empty
     //--------------------------------------------------------------------------
+    reqwriter->Reset();
     pStream->DisableIfEmpty( pSubStreamNum );
   }
 
@@ -448,207 +417,83 @@ namespace XrdCl
   void AsyncSocketHandler::OnWriteWhileHandshaking()
   {
     XRootDStatus st;
-    if( !pHSOutgoing )
+    if( !hswriter || !hswriter->HasMsg() )
     {
       if( !(st = DisableUplink()).IsOK() )
         OnFaultWhileHandshaking( st );
       return;
     }
-
-    if( !(st = WriteCurrentMessage( pHSOutgoing )).IsOK() )
+    //--------------------------------------------------------------------------
+    // Let's do the writing ...
+    //--------------------------------------------------------------------------
+    st = hswriter->Write();
+    if( !st.IsOK() )
     {
+      //------------------------------------------------------------------------
+      // We failed
+      //------------------------------------------------------------------------
       OnFaultWhileHandshaking( st );
       return;
     }
-
+    //--------------------------------------------------------------------------
+    // We are not done yet
+    //--------------------------------------------------------------------------
     if( st.code == suRetry ) return;
-
-    delete pHSOutgoing;
-    pHSOutgoing = 0;
-
-    st = pSocket->Flash();
-    if( !st.IsOK() )
-    {
-      Log *log = DefaultEnv::GetLog();
-      log->Error( AsyncSockMsg, "[%s] Unable to flash the socket: %s",
-                  pStreamName.c_str(), XrdSysE2T( st.errNo ) );
-      OnFaultWhileHandshaking( st );
-    }
-
+    //--------------------------------------------------------------------------
+    // Disable the uplink
+    // Note: at this point we don't deallocate the HS message as we might need
+    //       to re-send it in case of a kXR_wait response
+    //--------------------------------------------------------------------------
     if( !(st = DisableUplink()).IsOK() )
       OnFaultWhileHandshaking( st );
   }
 
-  //----------------------------------------------------------------------------
-  // Write the current message
-  //----------------------------------------------------------------------------
-  XRootDStatus AsyncSocketHandler::WriteCurrentMessage( Message *toWrite )
-  {
-    Log *log = DefaultEnv::GetLog();
-
-    //--------------------------------------------------------------------------
-    // Try to write down the current message
-    //--------------------------------------------------------------------------
-    Message  *msg             = toWrite;
-    size_t    leftToBeWritten = msg->GetSize()-msg->GetCursor();
-
-    while( leftToBeWritten )
-    {
-      int bytesWritten = 0;
-      XRootDStatus st = pSocket->Send( msg->GetBufferAtCursor(), leftToBeWritten, bytesWritten );
-
-      if( !st.IsOK() )
-      {
-        toWrite->SetCursor( 0 );
-        return st;
-      }
-
-      if( st.code == suRetry ) return st;
-
-      msg->AdvanceCursor( bytesWritten );
-      leftToBeWritten -= bytesWritten;
-    }
-
-    //--------------------------------------------------------------------------
-    // We have written the message successfully
-    //--------------------------------------------------------------------------
-    log->Dump( AsyncSockMsg, "[%s] Wrote a message: %s (0x%x), %d bytes",
-               pStreamName.c_str(), toWrite->GetDescription().c_str(),
-               toWrite, toWrite->GetSize() );
-    return XRootDStatus();
-  }
-
-  //----------------------------------------------------------------------------
-  // Write the message and its signature
-  //----------------------------------------------------------------------------
-  XRootDStatus AsyncSocketHandler::WriteMessageAndRaw( Message *toWrite, Message *&sign )
-  {
-    XRootDStatus st;
-
-    if( sign )
-    {
-      st = WriteCurrentMessage( sign );
-      if( !st.IsOK() || st.code == suRetry )
-        return st;
-    }
-
-    st = WriteCurrentMessage( toWrite );
-    if( !st.IsOK() || st.code == suRetry )
-      return st;
-
-    if( pOutHandler->IsRaw() )
-    {
-      uint32_t bytesWritten = 0;
-      st = pOutHandler->WriteMessageBody( pSocket, bytesWritten );
-      pOutMsgSize += bytesWritten;
-      if( !st.IsOK() || st.code == suRetry )
-        return st;
-    }
-
-    st = pSocket->Flash();
-    if( !st.IsOK() )
-    {
-      Log *log = DefaultEnv::GetLog();
-      log->Error( AsyncSockMsg, "[%s] Unable to flash the socket: %s",
-                  pStreamName.c_str(), XrdSysE2T( st.errNo ) );
-    }
-
-    return st;
-  }
-
   //----------------------------------------------------------------------------
   // Got a read readiness event
   //----------------------------------------------------------------------------
   void AsyncSocketHandler::OnRead()
   {
     //--------------------------------------------------------------------------
-    // There is no incoming message currently being processed so we create
-    // a new one
+    // Make sure the response reader object exists
     //--------------------------------------------------------------------------
-    if( !pIncoming )
+    if( !rspreader )
     {
-      pHeaderDone  = false;
-      pIncoming    = new Message();
-      pIncHandler  = std::make_pair( (IncomingMsgHandler*)0, false );
-      pIncMsgSize  = 0;
-    }
-
-    XRootDStatus  st;
-    Log    *log = DefaultEnv::GetLog();
-
-    //--------------------------------------------------------------------------
-    // We need to read the header first
-    //--------------------------------------------------------------------------
-    if( !pHeaderDone )
-    {
-      st = pTransport->GetHeader( pIncoming, pSocket );
-      if( !st.IsOK() )
-      {
-        OnFault( st );
-        return;
-      }
-
-      if( st.code == suRetry )
-        return;
-
-      log->Dump( AsyncSockMsg, "[%s] Received message header for 0x%x size: %d",
-                pStreamName.c_str(), pIncoming, pIncoming->GetCursor() );
-      pIncMsgSize = pIncoming->GetCursor();
-      pHeaderDone = true;
-      std::pair<IncomingMsgHandler *, bool> raw;
-      pIncHandler = pStream->InstallIncHandler( pIncoming, pSubStreamNum );
-
-      if( pIncHandler.first )
-      {
-        log->Dump( AsyncSockMsg, "[%s] Will use the raw handler to read body "
-                   "of message 0x%x", pStreamName.c_str(), pIncoming );
-      }
+      OnFault( XRootDStatus( stError, errInternal, 0, "Response reader is null." ) );
+      return;
     }
 
     //--------------------------------------------------------------------------
-    // We need to call a raw message handler to get the data from the socket
+    // Readout the data from the socket
     //--------------------------------------------------------------------------
-    if( pIncHandler.first )
+    XRootDStatus st = rspreader->Read();
+
+    //--------------------------------------------------------------------------
+    // Handler header corruption
+    //--------------------------------------------------------------------------
+    if( !st.IsOK() && st.code == errCorruptedHeader )
     {
-      uint32_t bytesRead = 0;
-      st = pIncHandler.first->ReadMessageBody( pIncoming, pSocket,
-                                               bytesRead );
-      if( !st.IsOK() )
-      {
-        OnFault( st );
-        return;
-      }
-      pIncMsgSize += bytesRead;
-
-      if( st.code == suRetry )
-        return;
-    }
-    //--------------------------------------------------------------------------
-    // No raw handler, so we read the message to the buffer
-    //--------------------------------------------------------------------------
-    else
-    {
-      st = pTransport->GetBody( pIncoming, pSocket );
-      if( !st.IsOK() )
-      {
-        OnFault( st );
-        return;
-      }
-
-      if( st.code == suRetry )
-        return;
-
-      pIncMsgSize = pIncoming->GetSize();
+      OnHeaderCorruption();
+      return;
     }
 
     //--------------------------------------------------------------------------
-    // Report the incoming message
+    // Handler other errors
     //--------------------------------------------------------------------------
-    log->Dump( AsyncSockMsg, "[%s] Received message 0x%x of %d bytes",
-               pStreamName.c_str(), pIncoming, pIncMsgSize );
+    if( !st.IsOK() )
+    {
+      OnFault( st );
+      return;
+    }
 
-    pStream->OnIncoming( pSubStreamNum, pIncoming, pIncMsgSize );
-    pIncoming = 0;
+    //--------------------------------------------------------------------------
+    // We are not done yet
+    //--------------------------------------------------------------------------
+    if( st.code == suRetry ) return;
+
+    //--------------------------------------------------------------------------
+    // We are done, reset the response reader so we can read out next message
+    //--------------------------------------------------------------------------
+    rspreader->Reset();
   }
 
   //----------------------------------------------------------------------------
@@ -656,11 +501,20 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void AsyncSocketHandler::OnReadWhileHandshaking()
   {
+    //--------------------------------------------------------------------------
+    // Make sure the response reader object exists
+    //--------------------------------------------------------------------------
+    if( !hsreader )
+    {
+      OnFault( XRootDStatus( stError, errInternal, 0, "Hand-shake reader is null." ) );
+      return;
+    }
+
     //--------------------------------------------------------------------------
     // Read the message and let the transport handler look at it when
     // reading has finished
     //--------------------------------------------------------------------------
-    XRootDStatus st = ReadMessage( pHSIncoming );
+    XRootDStatus st = hsreader->Read();
     if( !st.IsOK() )
     {
       OnFaultWhileHandshaking( st );
@@ -670,20 +524,19 @@ namespace XrdCl
     if( st.code != suDone )
       return;
 
-    HandleHandShake();
+    HandleHandShake( hsreader->ReleaseMsg() );
   }
 
   //------------------------------------------------------------------------
   // Handle the handshake message
   //------------------------------------------------------------------------
-  void AsyncSocketHandler::HandleHandShake()
+  void AsyncSocketHandler::HandleHandShake( std::unique_ptr<Message> msg )
   {
     //--------------------------------------------------------------------------
     // OK, we have a new message, let's deal with it;
     //--------------------------------------------------------------------------
-    pHandShakeData->in = pHSIncoming;
-    pHSIncoming = 0;
-    XRootDStatus st = pTransport->HandShake( pHandShakeData, *pChannelData );
+    pHandShakeData->in = msg.release();
+    XRootDStatus st = pTransport->HandShake( pHandShakeData.get(), *pChannelData );
 
     //--------------------------------------------------------------------------
     // Deal with wait responses
@@ -699,38 +552,54 @@ namespace XrdCl
       return;
     }
 
-    //--------------------------------------------------------------------------
-    // We are handling a wait response and the transport handler told
-    // as to retry the request
-    //--------------------------------------------------------------------------
-    if( st.code == suRetry && waitSeconds >= 0 )
+    if( st.code == suRetry )
     {
-      time_t resendTime = ::time( 0 ) + waitSeconds;
-      if( resendTime > pConnectionStarted + pConnectionTimeout )
+      //------------------------------------------------------------------------
+      // We are handling a wait response and the transport handler told
+      // as to retry the request
+      //------------------------------------------------------------------------
+      if( waitSeconds >=0 )
       {
-        Log *log = DefaultEnv::GetLog();
-        log->Error( AsyncSockMsg,
-                    "[%s] Wont retry kXR_endsess request because would"
-                    "reach connection timeout.",
-                    pStreamName.c_str() );
+        time_t resendTime = ::time( 0 ) + waitSeconds;
+        if( resendTime > pConnectionStarted + pConnectionTimeout )
+        {
+          Log *log = DefaultEnv::GetLog();
+          log->Error( AsyncSockMsg,
+                      "[%s] Won't retry kXR_endsess request because would"
+                      "reach connection timeout.",
+                      pStreamName.c_str() );
 
-        OnFaultWhileHandshaking( XRootDStatus( stError, errSocketTimeout ) );
+          OnFaultWhileHandshaking( XRootDStatus( stError, errSocketTimeout ) );
+        }
+        else
+        {
+          //--------------------------------------------------------------------
+          // We need to wait before replaying the request
+          //--------------------------------------------------------------------
+          Log *log = DefaultEnv::GetLog();
+          log->Debug( AsyncSockMsg, "[%s] Received a wait response to endsess request, "
+                      "will wait for %d seconds before replaying the endsess request",
+                      waitSeconds );
+          pHSWaitStarted = time( 0 );
+          pHSWaitSeconds = waitSeconds;
+        }
+        return;
       }
-      else
+      //------------------------------------------------------------------------
+      // We are re-sending a protocol request
+      //------------------------------------------------------------------------
+      else if( pHandShakeData->out )
       {
-        TaskManager *taskMgr = DefaultEnv::GetPostMaster()->GetTaskManager();
-        WaitTask *task = new WaitTask( this, pHandShakeData->out );
-        pHandShakeData->out = 0;
-        taskMgr->RegisterTask( task, resendTime );
+        SendHSMsg();
+        return;
       }
-      return;
     }
 
     //--------------------------------------------------------------------------
-    // If now is the time enable encryption
+    // If now is the time to enable encryption
     //--------------------------------------------------------------------------
     if( !pSocket->IsEncrypted() &&
-         pTransport->NeedEncryption( pHandShakeData, *pChannelData ) )
+         pTransport->NeedEncryption( pHandShakeData.get(), *pChannelData ) )
     {
       XRootDStatus st = DoTlsHandShake();
       if( !st.IsOK() || st.code == suRetry ) return;
@@ -757,7 +626,14 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( done )
     {
-      delete pHandShakeData;
+      pHandShakeData.reset();
+      hswriter.reset();
+      hsreader.reset();
+      //------------------------------------------------------------------------
+      // Initialize the request writer & reader
+      //------------------------------------------------------------------------
+      reqwriter.reset( new AsyncMsgWriter( *pTransport, *pSocket, pStreamName, *pStream, pSubStreamNum, *pChannelData ) );
+      rspreader.reset( new AsyncMsgReader( *pTransport, *pSocket, pStreamName, *pStream, pSubStreamNum ) );
       XRootDStatus st;
       if( !(st = EnableUplink()).IsOK() )
       {
@@ -772,53 +648,10 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     else if( pHandShakeData->out )
     {
-      pHSOutgoing = pHandShakeData->out;
-      pHandShakeData->out = 0;
-      XRootDStatus st;
-      if( !(st = EnableUplink()).IsOK() )
-      {
-        OnFaultWhileHandshaking( st );
-        return;
-      }
+      SendHSMsg();
     }
   }
 
-  //----------------------------------------------------------------------------
-  // Read a message
-  //----------------------------------------------------------------------------
-  XRootDStatus AsyncSocketHandler::ReadMessage( Message *&toRead )
-  {
-    if( !toRead )
-    {
-      pHeaderDone = false;
-      toRead      = new Message();
-    }
-
-    XRootDStatus  st;
-    Log    *log = DefaultEnv::GetLog();
-    if( !pHeaderDone )
-    {
-      st = pTransport->GetHeader( toRead, pSocket );
-      if( st.IsOK() && st.code == suDone )
-      {
-        log->Dump( AsyncSockMsg,
-                  "[%s] Received message header, size: %d",
-                  pStreamName.c_str(), toRead->GetCursor() );
-        pHeaderDone = true;
-      }
-      else
-        return st;
-    }
-
-    st = pTransport->GetBody( toRead, pSocket );
-    if( st.IsOK() && st.code == suDone )
-    {
-      log->Dump( AsyncSockMsg, "[%s] Received a message of %d bytes",
-                 pStreamName.c_str(), toRead->GetSize() );
-    }
-    return st;
-  }
-
   //----------------------------------------------------------------------------
   // Handle fault
   //----------------------------------------------------------------------------
@@ -828,12 +661,8 @@ namespace XrdCl
     log->Error( AsyncSockMsg, "[%s] Socket error encountered: %s",
                 pStreamName.c_str(), st.ToString().c_str() );
 
-    if( !pIncHandler.second )
-      delete pIncoming;
-
-    pIncoming   = 0;
-    pOutgoing   = 0;
-    pOutHandler = 0;
+    rspreader.reset();
+    reqwriter.reset();
 
     pStream->OnError( pSubStreamNum, st );
   }
@@ -846,10 +675,8 @@ namespace XrdCl
     Log *log = DefaultEnv::GetLog();
     log->Error( AsyncSockMsg, "[%s] Socket error while handshaking: %s",
                 pStreamName.c_str(), st.ToString().c_str() );
-    delete pHSIncoming;
-    delete pHSOutgoing;
-    pHSIncoming = 0;
-    pHSOutgoing = 0;
+    hsreader.reset();
+    hswriter.reset();
 
     pStream->OnConnectError( pSubStreamNum, st );
   }
@@ -872,16 +699,8 @@ namespace XrdCl
 
     if( isBroken )
     {
-      // if we are here it means Stream::OnError has been
-      // called from inside of Stream::OnReadTimeout, this
-      // in turn means that the ownership of following
-      // pointers, has been transfered to the inQueue
-      if( !pIncHandler.second )
-        delete pIncoming;
-
-      pIncoming   = 0;
-      pOutgoing   = 0;
-      pOutHandler = 0;
+      rspreader.reset();
+      reqwriter.reset();
     }
   }
 
@@ -895,9 +714,24 @@ namespace XrdCl
       OnFaultWhileHandshaking( XRootDStatus( stError, errSocketTimeout ) );
   }
 
-  //------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
+  // Handle header corruption in case of kXR_status response
+  //----------------------------------------------------------------------------
+  void AsyncSocketHandler::OnHeaderCorruption()
+  {
+    //--------------------------------------------------------------------------
+    // We need to force a socket error so this is handled in a similar way as
+    // a stream t/o and all requests are retried
+    //--------------------------------------------------------------------------
+    pStream->ForceError( XRootDStatus( stError, errSocketError ) );
+
+    rspreader.reset();
+    reqwriter.reset();
+  }
+
+  //----------------------------------------------------------------------------
   // Carry out the TLS hand-shake
-  //------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   XRootDStatus AsyncSocketHandler::DoTlsHandShake()
   {
     Log *log = DefaultEnv::GetLog();
@@ -923,21 +757,45 @@ namespace XrdCl
     return st;
   }
 
-  //------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   // Handle read/write event if we are in the middle of a TLS hand-shake
-  //------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   inline void AsyncSocketHandler::OnTLSHandShake()
   {
     XRootDStatus st = DoTlsHandShake();
     if( !st.IsOK() || st.code == suRetry ) return;
 
-    HandShakeNextStep( pTransport->HandShakeDone( pHandShakeData,
+    HandShakeNextStep( pTransport->HandShakeDone( pHandShakeData.get(),
                                                   *pChannelData ) );
   }
 
-  void AsyncSocketHandler::RetryHSMsg( Message *msg )
+  //----------------------------------------------------------------------------
+  // Prepare a HS writer for sending and enable uplink
+  //----------------------------------------------------------------------------
+  void AsyncSocketHandler::SendHSMsg()
   {
-    pHSOutgoing = msg;
+    if( !hswriter )
+    {
+      OnFaultWhileHandshaking( XRootDStatus( stError, errInternal, 0,
+                                             "HS writer object missing!" ) );
+      return;
+    }
+    //--------------------------------------------------------------------------
+    // We only set a new HS message if this is not a replay due to kXR_wait
+    //--------------------------------------------------------------------------
+    if( !pHSWaitSeconds )
+    {
+      hswriter->Reset( pHandShakeData->out );
+      pHandShakeData->out = nullptr;
+    }
+    //--------------------------------------------------------------------------
+    // otherwise we replay the kXR_endsess request
+    //--------------------------------------------------------------------------
+    else
+      hswriter->Replay();
+    //--------------------------------------------------------------------------
+    // Enable writing so we can replay the HS message
+    //--------------------------------------------------------------------------
     XRootDStatus st;
     if( !(st = EnableUplink()).IsOK() )
     {
@@ -956,4 +814,24 @@ namespace XrdCl
       waitSeconds = rsp->body.wait.seconds;
     return waitSeconds;
   }
+
+  //----------------------------------------------------------------------------
+  // Check if HS wait time elapsed
+  //----------------------------------------------------------------------------
+  void AsyncSocketHandler::CheckHSWait()
+  {
+    time_t now = time( 0 );
+    if( now - pHSWaitStarted >= pHSWaitSeconds )
+    {
+      Log *log = DefaultEnv::GetLog();
+      log->Debug( AsyncSockMsg, "[%s] The hand-shake wait time elapsed, will "
+                  "replay the endsess request.", pStreamName.c_str() );
+      SendHSMsg();
+      //------------------------------------------------------------------------
+      // Make sure the wait state is reset
+      //------------------------------------------------------------------------
+      pHSWaitSeconds = 0;
+      pHSWaitStarted = 0;
+    }
+  }
 }
