@@ -34,7 +34,7 @@
 #include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysPthread.hh"
 
-#include <stdio.h>
+#include <cstdio>
 #include <iostream>
 #include <iomanip>
 #include <limits>
@@ -49,7 +49,8 @@ class ProgressDisplay: public XrdCl::CopyProgressHandler
     //! Constructor
     //--------------------------------------------------------------------------
     ProgressDisplay(): pPrevious(0), pPrintProgressBar(true),
-      pPrintSourceCheckSum(false), pPrintTargetCheckSum(false)
+      pPrintSourceCheckSum(false), pPrintTargetCheckSum(false),
+      pPrintAdditionalCheckSum(false)
     {}
 
     //--------------------------------------------------------------------------
@@ -128,6 +129,14 @@ class ProgressDisplay: public XrdCl::CopyProgressHandler
         PrintCheckSum( d.target, checkSum, size );
       }
 
+      if( pPrintAdditionalCheckSum )
+      {
+        std::vector<std::string> addcksums;
+        results->Get( "additionalCkeckSum", addcksums );
+        for( auto &cks : addcksums )
+          PrintCheckSum( d.source, cks, size );
+      }
+
       pOngoingJobs.erase(it);
     }
 
@@ -270,6 +279,7 @@ class ProgressDisplay: public XrdCl::CopyProgressHandler
     void PrintProgressBar( bool print )    { pPrintProgressBar    = print; }
     void PrintSourceCheckSum( bool print ) { pPrintSourceCheckSum = print; }
     void PrintTargetCheckSum( bool print ) { pPrintTargetCheckSum = print; }
+    void PrintAdditionalCheckSum( bool print ) { pPrintAdditionalCheckSum = print; }
 
   private:
     struct JobData
@@ -287,6 +297,7 @@ class ProgressDisplay: public XrdCl::CopyProgressHandler
     bool                        pPrintProgressBar;
     bool                        pPrintSourceCheckSum;
     bool                        pPrintTargetCheckSum;
+    bool                        pPrintAdditionalCheckSum;
     std::map<uint16_t, JobData> pOngoingJobs;
     XrdSysRecMutex              pMutex;
 };
@@ -427,8 +438,8 @@ XrdCpFile *IndexRemote( XrdCl::FileSystem *fs,
     current = new XrdCpFile( path.c_str(), badUrl );
     if( badUrl )
     {
-      delete current;
       log->Error( AppMsg, "Bad URL: %s", current->Path );
+      delete current;
       return 0;
     }
 
@@ -494,13 +505,15 @@ int main( int argc, char **argv )
   bool         rmOnBadCksum  = false;
   bool         continue_     = false;
   bool         recurse       = false;
+  bool         zipappend     = false;
   std::string thirdParty = "none";
 
-  if( config.Want( XrdCpConfig::DoPosc ) )     posc       = true;
-  if( config.Want( XrdCpConfig::DoForce ) )    force      = true;
-  if( config.Want( XrdCpConfig::DoCoerce ) )   coerce     = true;
-  if( config.Want( XrdCpConfig::DoTpc ) )      thirdParty = "first";
-  if( config.Want( XrdCpConfig::DoTpcOnly ) )  thirdParty = "only";
+  if( config.Want( XrdCpConfig::DoPosc ) )      posc       = true;
+  if( config.Want( XrdCpConfig::DoForce ) )     force      = true;
+  if( config.Want( XrdCpConfig::DoCoerce ) )    coerce     = true;
+  if( config.Want( XrdCpConfig::DoTpc ) )       thirdParty = "first";
+  if( config.Want( XrdCpConfig::DoTpcOnly ) )   thirdParty = "only";
+  if( config.Want( XrdCpConfig::DoZipAppend ) ) zipappend  = true;
   if( config.Want( XrdCpConfig::DoTpcDlgt ) )
   {
     // the env var is being set already here (we are issuing a stat
@@ -572,6 +585,9 @@ int main( int argc, char **argv )
     }
   }
 
+  if( !config.AddCksVal.empty() )
+    progress.PrintAdditionalCheckSum( true );
+
   //----------------------------------------------------------------------------
   // ZIP archive
   //----------------------------------------------------------------------------
@@ -601,6 +617,12 @@ int main( int argc, char **argv )
   if( config.nStrm != 0 )
     env->PutInt( "SubStreamsPerChannel", config.nStrm + 1 /*stands for the control stream*/ );
 
+  if( config.Retry != -1 )
+  {
+    env->PutInt( "CpRetry", config.Retry );
+    env->PutString( "CpRetryPolicy", config.RetryPolicy );
+  }
+
   if( config.Want( XrdCpConfig::DoNoTlsOK ) )
     env->PutInt( "NoTlsOK", 1 );
 
@@ -703,11 +725,12 @@ int main( int argc, char **argv )
     delete statInfo;
   }
 
-  if( !targetIsDir && targetExists && !force && !recurse )
+  if( !targetIsDir && targetExists && !force && !recurse && !zipappend )
   {
     XRootDStatus st( stError, errInvalidOp, EEXIST );
     // Unable to create /tmp/test.txt; file exists
     log->Error( AppMsg, "%s (destination)", st.ToString().c_str() );
+    std::cerr << "Run: " << st.ToStr() << std::endl;
     return st.GetShellCode();
   }
 
@@ -727,6 +750,7 @@ int main( int argc, char **argv )
   // If we're doing remote recursive copy, chain all the files (if it's a
   // directory)
   //----------------------------------------------------------------------------
+  bool remoteSrcIsDir = false;
   if( config.Want( XrdCpConfig::DoRecurse ) &&
       config.srcFile->Protocol == XrdCpFile::isXroot )
   {
@@ -737,6 +761,7 @@ int main( int argc, char **argv )
     XRootDStatus st = fs->Stat( source.GetPath(), statInfo );
     if( st.IsOK() && statInfo->TestFlags( StatInfo::IsDir ) )
     {
+      remoteSrcIsDir = true;
       //------------------------------------------------------------------------
       // Recursively index the remote directory
       //------------------------------------------------------------------------
@@ -798,8 +823,14 @@ int main( int argc, char **argv )
     //--------------------------------------------------------------------------
     std::string target = dest;
 
-    // if the source was a file Dlen and Doff overlap
-    bool srcIsDir = std::string( sourceFile->Path ).size() == size_t( sourceFile->Doff + sourceFile->Dlen );
+
+    bool srcIsDir = false;
+    // if this is local file, for a directory Dlen + Doff will overlap with path size
+    if( strncmp( sourceFile->ProtName, "file", 4 ) == 0 )
+      srcIsDir = std::string( sourceFile->Path ).size() == size_t( sourceFile->Doff + sourceFile->Dlen );
+    // otherwise we are handling a remote file
+    else
+      srcIsDir = remoteSrcIsDir;
     // if this is a recursive copy make sure we preserve the directory structure
     if( config.Want( XrdCpConfig::DoRecurse ) && srcIsDir )
     {
@@ -818,34 +849,37 @@ int main( int argc, char **argv )
     }
     AppendCGI( target, config.dstOpq );
 
-    properties.Set( "source",         source         );
-    properties.Set( "target",         target         );
-    properties.Set( "force",          force          );
-    properties.Set( "posc",           posc           );
-    properties.Set( "coerce",         coerce         );
-    properties.Set( "makeDir",        makedir        );
-    properties.Set( "dynamicSource",  dynSrc         );
-    properties.Set( "thirdParty",     thirdParty     );
-    properties.Set( "checkSumMode",   checkSumMode   );
-    properties.Set( "checkSumType",   checkSumType   );
-    properties.Set( "checkSumPreset", checkSumPreset );
-    properties.Set( "chunkSize",      chunkSize      );
-    properties.Set( "parallelChunks", parallelChunks );
-    properties.Set( "zipArchive",     zip            );
-    properties.Set( "xcp",            xcp            );
-    properties.Set( "xcpBlockSize",   blockSize      );
-    properties.Set( "delegate",       delegate       );
-    properties.Set( "targetIsDir",    targetIsDir    );
-    properties.Set( "preserveXAttr",  preserveXAttr  );
-    properties.Set( "xrate",          config.xRate   );
-    properties.Set( "rmOnBadCksum",   rmOnBadCksum   );
-    properties.Set( "continue",       continue_      );
+    properties.Set( "source",          source                 );
+    properties.Set( "target",          target                 );
+    properties.Set( "force",           force                  );
+    properties.Set( "posc",            posc                   );
+    properties.Set( "coerce",          coerce                 );
+    properties.Set( "makeDir",         makedir                );
+    properties.Set( "dynamicSource",   dynSrc                 );
+    properties.Set( "thirdParty",      thirdParty             );
+    properties.Set( "checkSumMode",    checkSumMode           );
+    properties.Set( "checkSumType",    checkSumType           );
+    properties.Set( "checkSumPreset",  checkSumPreset         );
+    properties.Set( "chunkSize",       chunkSize              );
+    properties.Set( "parallelChunks",  parallelChunks         );
+    properties.Set( "zipArchive",      zip                    );
+    properties.Set( "xcp",             xcp                    );
+    properties.Set( "xcpBlockSize",    blockSize              );
+    properties.Set( "delegate",        delegate               );
+    properties.Set( "targetIsDir",     targetIsDir            );
+    properties.Set( "preserveXAttr",   preserveXAttr          );
+    properties.Set( "xrate",           config.xRate           );
+    properties.Set( "xrateThreashold", config.xRateThreashold );
+    properties.Set( "rmOnBadCksum",    rmOnBadCksum           );
+    properties.Set( "continue",        continue_              );
+    properties.Set( "zipAppend",       zipappend              );
+    properties.Set( "addcksums",       config.AddCksVal );
 
     if( zip )
-      properties.Set( "zipSource",    zipFile        );
+      properties.Set( "zipSource",     zipFile                );
 
     if( xcp )
-      properties.Set( "nbXcpSources", nbSources      );
+      properties.Set( "nbXcpSources",  nbSources              );
 
 
     XRootDStatus st = process.AddJob( properties, results );
