@@ -33,13 +33,13 @@
 #include "XrdCl/XrdClCopyJob.hh"
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClJobManager.hh"
-#include "XrdCl/XrdClUglyHacks.hh"
 #include "XrdCl/XrdClRedirectorRegistry.hh"
+#include "XrdCl/XrdClConstants.hh"
+#include "XrdSys/XrdSysPthread.hh"
 
 #include <sys/time.h>
 
 #include <memory>
-#include <iostream>
 
 namespace
 {
@@ -50,9 +50,17 @@ namespace
                      XrdCl::CopyProgressHandler *progress,
                      uint16_t                    currentJob,
                      uint16_t                    totalJobs,
-                     XrdCl::Semaphore           *sem = 0 ):
+                     XrdSysSemaphore            *sem = 0 ):
         pJob(job), pProgress(progress), pCurrentJob(currentJob),
-        pTotalJobs(totalJobs), pSem(sem) {}
+        pTotalJobs(totalJobs), pSem(sem),
+        pWrtRetryCnt( XrdCl::DefaultRetryWrtAtLBLimit ),
+        pRetryCnt( XrdCl::DefaultCpRetry ),
+        pRetryPolicy( XrdCl::DefaultCpRetryPolicy )
+      {
+        XrdCl::DefaultEnv::GetEnv()->GetInt( "RetryWrtAtLBLimit", pWrtRetryCnt );
+        XrdCl::DefaultEnv::GetEnv()->GetInt( "CpRetry", pRetryCnt );
+        XrdCl::DefaultEnv::GetEnv()->GetString( "CpRetryPolicy", pRetryPolicy );
+      }
 
       //------------------------------------------------------------------------
       //! Run the job
@@ -83,7 +91,74 @@ namespace
         //----------------------------------------------------------------------
         // Do the copy
         //----------------------------------------------------------------------
-        XrdCl::XRootDStatus st = pJob->Run( pProgress );
+        XrdCl::XRootDStatus st;
+        while( true )
+        {
+          st = pJob->Run( pProgress );
+          //--------------------------------------------------------------------
+          // Retry due to write-recovery
+          //--------------------------------------------------------------------
+          if( !st.IsOK() && st.code == XrdCl::errRetry && pWrtRetryCnt > 0 )
+          {
+            std::string url;
+            pJob->GetResults()->Get( "LastURL", url );
+            XrdCl::URL lastURL( url );
+            XrdCl::URL::ParamsMap cgi = lastURL.GetParams();
+            auto itr = cgi.find( "tried" );
+            if( itr != cgi.end() )
+            {
+              std::string tried = itr->second;
+              if( tried[tried.size() - 1] != ',' ) tried += ',';
+              tried += lastURL.GetHostName();
+              cgi["tried"] = tried;
+            }
+            else
+              cgi["tried"] = lastURL.GetHostName();
+
+            std::string recoveryRedir;
+            pJob->GetResults()->Get( "WrtRecoveryRedir", recoveryRedir );
+            XrdCl::URL recRedirURL( recoveryRedir );
+
+            std::string target;
+            pJob->GetProperties()->Get( "target", target );
+            XrdCl::URL trgURL( target );
+            trgURL.SetHostName( recRedirURL.GetHostName() );
+            trgURL.SetPort( recRedirURL.GetPort() );
+            trgURL.SetProtocol( recRedirURL.GetProtocol() );
+            trgURL.SetParams( cgi );
+            pJob->GetProperties()->Set( "target", trgURL.GetURL() );
+            pJob->Init();
+
+            // we have a new job, let's try again
+            --pWrtRetryCnt;
+            continue;
+          }
+          //--------------------------------------------------------------------
+          // Copy job retry
+          //--------------------------------------------------------------------
+          if( !st.IsOK() && pRetryCnt > 0 &&
+              ( XrdCl::Status::IsSocketError( st.code ) ||
+                st.code == XrdCl::errOperationExpired   ||
+                st.code == XrdCl::errThresholdExceeded ) )
+          {
+            if( pRetryPolicy == "continue" )
+            {
+              pJob->GetProperties()->Set( "force", false );
+              pJob->GetProperties()->Set( "continue", true );
+            }
+            else
+            {
+              pJob->GetProperties()->Set( "force", true );
+              pJob->GetProperties()->Set( "continue", false );
+            }
+            --pRetryCnt;
+            continue;
+          }
+
+          // we only loop in case of retry error
+          break;
+        }
+
         pJob->GetResults()->Set( "status", st );
 
         //----------------------------------------------------------------------
@@ -115,7 +190,10 @@ namespace
       XrdCl::CopyProgressHandler *pProgress;
       uint16_t                    pCurrentJob;
       uint16_t                    pTotalJobs;
-      XrdCl::Semaphore           *pSem;
+      XrdSysSemaphore            *pSem;
+      int                         pWrtRetryCnt;
+      int                         pRetryCnt;
+      std::string                 pRetryPolicy;
   };
 };
 
@@ -185,7 +263,7 @@ namespace XrdCl
     PropertyList &p = pImpl->pJobProperties.back();
 
     const char *bools[] = {"target", "force", "posc", "coerce", "makeDir", "zipArchive",
-                           "xcp", "preserveXAttr", "rmOnBadCksum", "continue", 0};
+                           "xcp", "preserveXAttr", "rmOnBadCksum", "continue", "zipAppend", 0};
     for( int i = 0; bools[i]; ++i )
       if( !p.HasProperty( bools[i] ) )
         p.Set( bools[i], false );
@@ -251,12 +329,26 @@ namespace XrdCl
       p.Set( "tpcTimeout", val );
     }
 
+    if( !p.HasProperty( "cpTimeout" ) )
+    {
+      int val = DefaultCPTimeout;
+      env->GetInt( "CPTimeout", val );
+      p.Set( "cpTimeout", val );
+    }
+
     if( !p.HasProperty( "dynamicSource" ) )
       p.Set( "dynamicSource", false );
 
     if( !p.HasProperty( "xrate" ) )
       p.Set( "xrate", 0 );
 
+    if( !p.HasProperty( "xrateThreashold" ) || p.Get<long long>( "xrateThreashold" ) == 0 )
+    {
+      int val = DefaultXRateThreshold;
+      env->GetInt( "XRateThreshold", val );
+      p.Set( "xrateThreashold", val );
+    }
+
     //--------------------------------------------------------------------------
     // Insert the properties
     //--------------------------------------------------------------------------
@@ -463,7 +555,7 @@ namespace XrdCl
         return XRootDStatus( stError, errOSError, 0,
                              "Unable to start job manager" );
 
-      Semaphore *sem = new Semaphore(0);
+      XrdSysSemaphore *sem = new XrdSysSemaphore(0);
       std::vector<QueuedCopyJob*> queued;
       for( it = pImpl->pJobs.begin(); it != pImpl->pJobs.end(); ++it )
       {
