@@ -32,6 +32,8 @@
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClCopyProcess.hh"
 #include "XrdCl/XrdClFile.hh"
+#include "XrdCl/XrdClFileSystemOperations.hh"
+#include "XrdCl/XrdClParallelOperation.hh"
 #include "XrdSys/XrdSysE2T.hh"
 
 #include <cstdlib>
@@ -210,9 +212,59 @@ XRootDStatus DoCD( FileSystem                      *fs,
 //------------------------------------------------------------------------------
 // Helper function to calculate number of digits in a number
 //------------------------------------------------------------------------------
-int nbDigits( uint64_t nb )
+uint32_t nbDigits( uint64_t nb )
 {
-  return int( log10( double(nb) ) + 1);
+  if( nb == 0 ) return 1;
+  return uint32_t( log10( double(nb) ) + 1);
+}
+
+
+void PrintDirListStatInfo( StatInfo *info, bool hascks = false, uint32_t ownerwidth = 0, uint32_t groupwidth = 0, uint32_t sizewidth = 0 )
+{
+  if( info->ExtendedFormat() )
+  {
+    if( info->TestFlags( StatInfo::IsDir ) )
+      std::cout << "d";
+    else
+      std::cout << "-";
+    std::cout << info->GetModeAsOctString();
+
+    std::cout << " " << std::setw( ownerwidth ) << info->GetOwner();
+    std::cout << " " << std::setw( groupwidth ) << info->GetGroup();
+    std::cout << " " << std::setw( sizewidth ) << info->GetSize();
+    if( hascks && info->HasChecksum() )
+      std::cout << " " << std::setw( sizewidth ) << info->GetChecksum();
+    std::cout << " " << info->GetModTimeAsString() << " ";
+  }
+  else
+  {
+    if( info->TestFlags( StatInfo::IsDir ) )
+      std::cout << "d";
+    else
+      std::cout << "-";
+
+    if( info->TestFlags( StatInfo::IsReadable ) )
+      std::cout << "r";
+    else
+      std::cout << "-";
+
+    if( info->TestFlags( StatInfo::IsWritable ) )
+      std::cout << "w";
+    else
+      std::cout << "-";
+
+    if( info->TestFlags( StatInfo::XBitSet ) )
+      std::cout << "x";
+    else
+      std::cout << "-";
+
+    std::cout << " " << info->GetModTimeAsString();
+
+    uint64_t size = info->GetSize();
+    int width = nbDigits( size ) + 2;
+    if( width < 12 ) width = 12;
+    std::cout << std::setw( width ) << info->GetSize() << " ";
+  }
 }
 
 //------------------------------------------------------------------------------
@@ -229,6 +281,7 @@ XRootDStatus DoLS( FileSystem                      *fs,
   uint32_t    argc     = args.size();
   bool        stats    = false;
   bool        showUrls = false;
+  bool        hascks   = false;
   std::string path;
   DirListFlags::Flags flags = DirListFlags::Locate | DirListFlags::Merge;
 
@@ -261,6 +314,13 @@ XRootDStatus DoLS( FileSystem                      *fs,
       // check if file is a ZIP archive if yes list content
       flags |= DirListFlags::Zip;
     }
+    else if( args[i] == "-C" )
+    {
+      // query checksum for each entry in the directory
+      hascks = true;
+      stats  = true;
+      flags |= DirListFlags::Cksm;
+    }
     else
       path = args[i];
   }
@@ -282,13 +342,44 @@ XRootDStatus DoLS( FileSystem                      *fs,
     }
   }
 
-  log->Debug( AppMsg, "Attempting to list: %s", newPath.c_str() );
+  //----------------------------------------------------------------------------
+  // Stat the entry so we know if it is a file or a directory
+  //----------------------------------------------------------------------------
+  log->Debug( AppMsg, "Attempting to stat: %s", newPath.c_str() );
+
+  StatInfo *info = 0;
+  XRootDStatus st = fs->Stat( newPath, info );
+  std::unique_ptr<StatInfo> ptr( info );
+  if( !st.IsOK() )
+  {
+    log->Error( AppMsg, "Unable to stat the path: %s", st.ToStr().c_str() );
+    return st;
+  }
+
+  if( !info->TestFlags( StatInfo::IsDir ) &&
+      !( flags & DirListFlags::Zip ) )
+  {
+    if( stats )
+      PrintDirListStatInfo( info );
+
+    if( showUrls )
+    {
+      std::string url;
+      fs->GetProperty( "LastURL", url );
+      std::cout << url;
+    }
+    std::cout << newPath << std::endl;
+    return XRootDStatus();
+  }
+
 
   //----------------------------------------------------------------------------
   // Ask for the list
   //----------------------------------------------------------------------------
+  log->Debug( AppMsg, "Attempting to list: %s", newPath.c_str() );
+
   DirectoryList *list;
-  XRootDStatus st = fs->DirList( newPath, flags, list );
+  st = fs->DirList( newPath, flags, list );
   if( !st.IsOK() )
   {
     log->Error( AppMsg, "Unable to list the path: %s", st.ToStr().c_str() );
@@ -301,69 +392,33 @@ XRootDStatus DoLS( FileSystem                      *fs,
     std::cerr << "incomplete." << std::endl;
   }
 
+  uint32_t ownerwidth = 0, groupwidth = 0, sizewidth = 0, ckswidth = 0;
+  DirectoryList::Iterator it;
+  for( it = list->Begin(); it != list->End() && stats; ++it )
+  {
+    StatInfo *info = (*it)->GetStatInfo();
+    if( ownerwidth < info->GetOwner().size() )
+      ownerwidth = info->GetOwner().size();
+    if( groupwidth < info->GetGroup().size() )
+      groupwidth = info->GetGroup().size();
+    if( sizewidth < nbDigits( info->GetSize() ) )
+      sizewidth = nbDigits( info->GetSize() );
+    if( ckswidth < info->GetChecksum().size() )
+      ckswidth = info->GetChecksum().size();
+  }
+
   //----------------------------------------------------------------------------
   // Print the results
   //----------------------------------------------------------------------------
-  DirectoryList::Iterator it;
   for( it = list->Begin(); it != list->End(); ++it )
   {
     if( stats )
     {
       StatInfo *info = (*it)->GetStatInfo();
       if( !info )
-      {
         std::cout << "---- 0000-00-00 00:00:00            ? ";
-      }
       else
-      {
-        if( info->ExtendedFormat() )
-        {
-          if( info->TestFlags( StatInfo::IsDir ) )
-            std::cout << "d";
-          else
-            std::cout << "-";
-          std::cout << info->GetModeAsOctString();
-
-          std::cout << " " << info->GetOwner();
-          std::cout << " " << info->GetGroup();
-
-          uint64_t size = info->GetSize();
-          int width = nbDigits( size ) + 2;
-          if( width < 12 ) width = 12;
-
-          std::cout << std::setw( width ) << info->GetSize();
-          std::cout << " " << info->GetModTimeAsString() << " ";
-        }
-        else
-        {
-          if( info->TestFlags( StatInfo::IsDir ) )
-            std::cout << "d";
-          else
-            std::cout << "-";
-
-          if( info->TestFlags( StatInfo::IsReadable ) )
-            std::cout << "r";
-          else
-            std::cout << "-";
-
-          if( info->TestFlags( StatInfo::IsWritable ) )
-            std::cout << "w";
-          else
-            std::cout << "-";
-
-          if( info->TestFlags( StatInfo::XBitSet ) )
-            std::cout << "x";
-          else
-            std::cout << "-";
-
-          std::cout << " " << info->GetModTimeAsString();
-
-          uint64_t size = info->GetSize();
-          int width = nbDigits( size ) + 2;
-          if( width < 12 ) width = 12;
-          std::cout << std::setw( width ) << info->GetSize() << " ";
-        }
-      }
+        PrintDirListStatInfo( info, hascks, ownerwidth, groupwidth, sizewidth );
     }
     if( showUrls )
       std::cout << "root://" << (*it)->GetHostAddress() << "/";
@@ -538,30 +593,57 @@ XRootDStatus DoRm( FileSystem                      *fs,
   Log         *log     = DefaultEnv::GetLog();
   uint32_t     argc    = args.size();
 
-  if( argc != 2 )
+  if( argc < 2 )
   {
     log->Error( AppMsg, "Wrong number of arguments." );
     return XRootDStatus( stError, errInvalidArgs );
   }
 
-  std::string fullPath;
-  if( !BuildPath( fullPath, env, args[1] ).IsOK() )
+  struct print_t
   {
-    log->Error( AppMsg, "Invalid path." );
-    return XRootDStatus( stError, errInvalidArgs );
+    void print( const std::string &msg )
+    {
+      std::unique_lock<std::mutex> lck( mtx );
+      std::cout << msg << '\n';
+    }
+    std::mutex mtx;
+  };
+  std::shared_ptr<print_t> print;
+  if( argc - 1 > 1 )
+    print = std::make_shared<print_t>();
+
+  std::vector<Pipeline> rms;
+  rms.reserve( argc - 1 );
+  for( size_t i = 1; i < argc; ++i )
+  {
+    std::string fullPath;
+    if( !BuildPath( fullPath, env, args[i] ).IsOK() )
+    {
+      log->Error( AppMsg, "Invalid path: %s", fullPath.c_str() );
+      return XRootDStatus( stError, errInvalidArgs );
+    }
+    rms.emplace_back( Rm( fs, fullPath ) >>
+                      [log, fullPath, print]( XRootDStatus &st )
+                      {
+                        if( !st.IsOK() )
+                        {
+                          log->Error( AppMsg, "Unable remove %s: %s",
+                                              fullPath.c_str(),
+                                              st.ToStr().c_str() );
+                        }
+                        if( print )
+                        {
+                          print->print( "rm " + fullPath + " : " + st.ToString() );
+                        }
+                      } );
   }
 
   //----------------------------------------------------------------------------
   // Run the query
   //----------------------------------------------------------------------------
-  XRootDStatus st = fs->Rm( fullPath );
+  XRootDStatus st = WaitFor( Parallel( rms ) );
   if( !st.IsOK() )
-  {
-    log->Error( AppMsg, "Unable remove %s: %s",
-                        fullPath.c_str(),
-                        st.ToStr().c_str() );
     return st;
-  }
 
   return XRootDStatus();
 }
@@ -1316,7 +1398,7 @@ XRootDStatus DoCat( FileSystem                      *fs,
   Log         *log     = DefaultEnv::GetLog();
   uint32_t     argc    = args.size();
 
-  if( argc != 2 && argc != 4 )
+  if( argc < 2 )
   {
     log->Error( AppMsg, "Wrong number of arguments." );
     return XRootDStatus( stError, errInvalidArgs );
@@ -1330,7 +1412,7 @@ XRootDStatus DoCat( FileSystem                      *fs,
     return XRootDStatus( stError, errInvalidAddr );
   }
 
-  std::string remote;
+  std::vector<std::string> remotes;
   std::string local;
 
   for( uint32_t i = 1; i < args.size(); ++i )
@@ -1349,44 +1431,59 @@ XRootDStatus DoCat( FileSystem                      *fs,
       }
     }
     else
-      remote = args[i];
+      remotes.emplace_back( args[i] );
   }
 
-  std::string remoteFile;
-  if( !BuildPath( remoteFile, env, remote ).IsOK() )
+  if( !local.empty() && remotes.size() > 1 )
   {
-    log->Error( AppMsg, "Invalid path." );
+    log->Error( AppMsg, "If '-o' is used only can be used with only one remote file." );
     return XRootDStatus( stError, errInvalidArgs );
   }
 
-  URL remoteUrl( server );
-  remoteUrl.SetPath( remoteFile );
+  std::vector<URL> remoteUrls;
+  remoteUrls.reserve( remotes.size() );
+  for( auto &remote : remotes )
+  {
+    std::string remoteFile;
+    if( !BuildPath( remoteFile, env, remote ).IsOK() )
+    {
+      log->Error( AppMsg, "Invalid path." );
+      return XRootDStatus( stError, errInvalidArgs );
+    }
+
+    remoteUrls.emplace_back( server );
+    remoteUrls.back().SetPath( remoteFile );
+  }
 
   //----------------------------------------------------------------------------
   // Fetch the data
   //----------------------------------------------------------------------------
   CopyProgressHandler *handler = 0; ProgressDisplay d;
-  CopyProcess process; PropertyList props; PropertyList results;
+  CopyProcess process;
+  std::vector<PropertyList> props( remoteUrls.size() ), results( remoteUrls.size() );
 
-  props.Set( "source", remoteUrl.GetURL() );
-  if( !local.empty() )
+  for( size_t i = 0; i < remoteUrls.size(); ++i )
   {
-    props.Set( "target", std::string( "file://" ) + local );
-    handler = &d;
-  }
-  else
-    props.Set( "target", "stdio://-" );
+    props[i].Set( "source", remoteUrls[i].GetURL() );
+    if( !local.empty() )
+    {
+      props[i].Set( "target", std::string( "file://" ) + local );
+      handler = &d;
+    }
+    else
+      props[i].Set( "target", "stdio://-" );
 
-  props.Set( "dynamicSource", true );
+    props[i].Set( "dynamicSource", true );
 
-  XRootDStatus st = process.AddJob( props, &results );
-  if( !st.IsOK() )
-  {
-    log->Error( AppMsg, "Job adding failed: %s.", st.ToStr().c_str() );
-    return st;
+    XRootDStatus st = process.AddJob( props[i], &results[i] );
+    if( !st.IsOK() )
+    {
+      log->Error( AppMsg, "Job adding failed: %s.", st.ToStr().c_str() );
+      return st;
+    }
   }
 
-  st = process.Prepare();
+  XRootDStatus st = process.Prepare();
   if( !st.IsOK() )
   {
     log->Error( AppMsg, "Copy preparation failed: %s.", st.ToStr().c_str() );
@@ -1639,7 +1736,7 @@ XRootDStatus DoXAttr( FileSystem                      *fs,
 
       std::vector<XAttrStatus> result;
       XRootDStatus status = fs->SetXAttr( path, attrs, result );
-      XAttrStatus xst = result.front();
+      XAttrStatus xst = status.IsOK() ? result.front() : XAttrStatus( key, status );
 
       if( !xst.status.IsOK() )
         status = xst.status;
@@ -1665,7 +1762,7 @@ XRootDStatus DoXAttr( FileSystem                      *fs,
 
       std::vector<XAttr> result;
       XRootDStatus status = fs->GetXAttr( path, attrs, result );
-      XAttr xattr = result.front();
+      XAttr xattr = status.IsOK() ? result.front() : XAttr( key, status );
 
       if( !xattr.status.IsOK() )
         status = xattr.status;
@@ -1697,7 +1794,7 @@ XRootDStatus DoXAttr( FileSystem                      *fs,
 
       std::vector<XAttrStatus> result ;
       XRootDStatus status = fs->DelXAttr( path, attrs, result );
-      XAttrStatus xst = result.front();
+      XAttrStatus xst = status.IsOK() ? result.front() : XAttrStatus( key, status );
 
       if( !xst.status.IsOK() )
         status = xst.status;
@@ -1768,13 +1865,14 @@ XRootDStatus PrintHelp( FileSystem *, Env *,
   printf( "     Modify permissions. Permission string example:\n"             );
   printf( "     rwxr-x--x\n\n"                                                );
 
-  printf( "   ls [-l] [-u] [-R] [-D] [-Z] [dirname]\n"                        );
+  printf( "   ls [-l] [-u] [-R] [-D] [-Z] [-C] [dirname]\n"                   );
   printf( "     Get directory listing.\n"                                     );
   printf( "     -l stat every entry and pring long listing\n"                 );
   printf( "     -u print paths as URLs\n"                                     );
   printf( "     -R list subdirectories recursively\n"                         );
   printf( "     -D show duplicate entries"                                    );
-  printf( "     -Z if a ZIP archive list its content\n\n"                     );
+  printf( "     -Z if a ZIP archive list its content\n"                       );
+  printf( "     -C checksum every entry\n\n"                                  );
 
   printf( "   locate [-n] [-r] [-d] [-m] [-i] [-p] <path>\n"                  );
   printf( "     Get the locations of the path.\n"                             );
@@ -1862,8 +1960,8 @@ XRootDStatus PrintHelp( FileSystem *, Env *,
   printf( "     -a abort stage request\n"                                   );
   printf( "     -e evict the file from disk cache\n\n"                      );
 
-  printf( "   cat [-o local file] file\n"                                   );
-  printf( "     Print contents of a file to stdout.\n"                      );
+  printf( "   cat [-o local file] files\n"                                  );
+  printf( "     Print contents of one or more files to stdout.\n"           );
   printf( "     -o print to the specified local file\n\n"                   );
 
   printf( "   tail [-c bytes] [-f] file\n"                                  );
