@@ -55,9 +55,9 @@ namespace XrdCl
     pStateHandler = new FileStateHandler( virtRedirect == EnableVirtRedirect, pPlugIn );
   }
 
-  //------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   // Destructor
-  //------------------------------------------------------------------------
+  //----------------------------------------------------------------------------
   File::~File()
   {
     //--------------------------------------------------------------------------
@@ -66,8 +66,11 @@ namespace XrdCl
     // its garbage collector, from its __cxa_finalize, ie. after the XrdCl lib
     // has been finalized by the linker. So, if we don't have the log object
     // at this point we just give up the hope.
+    // Also, make sure the PostMaster threads are running - if not the Close
+    // will hang forever (this could happen when Python interpreter exits).
     //--------------------------------------------------------------------------
-    if ( DefaultEnv::GetLog() && IsOpen() ) {XRootDStatus status = Close();}
+    if ( DefaultEnv::GetLog() && DefaultEnv::GetPostMaster()->IsRunning() && IsOpen() )
+      XRootDStatus status = Close();
     delete pStateHandler;
     delete pPlugIn;
   }
@@ -363,7 +366,10 @@ namespace XrdCl
                               ResponseHandler       *handler,
                               uint16_t               timeout )
   {
-    return XRootDStatus( stError, errNotImplemented );
+    if( pPlugIn )
+      return pPlugIn->PgWrite( offset, size, buffer, cksums, handler, timeout );
+
+    return pStateHandler->PgWrite( offset, size, buffer, cksums, handler, timeout );
   }
 
   //------------------------------------------------------------------------
@@ -375,7 +381,13 @@ namespace XrdCl
                               std::vector<uint32_t> &cksums,
                               uint16_t               timeout )
   {
-    return XRootDStatus( stError, errNotImplemented );
+    SyncResponseHandler handler;
+    XRootDStatus st = PgWrite( offset, size, buffer, cksums, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    XRootDStatus status = MessageUtils::WaitForStatus( &handler );
+    return status;
   }
 
   //----------------------------------------------------------------------------
@@ -821,6 +833,16 @@ namespace XrdCl
     return pStateHandler->IsOpen();
   }
 
+  //------------------------------------------------------------------------
+  //! Check if the file is using an encrypted connection
+  //------------------------------------------------------------------------
+  bool File::IsSecure() const
+  {
+    if( pPlugIn )
+      return false;
+    return pStateHandler->IsSecure();
+  }
+
   //----------------------------------------------------------------------------
   // Set file property
   //----------------------------------------------------------------------------
