@@ -28,27 +28,27 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Filter messages
   //----------------------------------------------------------------------------
-  bool InQueue::DiscardMessage(Message* msg, uint16_t& sid) const
+  bool InQueue::DiscardMessage( Message& msg, uint16_t& sid) const
   {
-    if( msg->GetSize() < 8 )
+    if( msg.GetSize() < 8 )
       return true;
 
-    ServerResponse *rsp = (ServerResponse *)msg->GetBuffer();
+    ServerResponse *rsp = (ServerResponse *)msg.GetBuffer();
 
     // We got an async message
     if( rsp->hdr.status == kXR_attn )
     {
-      if( msg->GetSize() < 12 )
-	return true;
+      if( msg.GetSize() < 12 )
+        return true;
 
       // We only care about async responses
       if( rsp->body.attn.actnum != (int32_t)htonl(kXR_asynresp) )
-	return true;
+        return true;
 
-      if( msg->GetSize() < 24 )
-	return true;
+      if( msg.GetSize() < 24 )
+        return true;
 
-      ServerResponse *embRsp = (ServerResponse*)msg->GetBuffer(16);
+      ServerResponse *embRsp = (ServerResponse*)msg.GetBuffer(16);
       sid = ((uint16_t)embRsp->hdr.streamid[1] << 8) | (uint16_t)embRsp->hdr.streamid[0];
     }
     else
@@ -62,13 +62,13 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Add a message to the queue
   //----------------------------------------------------------------------------
-  bool InQueue::AddMessage( Message *msg )
+  bool InQueue::AddMessage( std::shared_ptr<Message> msg )
   {
     uint16_t            action  = 0;
-    IncomingMsgHandler* handler = 0;
+    MsgHandler* handler = 0;
     uint16_t msgSid = 0;
 
-    if (DiscardMessage(msg, msgSid))
+    if (DiscardMessage(*msg, msgSid))
     {
       return true;
     }
@@ -82,17 +82,16 @@ namespace XrdCl
       handler = it->second.first;
       action  = handler->Examine( msg );
 
-      if( action & IncomingMsgHandler::RemoveHandler )
+      if( action & MsgHandler::RemoveHandler )
         pHandlers.erase( it );
     }
-
-    if( !(action & IncomingMsgHandler::Take) )
+    else
       pMessages[msgSid] = msg;
 
     pMutex.UnLock();
 
-    if( handler && !(action & IncomingMsgHandler::NoProcess) )
-      handler->Process( msg );
+    if( handler && !(action & MsgHandler::NoProcess) )
+      handler->Process();
 
     return true;
   }
@@ -100,44 +99,37 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Add a listener that should be notified about incoming messages
   //----------------------------------------------------------------------------
-  void InQueue::AddMessageHandler( IncomingMsgHandler *handler, time_t expires )
+  void InQueue::AddMessageHandler( MsgHandler *handler, time_t expires, bool &rmMsg )
   {
-    uint16_t action = 0;
     uint16_t handlerSid = handler->GetSid();
     XrdSysMutexHelper scopedLock( pMutex );
     MessageMap::iterator it = pMessages.find(handlerSid);
 
-    if (it != pMessages.end())
-    {
-      action = handler->Examine( it->second );
+    //--------------------------------------------------------------------------
+    // If there is a leftover message in the in-queue simply remove it, there's
+    // no way this is the actual message we are waiting for as we just send it
+    // over the wire and we are still occupying the event-loop thread
+    //--------------------------------------------------------------------------
+    if( ( rmMsg = it != pMessages.end() ) )
+      pMessages.erase( it );
 
-      if( action & IncomingMsgHandler::Take )
-      {
-        if( !(action & IncomingMsgHandler::NoProcess ) )
-          handler->Process( it->second );
-
-        pMessages.erase( it );
-      }
-    }
-
-    if( !(action & IncomingMsgHandler::RemoveHandler) )
-      pHandlers[handlerSid] = HandlerAndExpire( handler, expires );
+    pHandlers[handlerSid] = HandlerAndExpire( handler, expires );
   }
 
   //----------------------------------------------------------------------------
   // Get a message handler interested in receiving message whose header
   // is stored in msg
   //----------------------------------------------------------------------------
-  IncomingMsgHandler *InQueue::GetHandlerForMessage( Message  *msg,
-						     time_t   &expires,
-						     uint16_t &action )
+  MsgHandler *InQueue::GetHandlerForMessage( std::shared_ptr<Message> &msg,
+						                                         time_t                   &expires,
+						                                         uint16_t                 &action )
   {
     time_t   exp = 0;
     uint16_t act = 0;
     uint16_t msgSid = 0;
-    IncomingMsgHandler* handler = 0;
+    MsgHandler* handler = 0;
 
-    if (DiscardMessage(msg, msgSid))
+    if (DiscardMessage(*msg, msgSid))
     {
       return handler;
     }
@@ -151,7 +143,7 @@ namespace XrdCl
       act     = handler->Examine( msg );
       exp     = it->second.second;
 
-      if( act & IncomingMsgHandler::RemoveHandler )
+      if( act & MsgHandler::RemoveHandler )
         pHandlers.erase( it );
     }
 
@@ -167,7 +159,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Re-insert the handler without scanning the cached messages
   //----------------------------------------------------------------------------
-  void InQueue::ReAddMessageHandler( IncomingMsgHandler *handler,
+  void InQueue::ReAddMessageHandler( MsgHandler *handler,
 				     time_t              expires )
   {
     uint16_t handlerSid = handler->GetSid();
@@ -178,7 +170,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Remove a listener
   //----------------------------------------------------------------------------
-  void InQueue::RemoveMessageHandler( IncomingMsgHandler *handler )
+  void InQueue::RemoveMessageHandler( MsgHandler *handler )
   {
     uint16_t handlerSid = handler->GetSid();
     XrdSysMutexHelper scopedLock( pMutex );
@@ -188,7 +180,7 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Report an event to the handlers
   //----------------------------------------------------------------------------
-  void InQueue::ReportStreamEvent( IncomingMsgHandler::StreamEvent event,
+  void InQueue::ReportStreamEvent( MsgHandler::StreamEvent event,
 				   XRootDStatus                    status )
   {
     uint8_t action = 0;
@@ -197,7 +189,7 @@ namespace XrdCl
     {
       action = it->second.first->OnStreamEvent( event, status );
 
-      if( action & IncomingMsgHandler::RemoveHandler )
+      if( action & MsgHandler::RemoveHandler )
       {
         auto next = it; ++next;
         pHandlers.erase( it );
@@ -222,10 +214,10 @@ namespace XrdCl
     {
       if( it->second.second <= now )
       {
-        uint8_t act = it->second.first->OnStreamEvent( IncomingMsgHandler::Timeout,
+        uint8_t act = it->second.first->OnStreamEvent( MsgHandler::Timeout,
                                          Status( stError, errOperationExpired ) );
         auto next = it; ++next;
-        if( act & IncomingMsgHandler::RemoveHandler )
+        if( act & MsgHandler::RemoveHandler )
           pHandlers.erase( it );
         it = next;
       }
