@@ -40,11 +40,11 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Send a message
   //----------------------------------------------------------------------------
-  XRootDStatus MessageUtils::SendMessage( const URL               &url,
-                                          Message                 *msg,
-                                          ResponseHandler         *handler,
-                                          const MessageSendParams &sendParams,
-                                          LocalFileHandler        *lFileHandler )
+  XRootDStatus MessageUtils::SendMessage( const URL         &url,
+                                          Message           *msg,
+                                          ResponseHandler   *handler,
+                                          MessageSendParams &sendParams,
+                                          LocalFileHandler  *lFileHandler )
   {
     //--------------------------------------------------------------------------
     // Get the stuff needed to send the message
@@ -76,6 +76,21 @@ namespace XrdCl
       return st;
     }
 
+    //--------------------------------------------------------------------------
+    // Make sure that in case of checkpoint xeq request the embedded request
+    // SID is matching
+    //--------------------------------------------------------------------------
+    if( req->requestid == kXR_chkpoint )
+    {
+      ClientRequest *r = (ClientRequest*)req;
+      if( r->chkpoint.opcode == kXR_ckpXeq )
+      {
+        ClientRequest *xeq = (ClientRequest*) msg->GetBuffer( sizeof( ClientChkPointRequest ) );
+        xeq->header.streamid[0] = req->streamid[0];
+        xeq->header.streamid[1] = req->streamid[1];
+      }
+    }
+
     XRootDTransport::MarshallRequest( msg );
 
     //--------------------------------------------------------------------------
@@ -87,14 +102,22 @@ namespace XrdCl
     msgHandler->SetRedirectAsAnswer( !sendParams.followRedirects );
     msgHandler->SetOksofarAsAnswer( sendParams.chunkedResponse );
     msgHandler->SetChunkList( sendParams.chunkList );
+    msgHandler->SetKernelBuffer( sendParams.kbuff );
     msgHandler->SetRedirectCounter( sendParams.redirectLimit );
     msgHandler->SetStateful( sendParams.stateful );
+    msgHandler->SetCrc32cDigests( std::move( sendParams.crc32cDigests ) );
 
     if( sendParams.loadBalancer.url.IsValid() )
       msgHandler->SetLoadBalancer( sendParams.loadBalancer );
 
     HostList *list = 0;
-    list = new HostList();
+    if( sendParams.hostList )
+    {
+      list = sendParams.hostList;
+      sendParams.hostList = nullptr;
+    }
+    else
+      list = new HostList();
     list->push_back( url );
     msgHandler->SetHostList( list );
 
@@ -116,7 +139,6 @@ namespace XrdCl
       // Release the SID as the request was never send
       sidMgr->ReleaseSID( req->streamid );
       delete msgHandler;
-      delete list;
       return st;
     }
     return XRootDStatus();
@@ -166,7 +188,7 @@ namespace XrdCl
     msgHandler->SetFollowMetalink( true );
 
     HostInfo info( url, true );
-    info.flags = kXR_isManager | kXR_attrMeta;
+    info.flags = kXR_isManager | kXR_attrMeta | kXR_attrVirtRdr;
     sendParams.loadBalancer = info;
     msgHandler->SetLoadBalancer( info );
 
