@@ -46,6 +46,7 @@ namespace
     { errDataError,            "Received corrupted data" },
     { errNotImplemented,       "Operation is not implemented" },
     { errNoMoreReplicas,       "No more replicas to try" },
+    { errPipelineFailed,       "Pipeline failed" },
     { errInvalidAddr,          "Invalid address"      },
     { errSocketError,          "Socket error"         },
     { errSocketTimeout,        "Socket timeout"       },
@@ -66,6 +67,7 @@ namespace
     { errQueryNotSupported,    "Query not supported"  },
     { errOperationExpired,     "Operation expired"    },
     { errOperationInterrupted, "Operation interrupted" },
+    { errThresholdExceeded,    "Threshold exceeded"   },
     { errNoMoreFreeSIDs,       "No more free SIDs"    },
     { errInvalidRedirectURL,   "Invalid redirect URL" },
     { errInvalidResponse,      "Invalid response"     },
