@@ -44,44 +44,24 @@
 
 namespace XrdCl
 {
-  //----------------------------------------------------------------------------
-  // Outgoing message helper
-  //----------------------------------------------------------------------------
-  struct OutMessageHelper
-  {
-    OutMessageHelper( Message              *message = 0,
-                      OutgoingMsgHandler   *hndlr   = 0,
-                      time_t                expir   = 0,
-                      bool                  statefu = 0 ):
-      msg( message ), handler( hndlr ), expires( expir ), stateful( statefu ) {}
-    void Reset()
-    {
-      msg = 0; handler = 0; expires = 0; stateful = 0;
-    }
-    Message              *msg;
-    OutgoingMsgHandler   *handler;
-    time_t                expires;
-    bool                  stateful;
-  };
-
   //----------------------------------------------------------------------------
   // Incoming message helper
   //----------------------------------------------------------------------------
   struct InMessageHelper
   {
-    InMessageHelper( Message              *message = 0,
-                     IncomingMsgHandler   *hndlr   = 0,
-                     time_t                expir   = 0,
-                     uint16_t              actio   = 0 ):
+    InMessageHelper( Message      *message = 0,
+                     MsgHandler   *hndlr   = 0,
+                     time_t        expir   = 0,
+                     uint16_t      actio   = 0 ):
       msg( message ), handler( hndlr ), expires( expir ), action( actio ) {}
     void Reset()
     {
       msg = 0; handler = 0; expires = 0; action = 0;
     }
-    Message              *msg;
-    IncomingMsgHandler   *handler;
-    time_t                expires;
-    uint16_t              action;
+    Message      *msg;
+    MsgHandler   *handler;
+    time_t        expires;
+    uint16_t      action;
   };
 
   //----------------------------------------------------------------------------
@@ -100,7 +80,7 @@ namespace XrdCl
     }
     AsyncSocketHandler   *socket;
     OutQueue             *outQueue;
-    OutMessageHelper      outMsgHelper;
+    OutQueue::MsgHelper   outMsgHelper;
     InMessageHelper       inMsgHelper;
     Socket::SocketStatus  status;
   };
@@ -108,8 +88,9 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Constructor
   //----------------------------------------------------------------------------
-  Stream::Stream( const URL *url ):
+  Stream::Stream( const URL *url, const URL &prefer ):
     pUrl( url ),
+    pPrefer( prefer ),
     pTransport( 0 ),
     pPoller( 0 ),
     pTaskManager( 0 ),
@@ -121,7 +102,6 @@ namespace XrdCl
     pConnectionInitTime( 0 ),
     pAddressType( Utils::IPAll ),
     pSessionId( 0 ),
-    pQueueIncMsgJob(0),
     pBytesSent( 0 ),
     pBytesReceived( 0 )
   {
@@ -167,8 +147,6 @@ namespace XrdCl
     SubStreamList::iterator it;
     for( it = pSubStreams.begin(); it != pSubStreams.end(); ++it )
       delete *it;
-
-    delete pQueueIncMsgJob;
   }
 
   //----------------------------------------------------------------------------
@@ -180,9 +158,7 @@ namespace XrdCl
       return XRootDStatus( stError, errUninitialized );
 
     AsyncSocketHandler *s = new AsyncSocketHandler( *pUrl, pPoller, pTransport,
-                                                    pChannelData, 0 );
-    s->SetStream( this );
-
+                                                    pChannelData, 0, this );
     pSubStreams.push_back( new SubStreamData() );
     pSubStreams[0]->socket = s;
     return XRootDStatus();
@@ -254,6 +230,33 @@ namespace XrdCl
       return st;
     }
 
+    if( pPrefer.IsValid() )
+    {
+      std::vector<XrdNetAddr> addrresses;
+      XRootDStatus st = Utils::GetHostAddresses( addrresses, pPrefer, pAddressType );
+      if( !st.IsOK() )
+      {
+        log->Error( PostMasterMsg, "[%s] Unable to resolve IP address for %s",
+                    pStreamName.c_str(), pPrefer.GetHostName().c_str() );
+      }
+      else
+      {
+        std::vector<XrdNetAddr> tmp;
+        tmp.reserve( pAddresses.size() );
+        // first add all remaining addresses
+        auto itr = pAddresses.begin();
+        for( ; itr != pAddresses.end() ; ++itr )
+        {
+          if( !HasNetAddr( *itr, addrresses ) )
+            tmp.push_back( *itr );
+        }
+        // then copy all 'preferred' addresses
+        std::copy( addrresses.begin(), addrresses.end(), std::back_inserter( tmp ) );
+        // and keep the result
+        pAddresses.swap( tmp );
+      }
+    }
+
     Utils::LogHostAddresses( log, PostMasterMsg, pUrl->GetHostId(),
                              pAddresses );
 
@@ -275,10 +278,10 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Queue the message for sending
   //----------------------------------------------------------------------------
-  XRootDStatus Stream::Send( Message              *msg,
-                             OutgoingMsgHandler   *handler,
-                             bool                  stateful,
-                             time_t                expires )
+  XRootDStatus Stream::Send( Message      *msg,
+                             MsgHandler   *handler,
+                             bool          stateful,
+                             time_t        expires )
   {
     XrdSysMutexHelper scopedLock( pMutex );
     Log *log = DefaultEnv::GetLog();
@@ -404,9 +407,9 @@ namespace
 
 namespace XrdCl
 {
-  XRootDStatus Stream::RequestClose( Message *response )
+  XRootDStatus Stream::RequestClose( Message &response )
   {
-    ServerResponse *rsp = reinterpret_cast<ServerResponse*>( response->GetBuffer() );
+    ServerResponse *rsp = reinterpret_cast<ServerResponse*>( response.GetBuffer() );
     if( rsp->hdr.dlen < 4 ) return XRootDStatus( stError );
     Message            *msg;
     ClientCloseRequest *req;
@@ -424,81 +427,110 @@ namespace XrdCl
     return MessageUtils::SendMessage( *pUrl, msg, handler, params, 0 );
   }
 
+  //------------------------------------------------------------------------
+  // Check if message is a partial response
+  //------------------------------------------------------------------------
+  bool Stream::IsPartial( Message &msg )
+  {
+    ServerResponseHeader *rsphdr = (ServerResponseHeader*)msg.GetBuffer();
+    if( rsphdr->status == kXR_oksofar )
+      return true;
+
+    if( rsphdr->status == kXR_status )
+    {
+      ServerResponseStatus *rspst = (ServerResponseStatus*)msg.GetBuffer();
+      if( rspst->bdy.resptype == XrdProto::kXR_PartialResult )
+        return true;
+    }
+
+    return false;
+  }
+
   //----------------------------------------------------------------------------
   // Call back when a message has been reconstructed
   //----------------------------------------------------------------------------
   void Stream::OnIncoming( uint16_t subStream,
-                           Message  *msg,
+                           std::shared_ptr<Message>  msg,
                            uint32_t  bytesReceived )
   {
     msg->SetSessionId( pSessionId );
     pBytesReceived += bytesReceived;
 
-    uint32_t streamAction = pTransport->MessageReceived( msg, subStream,
+    uint32_t streamAction = pTransport->MessageReceived( *msg, subStream,
                                                          *pChannelData );
     if( streamAction & TransportHandler::DigestMsg )
       return;
 
     if( streamAction & TransportHandler::RequestClose )
     {
-      RequestClose( msg );
-      delete msg;
+      RequestClose( *msg );
       return;
     }
 
-    //--------------------------------------------------------------------------
-    // No handler, we cache and see what comes later
-    //--------------------------------------------------------------------------
     Log *log = DefaultEnv::GetLog();
     InMessageHelper &mh = pSubStreams[subStream]->inMsgHelper;
 
+    //--------------------------------------------------------------------------
+    // Check if we can obtain a handler now that we have the whole response
+    //--------------------------------------------------------------------------
     if( !mh.handler )
     {
-      log->Dump( PostMasterMsg, "[%s] Queuing received message: 0x%x.",
-                 pStreamName.c_str(), msg );
-
-      pJobManager->QueueJob( pQueueIncMsgJob, msg );
-      return;
+      //------------------------------------------------------------------------
+      // Try once more if we can obtain a handler for the message, for the async
+      // (kXR_attn) messages this can be only done once the whole message has
+      // been read out from the socket
+      //------------------------------------------------------------------------
+      mh.handler = pIncomingQueue->GetHandlerForMessage( msg,
+                                                         mh.expires,
+                                                         mh.action );
+      //------------------------------------------------------------------------
+      // No handler, we discard the message ...
+      //------------------------------------------------------------------------
+      if( !mh.handler )
+      {
+        ServerResponse *rsp = (ServerResponse*)msg->GetBuffer();
+        log->Warning( PostMasterMsg, "[%s] Discarding received message: 0x%x "
+                      "(status=%d, SID=[%d,%d]), no MsgHandler found.",
+                      pStreamName.c_str(), msg.get(), rsp->hdr.status,
+                      rsp->hdr.streamid[0], rsp->hdr.streamid[1] );
+        return;
+      }
     }
 
     //--------------------------------------------------------------------------
     // We have a handler, so we call the callback
     //--------------------------------------------------------------------------
     log->Dump( PostMasterMsg, "[%s] Handling received message: 0x%x.",
-               pStreamName.c_str(), msg );
+               pStreamName.c_str(), msg.get() );
 
-    if( !(mh.action & IncomingMsgHandler::RemoveHandler) )
+    if( !(mh.action & MsgHandler::RemoveHandler) )
       pIncomingQueue->ReAddMessageHandler( mh.handler, mh.expires );
 
-    if( mh.action & (IncomingMsgHandler::NoProcess|IncomingMsgHandler::Ignore) )
+    if( mh.action & (MsgHandler::NoProcess|MsgHandler::Ignore) )
     {
       log->Dump( PostMasterMsg, "[%s] Ignoring the processing handler for: 0x%x.",
                  pStreamName.c_str(), msg->GetDescription().c_str() );
 
-      // if we are handling oksofar we have to take down the timeout fence
-      ServerResponse *rsp = (ServerResponse *)msg->GetBuffer();
-      if( rsp->hdr.status == kXR_oksofar )
+      // if we are handling partial response we have to take down the timeout fence
+      if( IsPartial( *msg ) )
       {
         XRootDMsgHandler *xrdHandler = dynamic_cast<XRootDMsgHandler*>( mh.handler );
-        if( xrdHandler ) xrdHandler->TakeDownTimeoutFence();
+        if( xrdHandler ) xrdHandler->PartialReceived();
       }
 
-      bool delit = ( mh.action & IncomingMsgHandler::Ignore );
       mh.Reset();
-      if (delit) delete msg;
-
       return;
     }
 
     Job *job = new HandleIncMsgJob( mh.handler );
     mh.Reset();
-    pJobManager->QueueJob( job, msg );
+    pJobManager->QueueJob( job );
   }
 
   //----------------------------------------------------------------------------
   // Call when one of the sockets is ready to accept a new message
   //----------------------------------------------------------------------------
-  std::pair<Message *, OutgoingMsgHandler *>
+  std::pair<Message *, MsgHandler *>
     Stream::OnReadyToWrite( uint16_t subStream )
   {
     XrdSysMutexHelper scopedLock( pMutex );
@@ -509,10 +541,10 @@ namespace XrdCl
                  pSubStreams[subStream]->socket->GetStreamName().c_str() );
 
       pSubStreams[subStream]->socket->DisableUplink();
-      return std::make_pair( (Message *)0, (OutgoingMsgHandler *)0 );
+      return std::make_pair( (Message *)0, (MsgHandler *)0 );
     }
 
-    OutMessageHelper &h = pSubStreams[subStream]->outMsgHelper;
+    OutQueue::MsgHelper &h = pSubStreams[subStream]->outMsgHelper;
     h.msg = pSubStreams[subStream]->outQueue->PopMessage( h.handler,
                                                           h.expires,
                                                           h.stateful );
@@ -544,10 +576,20 @@ namespace XrdCl
   {
     pTransport->MessageSent( msg, subStream, bytesSent,
                              *pChannelData );
-    OutMessageHelper &h = pSubStreams[subStream]->outMsgHelper;
+    OutQueue::MsgHelper &h = pSubStreams[subStream]->outMsgHelper;
     pBytesSent += bytesSent;
     if( h.handler )
+    {
       h.handler->OnStatusReady( msg, XRootDStatus() );
+      bool rmMsg = false;
+      pIncomingQueue->AddMessageHandler( h.handler, h.handler->GetExpiration(), rmMsg );
+      if( rmMsg )
+      {
+        Log *log = DefaultEnv::GetLog();
+        log->Warning( PostMasterMsg, "[%s] Removed a leftover msg from the in-queue.",
+                      pStreamName.c_str(), subStream );
+      }
+    }
     pSubStreams[subStream]->outMsgHelper.Reset();
   }
 
@@ -577,9 +619,9 @@ namespace XrdCl
       {
         for( uint16_t i = 1; i < numSub; ++i )
         {
-          AsyncSocketHandler *s = new AsyncSocketHandler( *pUrl, pPoller,
-                                                   pTransport, pChannelData, i );
-          s->SetStream( this );
+          URL url = pTransport->GetBindPreference( *pUrl, *pChannelData );
+          AsyncSocketHandler *s = new AsyncSocketHandler( url, pPoller, pTransport,
+                                                          pChannelData, i, this );
           pSubStreams.push_back( new SubStreamData() );
           pSubStreams[i]->socket = s;
         }
@@ -658,6 +700,11 @@ namespace XrdCl
     pSubStreams[subStream]->socket->Close();
     time_t now = ::time(0);
 
+    //--------------------------------------------------------------------------
+    // For every connection error call the global connection error handler
+    //--------------------------------------------------------------------------
+    XrdCl::DefaultEnv::GetPostMaster()->NotifyConnErrHandler( *pUrl, status );
+
     //--------------------------------------------------------------------------
     // If we connected subStream == 0 and cannot connect >0 then we just give
     // up and move the outgoing messages to another queue
@@ -761,7 +808,7 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     if( pSubStreams[subStream]->outMsgHelper.msg )
     {
-      OutMessageHelper &h = pSubStreams[subStream]->outMsgHelper;
+      OutQueue::MsgHelper &h = pSubStreams[subStream]->outMsgHelper;
       pSubStreams[subStream]->outQueue->PushFront( h.msg, h.handler, h.expires,
                                                    h.stateful );
       pSubStreams[subStream]->outMsgHelper.Reset();
@@ -839,7 +886,7 @@ namespace XrdCl
       scopedLock.UnLock();
 
       q.Report( status );
-      pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::Broken, status );
+      pIncomingQueue->ReportStreamEvent( MsgHandler::Broken, status );
       pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status );
       return;
     }
@@ -865,7 +912,7 @@ namespace XrdCl
       //--------------------------------------------------------------------
       if( pSubStreams[substream]->outMsgHelper.msg )
       {
-        OutMessageHelper &h = pSubStreams[substream]->outMsgHelper;
+        OutQueue::MsgHelper &h = pSubStreams[substream]->outMsgHelper;
         pSubStreams[substream]->outQueue->PushFront( h.msg, h.handler, h.expires,
                                                      h.stateful );
         pSubStreams[substream]->outMsgHelper.Reset();
@@ -898,7 +945,7 @@ namespace XrdCl
 
       q.Report( status );
 
-      pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::Broken, status );
+      pIncomingQueue->ReportStreamEvent( MsgHandler::Broken, status );
       pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status );
     }
   }
@@ -934,7 +981,7 @@ namespace XrdCl
 
     status.status = stFatal;
     q.Report( status );
-    pIncomingQueue->ReportStreamEvent( IncomingMsgHandler::FatalError, status );
+    pIncomingQueue->ReportStreamEvent( MsgHandler::FatalError, status );
     pChannelEvHandlers.ReportEvent( ChannelEventHandler::FatalError, status );
 
   }
@@ -1051,8 +1098,8 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Install a incoming message handler
   //----------------------------------------------------------------------------
-  std::pair<IncomingMsgHandler *, bool>
-        Stream::InstallIncHandler( Message *msg, uint16_t stream )
+  MsgHandler*
+        Stream::InstallIncHandler( std::shared_ptr<Message> &msg, uint16_t stream )
   {
     InMessageHelper &mh = pSubStreams[stream]->inMsgHelper;
     if( !mh.handler )
@@ -1061,11 +1108,87 @@ namespace XrdCl
                                                          mh.action );
 
     if( !mh.handler )
-      return std::make_pair( (IncomingMsgHandler*)0, false );
+      return nullptr;
 
-    bool ownership = mh.action & IncomingMsgHandler::Take;
-    if( mh.action & IncomingMsgHandler::Raw )
-      return std::make_pair( mh.handler, ownership );
-    return std::make_pair( (IncomingMsgHandler*)0, ownership );
+    if( mh.action & MsgHandler::Raw )
+      return mh.handler;
+    return nullptr;
+  }
+
+  //----------------------------------------------------------------------------
+  //! In case the message is a kXR_status response it needs further attention
+  //!
+  //! @return : a MsgHandler in case we need to read out raw data
+  //----------------------------------------------------------------------------
+  uint16_t Stream::InspectStatusRsp( uint16_t     stream,
+                                     MsgHandler *&incHandler )
+  {
+    InMessageHelper &mh = pSubStreams[stream]->inMsgHelper;
+    if( !mh.handler ) return false;
+
+    uint16_t action = mh.handler->InspectStatusRsp();
+    mh.action |= action;
+
+    if( action & MsgHandler::RemoveHandler )
+      pIncomingQueue->RemoveMessageHandler( mh.handler );
+
+    if( action & MsgHandler::Raw )
+    {
+      incHandler = mh.handler;
+      return MsgHandler::Raw;
+    }
+
+    if( action & MsgHandler::Corrupted )
+      return MsgHandler::Corrupted;
+
+    if( action & MsgHandler::More )
+      return MsgHandler::More;
+
+    return MsgHandler::None;
+  }
+
+  //----------------------------------------------------------------------------
+  // Check if channel can be collapsed using given URL
+  //----------------------------------------------------------------------------
+  bool Stream::CanCollapse( const URL &url )
+  {
+    Log *log = DefaultEnv::GetLog();
+
+    //--------------------------------------------------------------------------
+    // Resolve all the addresses of the host we're supposed to connect to
+    //--------------------------------------------------------------------------
+    std::vector<XrdNetAddr> prefaddrs;
+    XRootDStatus st = Utils::GetHostAddresses( prefaddrs, url, pAddressType );
+    if( !st.IsOK() )
+    {
+      log->Error( PostMasterMsg, "[%s] Unable to resolve IP address for %s."
+                  , pStreamName.c_str(), url.GetHostName().c_str() );
+      return false;
+    }
+
+    //--------------------------------------------------------------------------
+    // Resolve all the addresses of the alias
+    //--------------------------------------------------------------------------
+    std::vector<XrdNetAddr> aliasaddrs;
+    st = Utils::GetHostAddresses( aliasaddrs, *pUrl, pAddressType );
+    if( !st.IsOK() )
+    {
+      log->Error( PostMasterMsg, "[%s] Unable to resolve IP address for %s."
+                  , pStreamName.c_str(), pUrl->GetHostName().c_str() );
+      return false;
+    }
+
+    //--------------------------------------------------------------------------
+    // Now check if the preferred host is part of the alias
+    //--------------------------------------------------------------------------
+    auto itr = prefaddrs.begin();
+    for( ; itr != prefaddrs.end() ; ++itr )
+    {
+      auto itr2 = aliasaddrs.begin();
+      for( ; itr2 != aliasaddrs.end() ; ++itr2 )
+        if( itr->Same( &*itr2 ) ) return true;
+    }
+
+    return false;
   }
 }
