@@ -217,7 +217,7 @@ namespace XrdCl
         XRootDStatus st = pSocketHandler->EnableUplink();
         if( !st.IsOK() ) status = st;
         //--------------------------------------------------------------------
-        // Return early so the revert state wont get cleared
+        // Return early so the revert state won't get cleared
         //--------------------------------------------------------------------
         return status;
       }
@@ -242,6 +242,25 @@ namespace XrdCl
     return status;
   }
 
+  //------------------------------------------------------------------------
+  //! (Fake) ReadV through the TLS layer from the socket
+  //! If necessary, will establish a TLS/SSL session.
+  //------------------------------------------------------------------------
+  XRootDStatus Tls::ReadV( iovec *iov, int iocnt, int &bytesRead )
+  {
+    bytesRead = 0;
+    for( int i = 0; i < iocnt; ++i )
+    {
+      int btsread = 0;
+      auto st = Read( static_cast<char*>( iov[i].iov_base ),
+                      iov[i].iov_len, btsread );
+      if( !st.IsOK() ) return st;
+      bytesRead += btsread;
+      if( st.code == suRetry ) return st;
+    }
+    return XRootDStatus();
+  }
+
   XRootDStatus Tls::Send( const char *buffer, size_t size, int &bytesWritten )
   {
     //--------------------------------------------------------------------------
@@ -279,7 +298,7 @@ namespace XrdCl
         XRootDStatus st = pSocketHandler->DisableUplink();
         if( !st.IsOK() ) status = st;
         //----------------------------------------------------------------------
-        // Return early so the revert state wont get cleared
+        // Return early so the revert state won't get cleared
         //----------------------------------------------------------------------
         return status;
       }
@@ -330,10 +349,14 @@ namespace XrdCl
       case XrdTls::TLS_UNK_Error:
       case XrdTls::TLS_SYS_Error: return XRootDStatus( stError, errTlsError, 0, msg );
 
+      case XrdTls::TLS_SSL_Error: return XRootDStatus( stFatal, errTlsError, EAGAIN, msg );
+
       case XrdTls::TLS_VER_Error:
-      case XrdTls::TLS_SSL_Error:
       case XrdTls::TLS_HNV_Error: return XRootDStatus( stFatal, errTlsError, 0, msg );
 
+      // the connection was closed by the server, treat this as a socket error
+      case XrdTls::TLS_CON_Closed: return XRootDStatus( stError, errSocketError );
+
       default:
         return XRootDStatus( stError, errTlsError, 0, msg );
     }
