@@ -598,21 +598,24 @@ namespace XrdCl
       offset( offset ),
       length( length ),
       buffer( buffer ),
-      cksums( std::move( cksums ) )
+      cksums( std::move( cksums ) ),
+      nbrepair( 0 )
     {
     }
 
     PageInfoImpl( PageInfoImpl &&pginf ) : offset( pginf.offset ),
                                            length( pginf.length ),
                                            buffer( pginf.buffer ),
-                                           cksums( std::move( pginf.cksums ) )
+                                           cksums( std::move( pginf.cksums ) ),
+                                           nbrepair( pginf.nbrepair )
     {
     }
 
-    uint64_t               offset; //> offset in the file
-    uint32_t               length; //> length of the data read
-    void                  *buffer; //> buffer with the read data
-    std::vector<uint32_t>  cksums; //> a vector of crc32c checksums
+    uint64_t               offset;   //> offset in the file
+    uint32_t               length;   //> length of the data read
+    void                  *buffer;   //> buffer with the read data
+    std::vector<uint32_t>  cksums;   //> a vector of crc32c checksums
+    size_t                 nbrepair; //> number of repaired pages
   };
 
   //----------------------------------------------------------------------------
@@ -631,6 +634,15 @@ namespace XrdCl
   {
   }
 
+  //----------------------------------------------------------------------------
+  //! Move assigment operator
+  //----------------------------------------------------------------------------
+  PageInfo& PageInfo::operator=( PageInfo &&pginf )
+  {
+    pImpl.swap( pginf.pImpl );
+    return *this;
+  }
+
   //----------------------------------------------------------------------------
   // Destructor
   //----------------------------------------------------------------------------
@@ -670,6 +682,72 @@ namespace XrdCl
     return pImpl->cksums;
   }
 
+  //----------------------------------------------------------------------------
+  // Set number of repaired pages
+  //----------------------------------------------------------------------------
+  void PageInfo::SetNbRepair( size_t nbrepair )
+  {
+    pImpl->nbrepair = nbrepair;
+  }
+
+  //----------------------------------------------------------------------------
+  // Get number of repaired pages
+  //----------------------------------------------------------------------------
+  size_t PageInfo::GetNbRepair()
+  {
+    return pImpl->nbrepair;
+  }
+
+  struct RetryInfoImpl
+  {
+      RetryInfoImpl( std::vector<std::tuple<uint64_t, uint32_t>> && retries ) :
+          retries( std::move( retries ) )
+      {
+
+      }
+
+      std::vector<std::tuple<uint64_t, uint32_t>> retries;
+  };
+
+  //----------------------------------------------------------------------------
+  // Constructor
+  //----------------------------------------------------------------------------
+  RetryInfo::RetryInfo( std::vector<std::tuple<uint64_t, uint32_t>> && retries ) :
+      pImpl( new RetryInfoImpl( std::move( retries ) ) )
+  {
+
+  }
+
+  //----------------------------------------------------------------------------
+  // Destructor
+  //----------------------------------------------------------------------------
+  RetryInfo::~RetryInfo(){ }
+
+  //----------------------------------------------------------------------------
+  // @return : true if some pages need retrying, false otherwise
+  //----------------------------------------------------------------------------
+  bool RetryInfo::NeedRetry()
+  {
+    return !pImpl->retries.empty();
+  }
+
+  //----------------------------------------------------------------------------
+  // @return number of pages that need to be retransmitted
+  //----------------------------------------------------------------------------
+  size_t RetryInfo::Size()
+  {
+    return pImpl->retries.size();
+  }
+
+  //----------------------------------------------------------------------------
+  // @return : offset and size of respective page that requires to be
+  //           retransmitted
+  //----------------------------------------------------------------------------
+  std::tuple<uint64_t, uint32_t> RetryInfo::At( size_t i )
+  {
+    return pImpl->retries[i];
+  }
+
   //------------------------------------------------------------------------
   // Factory function for generating handler objects from lambdas
   //------------------------------------------------------------------------
@@ -692,8 +770,11 @@ namespace XrdCl
           response = &nullref;
         // call the user completion handler
         func( *status, *response );
-        // deallocate the wrapper
-        delete this;
+        // check if this is a final respons
+        bool finalrsp = !( status->IsOK() && status->code == suContinue );
+        // deallocate the wrapper if final
+        if( finalrsp )
+          delete this;
       }
 
       std::function<void(XRootDStatus&, AnyObject&)> func;
@@ -712,10 +793,13 @@ namespace XrdCl
 
       void HandleResponse( XRootDStatus *status, AnyObject *response )
       {
+        // check if this is a final respons
+        bool finalrsp = !( status->IsOK() && status->code == suContinue );
         // call the user completion handler
         func( status, response );
-        // deallocate the wrapper
-        delete this;
+        // deallocate the wrapper if final
+        if( finalrsp )
+          delete this;
       }
 
       std::function<void(XRootDStatus*, AnyObject*)> func;
