@@ -28,10 +28,10 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <unistd.h>
 #include <netinet/in.h>
 #include <sys/types.h>
@@ -113,9 +113,7 @@ XrdCmsCluster::XrdCmsCluster()
      AltMent = -1;
      NodeCnt =  0;
      STHi    = -1;
-     SelWcnt = 0;
      SelWtot = 0;
-     SelRcnt = 0;
      SelRtot = 0;
      SelTcnt = 0;
      peerHost  = 0;
@@ -134,7 +132,7 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
    const char *act = "";
    XrdCmsNode *nP = 0;
    XrdCmsClustID *cidP = 0;
-   XrdSysMutexHelper STMHelper(STMutex);
+   XrdSysRWLockHelper STMHelper(STMutex, false); // Need write lock!
    int tmp, Slot, Free = -1, Bump1 = -1, Bump2 = -1, Bump3 = -1, aSet = 0;
    bool Special = (Status & (CMS_isMan|CMS_isPeer));
    bool SpecAlt = (Special && !(Status & CMS_isSuper));
@@ -189,7 +187,7 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
           }
       }
 
-// Reuse an old ID if we must or redirect the incomming node
+// Reuse an old ID if we must or redirect the incoming node
 //
    if (!nP) 
       {if (Free >= 0) Slot = Free;
@@ -209,7 +207,7 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
                    sendAList(NodeTab[Slot]->Link);
 
                 DEBUG(lp->ID << " bumps " << NodeTab[Slot]->Ident <<" #" <<Slot);
-                NodeTab[Slot]->Lock(true);
+                NodeTab[Slot]->Lock();
                 Remove("redirected", NodeTab[Slot], -1);
                 act = "Shoved ";
                }
@@ -268,7 +266,7 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
 
 // All done. Return the node locked.
 //
-   nP->Lock(false);
+   nP->Lock();
    return nP;
 }
   
@@ -276,7 +274,7 @@ XrdCmsNode *XrdCmsCluster::Add(XrdLink *lp, int port, int Status, int sport,
 /* Private:                       A d d A l t                                 */
 /******************************************************************************/
   
-// Warning STMutex must be held by the caller!
+// Warning STMutex must be held in write mode by the caller!
 
 XrdCmsNode *XrdCmsCluster::AddAlt(XrdCmsClustID *cidP, XrdLink *lp,
                                   int port, int Status, int sport,
@@ -336,9 +334,9 @@ void XrdCmsCluster::BlackList(XrdOucTList *blP)
    int i, blRD = 0;
    bool inBL;
 
-// Obtain a lock on the table
+// Obtain a lock on the table. We need this in write mode!
 //
-   STMutex.Lock();
+   STMutex.WriteLock();
 
 // Run through the table looking to put or out of the blacklist
 //
@@ -382,7 +380,7 @@ SMask_t XrdCmsCluster::Broadcast(SMask_t smask, const struct iovec *iod,
 
 // Obtain a lock on the table and screen out peer nodes
 //
-   STMutex.Lock();
+   STMutex.ReadLock(); // Sufficient to prevent modifications
    bmask = smask & peerMask;
 
 // Run through the table looking for nodes to send messages to. We don't need
@@ -392,12 +390,14 @@ SMask_t XrdCmsCluster::Broadcast(SMask_t smask, const struct iovec *iod,
    for (i = 0; i <= STHi; i++)
        {if ((nP = NodeTab[i]) && nP->isNode(bmask))
            {if (nP->isOffline) unQueried |= nP->Mask();
-               else {nP->g2Ref(STMutex);
+               else {nP->Ref();
+                     STMutex.UnLock();
                      if (nP->Send(iod, iovcnt, iotot) < 0)
                         {unQueried |= nP->Mask();
                          DEBUG(nP->Ident <<" is unreachable");
                         }
-                     nP->Ref2g(STMutex);
+                     nP->unRef();
+                     STMutex.ReadLock();
                     }
            }
        }
@@ -444,6 +444,8 @@ SMask_t XrdCmsCluster::Broadcast(SMask_t smask, XrdCms::CmsRRHdr &Hdr,
 /*                             B r o a d s e n d                              */
 /******************************************************************************/
 
+// Send message to first eligible node!
+
 int XrdCmsCluster::Broadsend(SMask_t Who, XrdCms::CmsRRHdr &Hdr, 
                              void *Data, int Dlen)
 {
@@ -461,21 +463,23 @@ int XrdCmsCluster::Broadsend(SMask_t Who, XrdCms::CmsRRHdr &Hdr,
 // Obtain a lock on the table and get the starting and ending position. Note
 // that the mechnism we use will necessarily skip newly added nodes.
 //
-   STMutex.Lock();
+   STMutex.ReadLock(); // Sufficient to prevent modifications
    Beg = Start = (Start <= STHi ? Start+1 : 0);
    Fin = STHi;
 
-// Run through the table looking for nodes to send messages to. We don't need
+// Run through the table looking for a node to send a message to. We don't need
 // the node lock for this but we do need to up the reference count to keep the
 // node pointer valid for the duration of the send() (may or may not block).
 //
 do{for (i = Beg; i <= Fin; i++)
        {if ((nP = NodeTab[i]) && nP->isNode(Who))
            {if (nP->isOffline) continue;
-            nP->g2Ref(STMutex);
-            if (nP->Send(ioV, 2, ioTot) >= 0) {nP->UnLock(); return 1;}
+            nP->Ref();
+            STMutex.UnLock();
+            if (nP->Send(ioV, 2, ioTot) >= 0) {nP->unRef(); return 1;}
             DEBUG(nP->Ident <<" is unreachable");
-            nP->Ref2g(STMutex);
+            nP->unRef();
+            STMutex.ReadLock();
            }
        }
     if (!Beg) break;
@@ -500,7 +504,7 @@ SMask_t XrdCmsCluster::getMask(const XrdNetAddr *addr)
 
 // Obtain a lock on the table
 //
-   STMutex.Lock();
+   STMutex.ReadLock();
 
 // Run through the table looking for a node with matching IP address
 //
@@ -540,7 +544,7 @@ XrdCmsSelected *XrdCmsCluster::List(SMask_t mask, CmsLSOpts opts, bool &oksel)
 // If only one wanted, the select appropriately
 //
    oksel = false;
-   STMutex.Lock();
+   STMutex.ReadLock();
    for (i = 0; i <= STHi; i++)
         if ((nP=NodeTab[i]) && (nP->NodeMask & mask))
            {oksel = true;
@@ -701,34 +705,42 @@ void *XrdCmsCluster::MonPerf()
   
 void *XrdCmsCluster::MonRefs()
 {
-   int  snooze_interval = 10*60, loopmax, loopcnt = 0;
-   int resetW, resetR, resetWR;
-
-// Compute snooze interval
-//
-   if ((loopmax = Config.RefReset / snooze_interval) <= 1)
-      {if (!Config.RefReset) loopmax = 0;
-          else {loopmax = 1; snooze_interval = Config.RefReset;}
-      }
+   XrdCmsNode *nP;
+   int  snooze_interval = 60, snooze_total = 0;
+   int  rCnt = 0, wCnt = 0;
+   bool resetW, resetR, resetRW;
 
 // Sleep for the snooze interval. If a reset was requested then do a selective
 // reset unless we reached our snooze maximum and enough selections have gone
 // by; in which case, do a global reset.
 //
    do {XrdSysTimer::Snooze(snooze_interval);
-       loopcnt++;
-       STMutex.Lock();
-       resetW  = (SelWcnt >= Config.RefTurn);
-       resetR  = (SelRcnt >= Config.RefTurn);
-       resetWR = (loopmax && loopcnt >= loopmax && (resetW || resetR));
-       if (resetWR)
-          {ResetRef((SMask_t)0, true);
-           if (resetW) {SelWtot += SelWcnt; SelWcnt = 0;}
-           if (resetR) {SelRtot += SelRcnt; SelRcnt = 0;}
-           loopcnt = 0;
-          }
+       int totR = 0, totW = 0;
+
+       STMutex.ReadLock();
+       for (int i = 0; i <= STHi; i++)
+           {if ((nP = NodeTab[i]))
+               {totR += nP->RefTotR;
+                totW += nP->RefTotW;
+               }
+           }
        STMutex.UnLock();
+
+       rCnt += (totR - SelRtot); SelRtot = totR;
+       wCnt += (totW - SelWtot); SelWtot = totW;
+       snooze_total += snooze_interval;
+
+       resetR  = (rCnt >= Config.RefTurn);
+       resetW  = (wCnt >= Config.RefTurn);
+       resetRW = (snooze_total >= Config.RefReset && (resetW || resetR));
+       if (resetRW)
+          {ResetRef((SMask_t)0);
+           if (resetR) rCnt = 0;
+           if (resetW) wCnt = 0;
+           snooze_total = 0;
+          }
       } while(1);
+
    return (void *)0;
 }
 
@@ -747,30 +759,32 @@ void XrdCmsCluster::Remove(XrdCmsNode *theNode)
 // Warning! The node object must be locked upon entry. The lock is released
 //          prior to returning to the caller. This entry obtains the node
 //          table lock. When immed != 0 then the node is immediately dropped.
-//          When immed if < 0 then the caller already holds the STMutex and it 
-//          is not released upon exit.
+//          When immed if < 0 then the caller already holds the STMutex in
+//          write mode and it is not released upon exit.
 
 void XrdCmsCluster::Remove(const char *reason, XrdCmsNode *theNode, int immed)
 {
    EPNAME("Remove_Node")
    struct theLocks
-          {XrdSysMutex *myMutex;
-           XrdCmsNode  *myNode;
-           int          myNID;
-           int          myInst;
-           bool         hasLK;
-           bool         doDrop;
-           char         myIdent[510];
+          {XrdSysRWLock *myMutex;
+           XrdCmsNode   *myNode;
+           int           myNID;
+           int           myInst;
+           bool          hasLK;
+           bool          doDrop;
+           char          myIdent[510];
 
-                       theLocks(XrdSysMutex *mtx, XrdCmsNode *node, int immed)
+                       theLocks(XrdSysRWLock *mtx, XrdCmsNode *node, int immed)
                                : myMutex(mtx), myNode(node), hasLK(immed < 0),
                                  doDrop(false)
                                {strlcpy(myIdent, node->Ident, sizeof(myIdent));
                                 myNID = node->ID(myInst);
                                 if (!hasLK)
-                                   {myNode->UnLock();
-                                    myMutex->Lock(); // Get global lock
-                                    myNode->Lock(true);
+                                   {myNode->Ref();        // Keep alive
+                                    myNode->UnLock();
+                                    myMutex->WriteLock(); // Get global lock
+                                    myNode->Lock();
+                                    myNode->unRef();      // Can't escape now
                                    }
                                }
                       ~theLocks()
@@ -802,7 +816,7 @@ void XrdCmsCluster::Remove(const char *reason, XrdCmsNode *theNode, int immed)
 
 // Mark node as being offline and remove any drop job from it
 //
-   theNode->isOffline = 1; // STMutex is held here
+   theNode->isOffline = 1; // Global lock is held here
 
 // If the node is connected we simply close the connection. This will cause
 // the connection handler to re-initiate the node removal. This condition
@@ -891,9 +905,10 @@ void XrdCmsCluster::ResetRef(SMask_t nMask, bool isLocked)
 
 // Obtain a lock on the table if not already locked
 //
-   if (!isLocked) STMutex.Lock();
+   if (!isLocked) STMutex.ReadLock();
 
-// Reset reference counts as needed
+// Reset reference counts as needed. We can do this with a read lock as the
+// reference counters are atomic.
 //
    for (int i = 0; i <= STHi; i++)
        {if ((nP = NodeTab[i]) && (doAll || nP->isNode(nMask)))
@@ -1093,13 +1108,13 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
 //
    selR.selPack = 0;
 
-// If we are exporting a shared-everything system then the incomming mask
+// If we are exporting a shared-everything system then the incoming mask
 // may have more than one server indicated. So, we need to do a full select.
 // This is forced when isMulti is true, indicating a choice may exist. Note
 // that the node, if any, is returned unlocked but we have the global mutex.
 //
    if (isMulti || baseFS.isDFS())
-      {STMutex.Lock();
+      {STMutex.ReadLock();
        nP = (Config.sched_RR ? SelbyRef(pmask,selR) : SelbyLoad(pmask,selR));
        if (nP) hlen = nP->netIF.GetName(hbuff, port, nType) + 1;
           else hlen = 0;
@@ -1107,7 +1122,7 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
        return hlen != 1;
       }
 
-// In shared-nothing systems the incomming mask will only have a single node.
+// In shared-nothing systems the incoming mask will only have a single node.
 // Compute the a single node number that is contained in the mask.
 //
    do {if (!(tmask = pmask & smLow)) Snum += 8;
@@ -1116,7 +1131,7 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
 
 // See if the node passes muster
 //
-   STMutex.Lock();
+   STMutex.ReadLock();
    if ((nP = NodeTab[Snum]))
       {     if (nP->isBad) nP = 0;
        else if (!Config.sched_RR && (nP->myLoad > Config.MaxLoad)) nP = 0;
@@ -1124,8 +1139,8 @@ int XrdCmsCluster::Select(SMask_t pmask, int &port, char *hbuff, int &hlen,
        if (nP)
           {if (isrw)
               if (nP->isNoStage || nP->DiskFree < nP->DiskMinF)    nP = 0;
-                 else {SelWcnt++; nP->RefTotW++; nP->RefW++;}
-              else    {SelRcnt++; nP->RefTotR++; nP->RefR++;}
+                 else {nP->RefTotW++; nP->RefW++;}
+              else    {nP->RefTotR++; nP->RefR++;}
           }
       }
 
@@ -1202,7 +1217,7 @@ void XrdCmsCluster::Space(SpaceData &sData, SMask_t smask)
 
 // Obtain a lock on the table and screen out peer nodes
 //
-   STMutex.Lock();
+   STMutex.ReadLock();
    bmask = smask & peerMask;
 
 // Run through the table getting space information
@@ -1276,7 +1291,6 @@ int XrdCmsCluster::Statt(char *bfr, int bln)
 
    XrdCmsRRQ::Info Frq;
    XrdCmsSelected *sp;
-   long long SelRnum, SelWnum;
    int mlen, tlen, n = 0;
    char shrBuff[80], stat[6], *stp;
    bool oksel;
@@ -1311,17 +1325,11 @@ int XrdCmsCluster::Statt(char *bfr, int bln)
    while(sp) {n++; sp = sp->next;}
    sp = mngrsp.sp;
 
-// Gather totals from the running total and the current value
-//
-   STMutex.Lock();
-   SelRnum = SelRtot + SelRcnt;
-   SelWnum = SelWtot + SelWcnt;
-   STMutex.UnLock();
-
 // Format the statistics
 //
+   long long lclTcnt = SelTcnt, lclRtot = SelRtot, lclWtot = SelWtot;
    mlen = snprintf(bfr, bln, statfmt1,
-          Config.myRType, SelTcnt, SelRnum, SelWnum, n);
+          Config.myRType, lclTcnt, lclRtot, lclWtot, n);
 
    if ((bln -= mlen) <= 0) return 0;
    tlen = mlen; bfr += mlen; n = 0; *shrBuff = 0;
@@ -1397,10 +1405,10 @@ XrdCmsNode *XrdCmsCluster::calcDelay(XrdCmsSelector &selR)
 /*                                  D r o p                                   */
 /******************************************************************************/
   
-// Warning: STMutex must be locked upon entry and the caller must release it
-//          if this method is called directily. Otherwise, the mutex will be
-//          obtained and released. Also, this method may only be called via
-//          Remove() either directly or via a defered job scheduled by that
+// Warning: STMutex must be locked in write upon entry and the caller must
+//          release it if this method is called directily. Otherwise, the mutex
+//          will be obtained and released. Also, this method may only be called
+//          via Remove() either directly or via a deferred job scheduled by that
 //          method. This method actually deletes the node object.
 
 int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
@@ -1411,7 +1419,7 @@ int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
 
 // If we are being called outside of a scheduled job, obtain the mutex
 //
-   if (djp) STMutex.Lock();
+   if (djp) STMutex.WriteLock();
 
 // Make sure this node is the right one
 //
@@ -1437,7 +1445,7 @@ int XrdCmsCluster::Drop(int sent, int sinst, XrdCmsDrop *djp)
 // Cleanup status
 //
    NodeTab[sent] = 0;
-   nP->isOffline = 1; // STMutex is locked
+   nP->isOffline = 1; // STMutex is locked in write mode
    nP->DropTime  = 0;
    nP->DropJob   = 0;
    nP->isBound   = 0;
@@ -1549,7 +1557,7 @@ void XrdCmsCluster::Record(char *path, const char *reason, bool force)
    msgcnt++; skipmsg = msgcnt & (force ? 0x0f : 0xff);
    mcMutex.UnLock();
 
-   if (!skipmsg) Say.Emsg(epname, "client defered;", reason, path);
+   if (!skipmsg) Say.Emsg(epname, "client deferred;", reason, path);
 }
  
 /******************************************************************************/
@@ -1573,9 +1581,11 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 // Indicate whether or not stable selection is required
 //
    if (!(Sel.Opts & XrdCmsSelect::Pack)) selR.selPack = 0;
-      else {SMask_t sVec = pmask;
+      else {unsigned int theHash = (Sel.Opts & XrdCmsSelect::UseAH
+                                 ?  Sel.AltHash : Sel.Path.Hash);
+            SMask_t sVec = pmask;
             for (count = 0; sVec; count++) sVec &= (sVec - 1);
-            if (count > 1) selR.selPack = affsel = (Sel.Path.Hash % count) + 1;
+            if (count > 1) selR.selPack = affsel = (theHash % count) + 1;
                else        selR.selPack = 0;
            }
 
@@ -1588,9 +1598,9 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 
 // Scan for a primary and alternate node (alternates do staging). At this
 // point we omit all peer nodes as they are our last resort. Note that Selbyxxx
-// returns the node unlocked but we have he global mutex so that is OK.
+// returns the node unlocked but we have the global mutex so that is OK.
 //
-   STMutex.Lock();
+   STMutex.ReadLock();
    mask = pmask & peerMask;
    while(pass--)
         {if (mask)
@@ -1625,7 +1635,7 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
        if (Sel.iovN && Sel.iovP) nP->Send(Sel.iovP, Sel.iovN);
 
        // Do special post proccessing when any of:
-       // a) isalt true: Secondary selection occured
+       // a) isalt true: Secondary selection occurred
        // b) Create set: File creation will occur
        //
        if (isalt || (Sel.Opts & XrdCmsSelect::Create))
@@ -1710,8 +1720,8 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 // want to execute this inline.
 //
 #define RefCount(sP, sPMulti, NeedSpace)                       \
-        if (NeedSpace) {SelWcnt++; sP->RefTotW++; sP->RefW++;} \
-           else        {SelRcnt++; sP->RefTotR++; sP->RefR++;} \
+        if (NeedSpace) {sP->RefTotW++; sP->RefW++;} \
+           else        {sP->RefTotR++; sP->RefR++;} \
         if (sPMulti && sP->Share && !sP->Shrem--)              \
            {sP->RefW += sP->Shrip; sP->RefR += sP->Shrip;      \
             sP->Shrem = sP->Share; sP->Shrin++;                \
@@ -1724,7 +1734,7 @@ int XrdCmsCluster::SelNode(XrdCmsSelect &Sel, SMask_t pmask, SMask_t amask)
 // Cost selection is used only for peer node selection as peers do not
 // report a load and handle their own scheduling.
 
-// Caller must have the STMutex locked. The returned node. if any, is unlocked.
+// Caller must have the STMutex locked. The returned node, if any, is unlocked.
 
 XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
 {
@@ -1769,7 +1779,7 @@ XrdCmsNode *XrdCmsCluster::SelbyCost(SMask_t mask, XrdCmsSelector &selR)
 /*                             S e l b y L o a d                              */
 /******************************************************************************/
 
-// Caller must have the STMutex locked. The returned node. if any, is unlocked.
+// Caller must have the STMutex locked. The returned node, if any, is unlocked.
   
 XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
 {
@@ -1819,7 +1829,7 @@ XrdCmsNode *XrdCmsCluster::SelbyLoad(SMask_t mask, XrdCmsSelector &selR)
 /*                              S e l b y R e f                               */
 /******************************************************************************/
 
-// Caller must have the STMutex locked. The returned node. if any, is unlocked.
+// Caller must have the STMutex locked. The returned node, if any, is unlocked.
 
 XrdCmsNode *XrdCmsCluster::SelbyRef(SMask_t mask, XrdCmsSelector &selR)
 {
@@ -1921,7 +1931,7 @@ int XrdCmsCluster::SelDFS(XrdCmsSelect &Sel, SMask_t amask,
 /*                             s e n d A L i s t                              */
 /******************************************************************************/
   
-// Single entry at a time, protected by STMutex!
+// Single entry at a time, protected by STMutex in write mode!
 
 void XrdCmsCluster::sendAList(XrdLink *lp)
 {
@@ -1963,7 +1973,7 @@ void XrdCmsCluster::sendAList(XrdLink *lp)
 /*                             s e t A l t M a n                              */
 /******************************************************************************/
   
-// Single entry at a time, protected by STMutex!
+// Single entry at a time, protected by STMutex in write mode!
   
 void XrdCmsCluster::setAltMan(int snum, XrdLink *lp, int port)
 {
