@@ -25,8 +25,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /*                                                                            */
 /******************************************************************************/
-#include <time.h>
-#include <string.h>
+#include <ctime>
+#include <cstring>
 
 #include "XrdCrypto/XrdCryptoX509Chain.hh"
 #include "XrdCrypto/XrdCryptoTrace.hh"
@@ -44,7 +44,7 @@
 
 // Description of errors
 static const char *X509ChainErrStr[] = {
-   "no error condition occured",         // 0
+   "no error condition occurred",         // 0
    "chain is inconsistent",              // 1
    "size exceeds max allowed depth",     // 2
    "invalid or missing CA",              // 3
