@@ -31,9 +31,9 @@
 /* OpenSSL utility functions                                                  */
 /*                                                                            */
 /* ************************************************************************** */
-#include <time.h>
-#include <errno.h>
-#include <stdlib.h>
+#include <ctime>
+#include <cerrno>
+#include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -297,6 +297,26 @@ XrdSutBucket *XrdCryptosslX509ExportChain(XrdCryptoX509Chain *chain,
    return bck;
 }
 
+//____________________________________________________________________________
+int XrdCryptosslX509ToFile(XrdCryptoX509 *x509, FILE *file, const char *fname)
+{
+   // Dump a single X509 certificate to a file in PEM format.
+   EPNAME("X509ChainToFile");
+
+   // Check inputs
+   if (!x509 || !file) {
+      DEBUG("Invalid inputs");
+      return -1;
+   }
+
+   if (PEM_write_X509(file, (X509 *)x509->Opaque()) != 1) {
+      DEBUG("error while writing certificate " << fname);
+      return -1;
+   }
+
+   return 0;
+}
+
 //____________________________________________________________________________
 int XrdCryptosslX509ChainToFile(XrdCryptoX509Chain *ch, const char *fn)
 {
@@ -432,7 +452,28 @@ int XrdCryptosslX509ParseStack(XrdTlsPeerCerts* pc, XrdCryptoX509Chain *chain)
 
 //____________________________________________________________________________
 int XrdCryptosslX509ParseFile(const char *fname,
-                              XrdCryptoX509Chain *chain)
+                              XrdCryptoX509Chain *chain, const char *fkey)
+{
+   EPNAME("X509ParseFile");
+
+   //
+   // Open file and read the content:
+   // it should contain blocks on information in PEM form
+   FILE *fcer = fopen(fname, "r");
+   if (!fcer) {
+      DEBUG("unable to open file (errno: "<<errno<<")");
+      return 0;
+   }
+
+   auto retval = XrdCryptosslX509ParseFile(fcer, chain, fname, fkey);
+   fclose(fcer);
+   return retval;
+}
+
+//____________________________________________________________________________
+int XrdCryptosslX509ParseFile(FILE *fcer,
+                              XrdCryptoX509Chain *chain,
+                              const char *fname, const char *fkey)
 {
    // Parse content of file 'fname' and add X509 certificates to
    // chain (which must be initialized by the caller).
@@ -441,9 +482,9 @@ int XrdCryptosslX509ParseFile(const char *fname,
    EPNAME("X509ParseFile");
    int nci = 0;
 
-   // Make sure we got a file to import
-   if (!fname) {
-      DEBUG("file name undefined: can do nothing");
+   // Make sure we got a valid file
+   if (!fcer) {
+      DEBUG("FILE object undefined: can do nothing");
       return nci;
    }
 
@@ -453,15 +494,6 @@ int XrdCryptosslX509ParseFile(const char *fname,
       return nci;
    }
 
-   //
-   // Open file and read the content:
-   // it should contain blocks on information in PEM form
-   FILE *fcer = fopen(fname, "r");
-   if (!fcer) {
-      DEBUG("unable to open file (errno: "<<errno<<")");
-      return nci;
-   }
-
    // Now read out certificates and add them to the chain
    X509 *xcer = 0;
    while (PEM_read_X509(fcer, &xcer, 0, 0)) {
@@ -482,13 +514,27 @@ int XrdCryptosslX509ParseFile(const char *fname,
    // If we found something, and we are asked to extract a key,
    // rewind and look for it
    if (nci) {
-      rewind(fcer);
+      FILE *fcersave = 0;
+      if (!fkey) {
+         // Look in the same file, after rewinding
+         rewind(fcer);
+      } else {
+         // We can close the file now
+         fcersave = fcer;
+	       // Open key file
+         fcer = fopen(fkey, "r");
+         if (!fcer) {
+           DEBUG("unable to open key file (errno: "<<errno<<")");
+           fcer = fcersave;
+           return nci;
+         }
+      }
       RSA  *rsap = 0;
       if (!PEM_read_RSAPrivateKey(fcer, &rsap, 0, 0)) {
-         DEBUG("no RSA private key found in file "<<fname);
+         DEBUG("no RSA private key found in file " << fname);
       } else {
-         DEBUG("found a RSA private key in file "<<fname);
-         // We need to complete the key: we save it temporarly
+         DEBUG("found a RSA private key in file " << fname);
+         // We need to complete the key: we save it temporarily
          // to a bio and check all the private keys of the
          // loaded certificates 
          bool ok = 1;
@@ -536,11 +582,13 @@ int XrdCryptosslX509ParseFile(const char *fname,
          // Cleanup
          BIO_free(bkey);
       }
+      if (fkey) {
+         // Re-establish original fcer pointer
+         fclose(fcer);
+         fcer = fcersave;
+      }
    }
 
-   // We can close the file now
-   fclose(fcer);
-
    // We are done
    return nci;
 }
@@ -608,7 +656,7 @@ int XrdCryptosslX509ParseBucket(XrdSutBucket *b, XrdCryptoX509Chain *chain)
          DEBUG("no RSA private key found in bucket ");
       } else {
          DEBUG("found a RSA private key in bucket ");
-         // We need to complete the key: we save it temporarly
+         // We need to complete the key: we save it temporarily
          // to a bio and check all the private keys of the
          // loaded certificates 
          bool ok = 1;
