@@ -31,7 +31,7 @@
 /* GSI utility functions                                                      */
 /*                                                                            */
 /* ************************************************************************** */
-#include <string.h>
+#include <cstring>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -381,6 +381,7 @@ int XrdCryptosslX509CreateProxy(const char *fnc, const char *fnk,
       PRINT("could not set subject name - return");
       return -kErrPX_SetAttribute;
    }
+
    //
    // Create the extension CertProxyInfo
    PROXY_CERT_INFO_EXTENSION *pci = PROXY_CERT_INFO_EXTENSION_new();
@@ -477,6 +478,7 @@ int XrdCryptosslX509CreateProxy(const char *fnc, const char *fnk,
       PRINT("could not set subject name");
       return -kErrPX_SetAttribute;
    }
+   X509_NAME_free(psubj);
 
    // Set issuer name
    if (X509_set_issuer_name(xPX, X509_get_subject_name(xEEC)) != 1) {
@@ -731,6 +733,7 @@ int XrdCryptosslX509CreateProxyReq(XrdCryptoX509 *xcpi,
       PRINT("could not set subject name - return");
       return -kErrPX_SetAttribute;
    }
+   X509_NAME_free(psubj);
    //
    // Create the extension CertProxyInfo
    PROXY_CERT_INFO_EXTENSION *pci = PROXY_CERT_INFO_EXTENSION_new();
@@ -856,7 +859,7 @@ int XrdCryptosslX509CreateProxyReq(XrdCryptoX509 *xcpi,
 
    // Cleanup
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
-   sk_X509_EXTENSION_free(esk);
+   sk_X509_EXTENSION_pop_free(esk, X509_EXTENSION_free);
 #else /* OPENSSL */
    sk_free(esk);
 #endif /* OPENSSL */
@@ -1045,6 +1048,7 @@ int XrdCryptosslX509SignProxyReq(XrdCryptoX509 *xcpi, XrdCryptoRSA *kcpi,
          // Notify what we added
          int crit = X509_EXTENSION_get_critical(xpiextdup);
          DEBUG("added extension '"<<s<<"', critical: " << crit);
+         X509_EXTENSION_free( xpiextdup );
       }
       // Do not free the extension: its owned by the certificate
       xpiext = 0;
@@ -1116,12 +1120,16 @@ int XrdCryptosslX509SignProxyReq(XrdCryptoX509 *xcpi, XrdCryptoRSA *kcpi,
       PRINT("problem converting data for extension");
       return -kErrPX_Error;
    }
+   PROXY_CERT_INFO_EXTENSION_free( pci );
+
    // Set extension name.
    ASN1_OBJECT *obj = OBJ_txt2obj(gsiProxyCertInfo_OID, 1);
    if (!obj || X509_EXTENSION_set_object(ext, obj) != 1) {
       PRINT("could not set extension name");
       return -kErrPX_SetAttribute;
    }
+   ASN1_OBJECT_free( obj );
+
    // flag as critical
    if (X509_EXTENSION_set_critical(ext, 1) != 1) {
       PRINT("could not set extension critical flag");
@@ -1141,6 +1149,9 @@ int XrdCryptosslX509SignProxyReq(XrdCryptoX509 *xcpi, XrdCryptoRSA *kcpi,
       return -kErrPX_Signing;
    }
 
+   EVP_PKEY_free( ekpi ); // decrement reference counter
+   X509_EXTENSION_free( ext );
+
    // Prepare outputs
    *xcpo = new XrdCryptosslX509(xpo);
 
@@ -1160,7 +1171,7 @@ int XrdCryptosslX509GetVOMSAttr(XrdCryptoX509 *xcpi, XrdOucString &vat)
 {
    // Get VOMS attributes from the certificate, if present
    // Return 0 in case of success, 1 if VOMS info is not available, < 0 if any
-   // error occured
+   // error occurred
    EPNAME("X509GetVOMSAttr");
 
    int rc = -1;
