@@ -37,6 +37,7 @@
 #include "XrdCl/XrdClParallelOperation.hh"
 #include "XrdCl/XrdClZipOperations.hh"
 #include "XrdCl/XrdClFileOperations.hh"
+#include "XrdCl/XrdClFinalOperation.hh"
 
 #include <algorithm>
 #include <iterator>
@@ -50,7 +51,8 @@ namespace XrdEc
   //---------------------------------------------------------------------------
   template<bool HasHndl>
   class OpenOnlyImpl: public XrdCl::ZipOperation<OpenOnlyImpl, HasHndl,
-      XrdCl::Resp<void>, XrdCl::Arg<std::string>>
+      XrdCl::Resp<void>, XrdCl::Arg<std::string>, XrdCl::Arg<bool>>
+
   {
     public:
 
@@ -58,12 +60,12 @@ namespace XrdEc
       // Inherit constructors from FileOperation (@see FileOperation)
       //-----------------------------------------------------------------------
       using XrdCl::ZipOperation<OpenOnlyImpl, HasHndl, XrdCl::Resp<void>,
-          XrdCl::Arg<std::string>>::ZipOperation;
+          XrdCl::Arg<std::string>, XrdCl::Arg<bool>>::ZipOperation;
 
       //-----------------------------------------------------------------------
       // Argument indexes in the args tuple
       //-----------------------------------------------------------------------
-      enum { UrlArg };
+      enum { UrlArg, UpdtArg };
 
       //-----------------------------------------------------------------------
       // @return : name of the operation (@see Operation)
@@ -86,9 +88,10 @@ namespace XrdEc
                                    uint16_t                pipelineTimeout )
       {
         std::string      url     = std::get<UrlArg>( this->args ).Get();
+        bool             updt    = std::get<UpdtArg>( this->args ).Get();
         uint16_t         timeout = pipelineTimeout < this->timeout ?
                                    pipelineTimeout : this->timeout;
-        return this->zip->OpenOnly( url, handler, timeout );
+        return this->zip->OpenOnly( url, updt, handler, timeout );
       }
   };
 
@@ -97,9 +100,11 @@ namespace XrdEc
   //---------------------------------------------------------------------------
   inline OpenOnlyImpl<false> OpenOnly( XrdCl::Ctx<XrdCl::ZipArchive> zip,
                                        XrdCl::Arg<std::string>       fn,
+                                       XrdCl::Arg<bool>              updt,
                                        uint16_t                      timeout = 0 )
   {
-    return OpenOnlyImpl<false>( std::move( zip ), std::move( fn ) ).Timeout( timeout );
+    return OpenOnlyImpl<false>( std::move( zip ), std::move( fn ),
+                                std::move( updt ) ).Timeout( timeout );
   }
 
   //-------------------------------------------------------------------------
@@ -138,13 +143,15 @@ namespace XrdEc
     // @param usrbuff  : user buffer for the data
     // @param usrcb    : user callback to be notified when the read operation
     //                   has been resolved
+    // @param timeout  : operation timeout
     //-----------------------------------------------------------------------
     static void read( std::shared_ptr<block_t> &self,
                       size_t                    strpid,
                       uint64_t                  offset,
                       uint32_t                  size,
                       char                     *usrbuff,
-                      callback_t                usrcb )
+                      callback_t                usrcb,
+                      uint16_t                  timeout )
     {
       std::unique_lock<std::mutex> lck( self->mtx );
 
@@ -154,7 +161,7 @@ namespace XrdEc
       if( self->state[strpid] == Empty )
       {
         self->reader.Read( self->blkid, strpid, self->stripes[strpid],
-                           read_callback( self, strpid ) );
+                           read_callback( self, strpid ), timeout );
         self->state[strpid] = Loading;
       }
       //---------------------------------------------------------------------
@@ -230,7 +237,7 @@ namespace XrdEc
       if( missingcnt + recoveringcnt == 0 ) return true;
       //---------------------------------------------------------------------
       // Check if we can do the recovery at all (if too many stripes are
-      // missing it wont be possible)
+      // missing it won't be possible)
       //---------------------------------------------------------------------
       if( missingcnt + recoveringcnt > self->objcfg.nbparity )
       {
@@ -418,7 +425,7 @@ namespace XrdEc
   //---------------------------------------------------------------------------
   // Open the erasure coded / striped object
   //---------------------------------------------------------------------------
-  void Reader::Open( XrdCl::ResponseHandler *handler )
+  void Reader::Open( XrdCl::ResponseHandler *handler, uint16_t timeout )
   {
     const size_t size = objcfg.plgr.size();
     std::vector<XrdCl::Pipeline> opens; opens.reserve( size );
@@ -427,33 +434,47 @@ namespace XrdEc
       // generate the URL
       std::string url = objcfg.GetDataUrl( i );
       // create the file object
-      dataarchs.emplace( url, std::make_shared<XrdCl::ZipArchive>() );
+      dataarchs.emplace( url, std::make_shared<XrdCl::ZipArchive>(
+          Config::Instance().enable_plugins ) );
       // open the archive
-      opens.emplace_back( OpenOnly( *dataarchs[url], url ) );
+      if( objcfg.nomtfile )
+      {
+        opens.emplace_back( XrdCl::OpenArchive( *dataarchs[url], url, XrdCl::OpenFlags::Read ) );
+      }
+      else
+        opens.emplace_back( OpenOnly( *dataarchs[url], url, false ) );
     }
+
+    auto pipehndl = [=]( const XrdCl::XRootDStatus &st )
+                    { // set the central directories in ZIP archives (if we use metadata files)
+                      auto itr = dataarchs.begin();
+                      for( ; itr != dataarchs.end() ; ++itr )
+                      {
+                        const std::string &url    = itr->first;
+                        auto              &zipptr = itr->second;
+                        if( zipptr->openstage == XrdCl::ZipArchive::NotParsed )
+                          zipptr->SetCD( metadata[url] );
+                        else if( zipptr->openstage != XrdCl::ZipArchive::Done && !metadata.empty() )
+                          AddMissing( metadata[url] );
+                        auto itr = zipptr->cdmap.begin();
+                        for( ; itr != zipptr->cdmap.end() ; ++itr )
+                        {
+                          urlmap.emplace( itr->first, url );
+                          size_t blknb = fntoblk( itr->first );
+                          if( blknb > lstblk ) lstblk = blknb;
+                        }
+                      }
+                      metadata.clear();
+                      // call user handler
+                      if( handler )
+                        handler->HandleResponse( new XrdCl::XRootDStatus( st ), nullptr );
+                    };
     // in parallel open the data files and read the metadata
-    XrdCl::Pipeline p = XrdCl::Parallel( ReadMetadata( 0 ), XrdCl::Parallel( opens ).AtLeast( objcfg.nbdata ) ) >>
-                          [=]( XrdCl::XRootDStatus &st )
-                          { // set the central directories in ZIP archives
-                            auto itr = dataarchs.begin();
-                            for( ; itr != dataarchs.end() ; ++itr )
-                            {
-                              const std::string &url    = itr->first;
-                              auto              &zipptr = itr->second;
-                              if( zipptr->openstage == XrdCl::ZipArchive::NotParsed )
-                                zipptr->SetCD( metadata[url] );
-                              else
-                                AddMissing( metadata[url] );
-                              auto itr = zipptr->cdmap.begin();
-                              for( ; itr != zipptr->cdmap.end() ; ++itr )
-                                urlmap.emplace( itr->first, url );
-                            }
-                            metadata.clear();
-                            // call user handler
-                            if( handler )
-                              handler->HandleResponse( new XrdCl::XRootDStatus( st ), nullptr );
-                          };
-    XrdCl::Async( std::move( p ) );
+    XrdCl::Pipeline p = objcfg.nomtfile
+                      ? XrdCl::Parallel( opens ).AtLeast( objcfg.nbdata ) | ReadSize( 0 ) | XrdCl::Final( pipehndl )
+                      : XrdCl::Parallel( ReadMetadata( 0 ),
+                                         XrdCl::Parallel( opens ).AtLeast( objcfg.nbdata ) ) >> pipehndl;
+    XrdCl::Async( std::move( p ), timeout );
   }
 
   //-----------------------------------------------------------------------
@@ -462,7 +483,8 @@ namespace XrdEc
   void Reader::Read( uint64_t                offset,
                      uint32_t                length,
                      void                   *buffer,
-                     XrdCl::ResponseHandler *handler )
+                     XrdCl::ResponseHandler *handler,
+                     uint16_t                timeout )
   {
     char *usrbuff = reinterpret_cast<char*>( buffer );
     typedef std::tuple<uint64_t, uint32_t,
@@ -484,12 +506,14 @@ namespace XrdEc
       //-------------------------------------------------------------------
       // Make sure we operate on a valid block
       //-------------------------------------------------------------------
+      std::unique_lock<std::mutex> lck( blkmtx );
       if( !block || block->blkid != blkid )
         block = std::make_shared<block_t>( blkid, *this, objcfg );
       //-------------------------------------------------------------------
       // Prepare the callback for reading from single stripe
       //-------------------------------------------------------------------
       auto blk = block;
+      lck.unlock();
       auto callback = [blk, rdctx, rdsize, rdmtx]( const XrdCl::XRootDStatus &st, uint32_t nbrd )
       {
         std::unique_lock<std::mutex> lck( *rdmtx );
@@ -527,7 +551,7 @@ namespace XrdEc
       //-------------------------------------------------------------------
       // Read data from a stripe
       //-------------------------------------------------------------------
-      block_t::read( block, strpid, rdoff, rdsize, usrbuff, callback );
+      block_t::read( blk, strpid, rdoff, rdsize, usrbuff, callback, timeout );
       //-------------------------------------------------------------------
       // Update absolute offset, read length, and user buffer
       //-------------------------------------------------------------------
@@ -540,7 +564,7 @@ namespace XrdEc
   //-----------------------------------------------------------------------
   // Close the data object
   //-----------------------------------------------------------------------
-  void Reader::Close( XrdCl::ResponseHandler *handler )
+  void Reader::Close( XrdCl::ResponseHandler *handler, uint16_t timeout )
   {
     //---------------------------------------------------------------------
     // prepare the pipelines ...
@@ -562,13 +586,13 @@ namespace XrdEc
     // if there is nothing to close just schedule the handler
     if( closes.empty() ) ScheduleHandler( handler );
     // otherwise close the archives
-    else XrdCl::Async( XrdCl::Parallel( closes ) >> handler );
+    else XrdCl::Async( XrdCl::Parallel( closes ) >> handler, timeout );
   }
 
   //-------------------------------------------------------------------------
   // on-definition is not allowed here beforeiven stripes from given block
   //-------------------------------------------------------------------------
-  void Reader::Read( size_t blknb, size_t strpnb, buffer_t &buffer, callback_t cb )
+  void Reader::Read( size_t blknb, size_t strpnb, buffer_t &buffer, callback_t cb, uint16_t timeout )
   {
     // generate the file name (blknb/strpnb)
     std::string fn = objcfg.GetFileName( blknb, strpnb );
@@ -576,7 +600,7 @@ namespace XrdEc
     auto itr = urlmap.find( fn );
     if( itr == urlmap.end() )
     {
-      auto st = !missing.count( fn ) ? XrdCl::XRootDStatus() :
+      auto st = !IsMissing( fn ) ? XrdCl::XRootDStatus() :
                 XrdCl::XRootDStatus( XrdCl::stError, XrdCl::errNotFound );
       ThreadPool::Instance().Execute( cb, st, 0 );
       return;
@@ -596,10 +620,10 @@ namespace XrdEc
     uint32_t rdsize = info->GetSize();
     delete info;
     // create a buffer for the data
-    buffer.resize( rdsize );
+    buffer.resize( objcfg.chunksize );
     // issue the read request
-    XrdCl::Async( XrdCl::ReadFrom( *zipptr, fn, 0, buffer.size(), buffer.data() ) >>
-                    [zipptr, fn, cb]( XrdCl::XRootDStatus &st, XrdCl::ChunkInfo &ch )
+    XrdCl::Async( XrdCl::ReadFrom( *zipptr, fn, 0, rdsize, buffer.data() ) >>
+                    [zipptr, fn, cb, this]( XrdCl::XRootDStatus &st, XrdCl::ChunkInfo &ch )
                     {
                       //---------------------------------------------------
                       // If read failed there's nothing to do, just pass the
@@ -627,7 +651,7 @@ namespace XrdEc
                       //---------------------------------------------------
                       // Verify data integrity
                       //---------------------------------------------------
-                      uint32_t cksum = crc32c( 0, ch.buffer, ch.length );
+                      uint32_t cksum = objcfg.digest( 0, ch.buffer, ch.length );
                       if( orgcksum != cksum )
                       {
                         cb( XrdCl::XRootDStatus( XrdCl::stError, XrdCl::errDataError ), 0 );
@@ -637,7 +661,7 @@ namespace XrdEc
                       // All is good, we can call now the user callback
                       //---------------------------------------------------
                       cb( XrdCl::XRootDStatus(), ch.length );
-                    } );
+                    }, timeout );
   }
 
   //-----------------------------------------------------------------------
@@ -647,7 +671,7 @@ namespace XrdEc
   {
     const size_t size = objcfg.plgr.size();
     // create the File object
-    auto file = std::make_shared<XrdCl::File>();
+    auto file = std::make_shared<XrdCl::File>( Config::Instance().enable_plugins );
     // prepare the URL for Open operation
     std::string url = objcfg.GetMetadataUrl( index );
     // arguments for the Read operation
@@ -702,6 +726,30 @@ namespace XrdEc
              } );
   }
 
+  //-----------------------------------------------------------------------
+  //! Read size from xattr
+  //!
+  //! @param index : placement's index
+  //-----------------------------------------------------------------------
+  XrdCl::Pipeline Reader::ReadSize( size_t index )
+  {
+    std::string url = objcfg.GetDataUrl( index );
+    return XrdCl::GetXAttr( dataarchs[url]->GetFile(), "xrdec.filesize" ) >>
+        [index, this]( XrdCl::XRootDStatus &st, std::string &size)
+        {
+          if( !st.IsOK() )
+          {
+            //-------------------------------------------------------------
+            // Check if we can recover the error or a diffrent location
+            //-------------------------------------------------------------
+            if( index + 1 < objcfg.plgr.size() )
+              XrdCl::Pipeline::Replace( ReadSize( index + 1 ) );
+            return;
+          }
+          filesize = std::stoull( size );
+        };
+  }
+
   //-----------------------------------------------------------------------
   // Parse metadata from chunk info object
   //-----------------------------------------------------------------------
@@ -727,10 +775,11 @@ namespace XrdEc
       buffer += lfh.lfhSize;
       length -= lfh.lfhSize;
       // verify the checksum
-      uint32_t crc32val = crc32c( 0, buffer, lfh.uncompressedSize );
+      uint32_t crc32val = objcfg.digest( 0, buffer, lfh.uncompressedSize );
       if( crc32val != lfh.ZCRC32 ) return false;
       // keep the metadata
-      metadata.emplace( lfh.filename, buffer_t( buffer, buffer + lfh.uncompressedSize ) );
+      std::string url = objcfg.GetDataUrl( std::stoull( lfh.filename ) );
+      metadata.emplace( url, buffer_t( buffer, buffer + lfh.uncompressedSize ) );
       buffer += lfh.uncompressedSize;
       length -= lfh.uncompressedSize;
     }
@@ -757,4 +806,18 @@ namespace XrdEc
     }
   }
 
+  //-----------------------------------------------------------------------
+  //! Check if chunk file name is missing
+  //-----------------------------------------------------------------------
+  bool Reader::IsMissing( const std::string &fn )
+  {
+    // if the chunk is in the missing set return true
+    if( missing.count( fn ) ) return true;
+    // if we don't have a metadata file and the chunk exceeds last chunk
+    // also return true
+    if( objcfg.nomtfile && fntoblk( fn ) <= lstblk ) return true;
+    // otherwise return false
+    return false;
+  }
+
 } /* namespace XrdEc */
