@@ -29,8 +29,8 @@
 
 #define FUSE_USE_VERSION 26
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <unistd.h>
 #include <stddef.h>
 
@@ -44,21 +44,21 @@
 #ifdef HAVE_FUSE
 #include <fuse.h>
 #include <fuse/fuse_opt.h>
-#include <ctype.h>
-#include <string.h>
+#include <cctype>
+#include <cstring>
 #include <fcntl.h>
 #include <dirent.h>
-#include <errno.h>
+#include <cerrno>
 #include <sys/time.h>
 #include <pthread.h>
 #include <pwd.h>
 #include <libgen.h>
 #include <syslog.h>
 #include <signal.h>
+#if defined(__linux__)
 #include <sys/prctl.h>
-#if !defined(__solaris__)
-#include <sys/xattr.h>
 #endif
+#include <sys/xattr.h>
 
 #include "XrdFfs/XrdFfsPosix.hh"
 #include "XrdFfs/XrdFfsMisc.hh"
@@ -120,7 +120,9 @@ static void* xrootdfs_init(struct fuse_conn_info *conn)
             syslog( LOG_ERR, "ERROR: Unable to set gid to %d", pw.pw_gid );
         if( setuid((uid_t)pw.pw_uid) != 0 )
             syslog( LOG_ERR, "ERROR: Unable to set uid to %d", pw.pw_uid );
+#if defined(__linux__)
         prctl(PR_SET_DUMPABLE, 1); // enable core dump after setuid/setgid
+#endif
     }
     free(pwbuf);
 
@@ -1184,7 +1186,7 @@ static int xrootdfs_removexattr(const char *path, const char *name)
 
 /*
  * We need this as casting function pointer to a function pointer
- * of a diffrent type and then calling it results in undefined
+ * of a different type and then calling it results in undefined
  * behavior (Annex J.2).
  *
  * GCC 8 available on Fedora 28 is very picky about those things.
