@@ -48,7 +48,7 @@
 #include <vector>
 #include <arpa/inet.h>
 #include <sstream>
-#include <ctype.h>
+#include <cctype>
 #include <sys/stat.h>
 #include <fcntl.h>
 
@@ -105,6 +105,13 @@ XrdSecService *XrdHttpProtocol::CIA = 0; // Authentication Server
 int XrdHttpProtocol::m_bio_type = 0; // BIO type identifier for our custom BIO.
 BIO_METHOD *XrdHttpProtocol::m_bio_method = NULL; // BIO method constructor.
 
+XrdSysTrace XrdHttpTrace("http");
+
+namespace
+{
+const char *TraceID = "Protocol";
+}
+
 namespace XrdHttpProtoInfo
 {
 XrdTlsContext *xrdctx = 0;
@@ -462,7 +469,7 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
 {
   int rc = 0;
 
-  TRACEI(DEBUG, " Process. lp:" << lp << " reqstate: " << CurrentReq.reqstate);
+  TRACEI(DEBUG, " Process. lp:"<<(void *)lp<<" reqstate: "<<CurrentReq.reqstate);
 
   if (!myBuff || !myBuff->buff || !myBuff->bsize) {
     TRACE(ALL, " Process. No buffer available. Internal error.");
@@ -576,7 +583,7 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
 
     // Read as many lines as possible into the buffer. An empty line breaks
     while ((rc = BuffgetLine(tmpline)) > 0) {
-      TRACE(DEBUG, " rc:" << rc << " got hdr line: " << tmpline);
+      TRACE(DEBUG, " rc:" << rc << " got hdr line: " << tmpline.c_str());
 
       if ((rc == 2) && (tmpline.length() > 1) && (tmpline[rc - 1] == '\n')) {
         CurrentReq.headerok = true;
@@ -586,7 +593,7 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
 
 
       if (CurrentReq.request == CurrentReq.rtUnset) {
-        TRACE(DEBUG, " Parsing first line: " << tmpline);
+        TRACE(DEBUG, " Parsing first line: " << tmpline.c_str());
         int result = CurrentReq.parseFirstLine((char *)tmpline.c_str(), rc);
         if (result < 0) {
           TRACE(DEBUG, " Parsing of first line failed with " << result);
@@ -687,7 +694,8 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
         dest += ":";
         dest += Port_str;
         dest += CurrentReq.resource.c_str();
-        TRACEI(REQ, " rc:" << rc << " self-redirecting to http with security token: '" << dest << "'");
+        TRACEI(REQ," rc:"<<rc<<" self-redirecting to http with security token: '"
+                   << dest.c_str() << "'");
 
         
         CurrentReq.appendOpaque(dest, &SecEntity, hash, timenow);
@@ -1407,19 +1415,39 @@ void XrdHttpProtocol::BuffConsume(int blen) {
 /******************************************************************************/
   
 /// Get a pointer, valid for up to blen bytes from the buffer. Returns the n
-/// of bytes that one is allowed to read
-
+/// of bytes that one is allowed to use in the *data block
+/// If wait = true then the call may wait for the data to come from the socket
+/// If wait = false then the call returns:
+///   - what's in the buffer if there's anything in the buffer
+///   - what can be read from the socket without waiting
+///   In this case an error will return -1, instead the absence of data
+///   will return 0
 int XrdHttpProtocol::BuffgetData(int blen, char **data, bool wait) {
   int rlen;
 
   TRACE(DEBUG, "BuffgetData: requested " << blen << " bytes");
-  
-  if (wait && (blen > BuffUsed())) {
-    TRACE(REQ, "BuffgetData: need to read " << blen - BuffUsed() << " bytes");
-    if ( getDataOneShot(blen - BuffUsed(), true) ) return 0;
+ 
+
+  if (wait) {
+    // If there's not enough data in the buffer then wait on the socket until it comes
+    if  (blen > BuffUsed()) {
+      TRACE(REQ, "BuffgetData: need to read " << blen - BuffUsed() << " bytes");
+      if ( getDataOneShot(blen - BuffUsed(), true) )
+        // The wanted data could not be read. Either timeout of connection closed
+        return 0;
+    }
+  } else {
+    // Get a peek at the socket, without waiting, if we have no data in the buffer
+    if ( !BuffUsed() ) {
+      if ( getDataOneShot(blen, false) )
+        // The wanted data could not be read. Either timeout of connection closed
+        return -1;
+    }
   }
 
-  if (myBuffStart < myBuffEnd) {
+  // And now make available the data taken from the buffer. Note that the buffer
+  // may be empty...
+  if (myBuffStart <= myBuffEnd) {
     rlen = min( (long) blen, (long)(myBuffEnd - myBuffStart) );
 
   } else
@@ -1589,7 +1617,7 @@ int XrdHttpProtocol::Configure(char *parms, XrdProtocol_Config * pi) {
   // Copy out the special info we want to use at top level
   //
   eDest.logger(pi->eDest->logger());
-  XrdHttpTrace = new XrdOucTrace(&eDest);
+  XrdHttpTrace.SetLogger(pi->eDest->logger());
   //  SI = new XrdXrootdStats(pi->Stats);
   Sched = pi->Sched;
   BPool = pi->BPool;
@@ -1612,7 +1640,7 @@ int XrdHttpProtocol::Configure(char *parms, XrdProtocol_Config * pi) {
   //
   rdf = (parms && *parms ? parms : pi->ConfigFN);
   if (rdf && Config(rdf, pi->theEnv)) return 0;
-  if (pi->DebugON) XrdHttpTrace->What = TRACE_ALL;
+  if (pi->DebugON) XrdHttpTrace.What = TRACE_ALL;
 
   // Set the redirect flag if we are a pure redirector
   myRole = kXR_isServer;
@@ -1633,7 +1661,8 @@ int XrdHttpProtocol::Configure(char *parms, XrdProtocol_Config * pi) {
 
   // Schedule protocol object cleanup
   //
-  ProtStack.Set(pi->Sched, XrdHttpTrace, TRACE_MEM);
+  ProtStack.Set(pi->Sched, &XrdHttpTrace,
+                (XrdHttpTrace.What & TRACE_MEM ? TRACE_MEM : 0));
   ProtStack.Set((pi->ConnMax / 3 ? pi->ConnMax / 3 : 30), 60 * 60);
 
   // Return success
@@ -2720,7 +2749,7 @@ int XrdHttpProtocol::xtrace(XrdOucStream & Config) {
     }
     val = Config.GetWord();
   }
-  XrdHttpTrace->What = trval;
+  XrdHttpTrace.What = trval;
   return 0;
 }
 
