@@ -40,7 +40,7 @@
 #include "XrdHttpReq.hh"
 #include "XrdHttpTrace.hh"
 #include "XrdHttpExtHandler.hh"
-#include <string.h>
+#include <cstring>
 #include <arpa/inet.h>
 #include <sstream>
 #include "XrdSys/XrdSysPlatform.hh"
@@ -66,6 +66,10 @@
 // This is to fix the trace macros
 #define TRACELINK prot->Link
 
+namespace
+{
+const char *TraceID = "Req";
+}
 
 static std::string convert_digest_rfc_name(const std::string &rfc_name_multiple)
 {
@@ -234,6 +238,12 @@ int XrdHttpReq::parseLine(char *line, int len) {
     // Trim left
     while ( (!isgraph(*val) || (!*val)) && (val < line+len)) val++;
 
+    // We memorize the headers also as a string                                                                                                                                              
+    // because external plugins may need to process it differently                                                                                                                          
+    std::string ss = val;
+    trim(ss);
+    allheaders[key] = ss;
+	  
     // Here we are supposed to initialize whatever flag or variable that is needed
     // by looking at the first token of the line
     // The token is key
@@ -287,11 +297,7 @@ int XrdHttpReq::parseLine(char *line, int len) {
       }
     }
 
-    // We memorize the heaers also as a string
-    // because external plugins may need to process it differently
-    std::string ss = val;
-    trim(ss);
-    allheaders[key] = ss;
+
     line[pos] = ':';
   }
 
@@ -633,8 +639,12 @@ bool XrdHttpReq::Error(XrdXrootd::Bridge::Context &info, //!< the result context
 
   xrdresp = kXR_error;
   xrderrcode = (XErrorCode) ecode;
-  this->etext = etext_;
 
+  if (etext_) {
+    char *s = escapeXML(etext_);
+    this->etext = s;
+    free(s);
+  }
 
   if (PostProcessHTTPReq()) reset();
 
@@ -728,7 +738,7 @@ bool XrdHttpReq::Redir(XrdXrootd::Bridge::Context &info, //!< the result context
     appendOpaque(redirdest, 0, 0, 0);
 
   
-  TRACE(REQ, " XrdHttpReq::Redir Redirecting to " << redirdest);
+  TRACE(REQ, " XrdHttpReq::Redir Redirecting to " << redirdest.c_str());
 
   prot->SendSimpleResp(302, NULL, (char *) redirdest.c_str(), 0, 0, keepalive);
 
@@ -983,7 +993,8 @@ int XrdHttpReq::ProcessHTTPReq() {
 
     char *q = quote(hdr2cgistr.c_str());
     resourceplusopaque.append(q);
-    TRACEI(DEBUG, "Appended header fields to opaque info: '" << hdr2cgistr << "'");
+    TRACEI(DEBUG, "Appended header fields to opaque info: '" 
+                 << hdr2cgistr.c_str() << "'");
     free(q);
     m_appended_hdr2cgistr = true;
     }
@@ -1365,8 +1376,9 @@ int XrdHttpReq::ProcessHTTPReq() {
 
 
         // We want to be invoked again after this request is finished
-        // Only if there is data to fetch from the socket
-        if (prot->BuffUsed() > 0)
+        // Only if there is data to fetch from the socket or there will
+        // never be more data
+        if (prot->BuffUsed() > 0 || (length == 0 && !sendcontinue))
           return 0;
 
         return 1;
@@ -1680,7 +1692,7 @@ int XrdHttpReq::ProcessHTTPReq() {
       xrdreq.mkdir.requestid = htons(kXR_mkdir);
 
       string s = resourceplusopaque.c_str();
-      xrdreq.mkdir.options[0] = (kXR_char) kXR_mkpath;
+      xrdreq.mkdir.options[0] = (kXR_char) kXR_mkdirpath;
 
       l = s.length() + 1;
       xrdreq.mkdir.dlen = htonl(l);
@@ -1764,7 +1776,9 @@ XrdHttpReq::PostProcessChecksum(std::string &digest_header) {
       return -1;
     }
 
-    TRACEI(REQ, "Checksum for HEAD " << resource << " " << reinterpret_cast<char *>(iovP[0].iov_base) << "=" << reinterpret_cast<char *>(iovP[iovN-1].iov_base));
+    TRACEI(REQ, "Checksum for HEAD " << resource.c_str() << " "
+               << reinterpret_cast<char *>(iovP[0].iov_base) << "=" 
+               << reinterpret_cast<char *>(iovP[iovN-1].iov_base));
     std::string response_name = convert_xrootd_to_rfc_name(reinterpret_cast<char *>(iovP[0].iov_base));
 
     bool convert_to_base64 = needs_base64_padding(convert_digest_rfc_name(m_req_digest));
@@ -1825,7 +1839,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
         if (iovN > 0) {
 
           // Now parse the stat info
-          TRACEI(REQ, "Stat for HEAD " << resource << " stat=" << (char *) iovP[0].iov_base);
+          TRACEI(REQ, "Stat for HEAD " << resource.c_str()
+                      << " stat=" << (char *) iovP[0].iov_base);
 
           long dummyl;
           sscanf((const char *) iovP[0].iov_base, "%ld %lld %ld %ld",
@@ -1927,7 +1942,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
               endp++;
 
               // Now parse the stat info
-              TRACEI(REQ, "Dirlist " << resource << " entry=" << entry << " stat=" << endp);
+              TRACEI(REQ, "Dirlist " << resource.c_str() << " entry=" << entry 
+                       << " stat=" << endp);
 
               long dummyl;
               sscanf(endp, "%ld %lld %ld %ld",
@@ -2084,7 +2100,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
               if (iovN > 0) {
                 
                 // Now parse the stat info
-                TRACEI(REQ, "Stat for GET " << resource << " stat=" << (char *) iovP[0].iov_base);
+                TRACEI(REQ, "Stat for GET " << resource.c_str()
+                         << " stat=" << (char *) iovP[0].iov_base);
                 
                 long dummyl;
                 sscanf((const char *) iovP[0].iov_base, "%ld %lld %ld %ld",
@@ -2100,7 +2117,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                 }
               }
               else {
-                TRACEI(REQ, "Can't find the stat information for '" << resource << "' Internal error?");
+                TRACEI(REQ, "Can't find the stat information for '" 
+                         << resource.c_str() << "' Internal error?");
               }
             }
             
@@ -2136,7 +2154,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
               // Now parse the stat info if we still don't have it
               if (filesize == 0) {
                 if (iovP[1].iov_len > 1) {
-                  TRACEI(REQ, "Stat for GET " << resource << " stat=" << (char *) iovP[1].iov_base);
+                  TRACEI(REQ, "Stat for GET " << resource.c_str() 
+                           << " stat=" << (char *) iovP[1].iov_base);
               
                   long dummyl;
                   sscanf((const char *) iovP[1].iov_base, "%ld %lld %ld %ld",
@@ -2403,7 +2422,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
           if (iovN > 0) {
 
             // Now parse the stat info
-            TRACEI(REQ, "Stat for removal " << resource << " stat=" << (char *) iovP[0].iov_base);
+            TRACEI(REQ, "Stat for removal " << resource.c_str() 
+                     << " stat=" << (char *) iovP[0].iov_base);
 
             long dummyl;
             sscanf((const char *) iovP[0].iov_base, "%ld %lld %ld %ld",
@@ -2452,7 +2472,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
             e.path = resource.c_str();
 
             // Now parse the stat info
-            TRACEI(REQ, "Collection " << resource << " entry=" << resource << " stat=" << (char *) iovP[0].iov_base);
+            TRACEI(REQ, "Collection " << resource.c_str()
+                     << " stat=" << (char *) iovP[0].iov_base);
 
             long dummyl;
             sscanf((const char *) iovP[0].iov_base, "%ld %lld %ld %ld",
@@ -2550,7 +2571,8 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                 endp++;
 
                 // Now parse the stat info
-                TRACEI(REQ, "Dirlist " << resource << " entry=" << entry << " stat=" << endp);
+                TRACEI(REQ, "Dirlist " <<resource.c_str() <<" entry=" <<entry
+                            << " stat=" << endp);
 
                 long dummyl;
                 sscanf(endp, "%ld %lld %ld %ld",
