@@ -1,8 +1,8 @@
 
-#include <regex>
 #include <cstring>
 #include <string>
 #include <iostream>
+#include <sstream>
 
 #include <uuid/uuid.h>
 #include "json.h"
@@ -138,6 +138,12 @@ Handler::GenerateID(const std::string &resource,
     uuid_unparse(uu, uuid_buf);
     std::string result(uuid_buf);
 
+// The following code shoul have been strictly for debugging purposes. This
+// added code skips it unless debug logging has been enabled. Due to the code
+// structure, indentation is a bit of a struggle as this is a minimal fix.
+//
+if (m_log->getMsgMask() & LogMask::Debug)
+   {
     std::stringstream ss;
     ss << "ID=" << result << ", ";
     ss << "resource=" << NormalizeSlashes(resource) << ", ";
@@ -159,7 +165,8 @@ Handler::GenerateID(const std::string &resource,
 
     ss << "expires=" << before;
 
-    m_log->Emsg("MacaroonGen", ss.str().c_str());
+    m_log->Emsg("MacaroonGen", ss.str().c_str());  // Mask::Debug
+   }
     return result;
 }
 
@@ -320,11 +327,13 @@ int Handler::ProcessTokenRequest(XrdHttpExtReq &req)
         }
         else if (value != path)
         {
+         if (m_log->getMsgMask() & LogMask::Error) {
             std::stringstream ss;
             ss << "Encountered requested scope request for authorization " << key
                << " with resource path " << value << "; however, prior request had path "
                << path;
-            m_log->Emsg("MacaroonRequest", ss.str().c_str());
+            m_log->Emsg("MacaroonRequest", ss.str().c_str()); // Mask::Error
+            }
             return req.SendSimpleResp(500, NULL, NULL, "Server only supports all scopes having the same path", 0);
         }
         other_caveats.push_back(key);
@@ -560,6 +569,7 @@ Handler::GenerateMacaroonResponse(XrdHttpExtReq &req, const std::string &resourc
     size_t size_hint = macaroon_serialize_size_hint(mac_with_date);
 
     std::vector<char> macaroon_resp; macaroon_resp.reserve(size_hint);
+    macaroon_resp.push_back((char)0);
     if (macaroon_serialize(mac_with_date, &macaroon_resp[0], size_hint, &mac_err))
     {
         printf("Returned macaroon_serialize code: %lu\n", (unsigned long)size_hint);
