@@ -29,14 +29,14 @@
 
 #include <unistd.h>
 #include <dirent.h>
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
 #include <memory.h>
-#include <string.h>
-#include <stdio.h>
-#include <time.h>
+#include <cstring>
+#include <cstdio>
+#include <ctime>
 #include <netdb.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -82,7 +82,6 @@
 #include "XrdOuc/XrdOucPgrwUtils.hh"
 #include "XrdOuc/XrdOucTList.hh"
 #include "XrdOuc/XrdOucTPC.hh"
-#include "XrdOuc/XrdOucTrace.hh"
 #include "XrdSec/XrdSecEntity.hh"
 #include "XrdSfs/XrdSfsAio.hh"
 #include "XrdSfs/XrdSfsFlags.hh"
@@ -102,7 +101,7 @@
 
 XrdSysError      OfsEroute(0);
 
-XrdOucTrace      OfsTrace(&OfsEroute);
+XrdSysTrace      OfsTrace("ofs");
 
 /******************************************************************************/
 /*               S t a t i s t i c a l   D a t a   O b j e c t                */
@@ -124,7 +123,7 @@ bool VerPgw(const char *buf, ssize_t off, size_t len, const uint32_t* csv,
    off_t badoff;
    int   badlen;
 
-// Verify incomming checksums
+// Verify incoming checksums
 //
    if (!XrdOucPgrwUtils::csVer(dInfo, badoff, badlen))
       {char eMsg[512];
@@ -505,7 +504,7 @@ int XrdOfsFile::open(const char          *path,      // In
 
 // Trace entry
 //
-   ZTRACE(open, std::hex <<open_mode <<"-" <<std::oct <<Mode <<std::dec <<" fn=" <<path);
+   ZTRACE(open, Xrd::hex1 <<open_mode <<"-" <<Xrd::oct1 <<Mode <<" fn=" <<path);
 
 // Verify that this object is not already associated with an open file
 //
@@ -1897,7 +1896,7 @@ int XrdOfs::chmod(const char             *path,    // In
 //
    if (!(retc = XrdOfsOss->Chmod(path, acc_mode, &chmod_Env))) return SFS_OK;
 
-// An error occured, return the error info
+// An error occurred, return the error info
 //
    return XrdOfsFS->Emsg(epname, einfo, retc, "change", path);
 }
@@ -1986,7 +1985,7 @@ int XrdOfs::exists(const char                *path,        // In
        return SFS_OK;
       }
 
-// An error occured, return the error info
+// An error occurred, return the error info
 //
    return XrdOfsFS->Emsg(epname, einfo, retc, "locate", path);
 }
@@ -2411,7 +2410,7 @@ int XrdOfs::truncate(const char             *path,    // In
 //
    if (!(retc = XrdOfsOss->Truncate(path, Size, &trunc_Env))) return SFS_OK;
 
-// An error occured, return the error info
+// An error occurred, return the error info
 //
    return XrdOfsFS->Emsg(epname, einfo, retc, "trunc", path);
 }
