@@ -28,11 +28,11 @@
 /******************************************************************************/
 
 #include <unistd.h>
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstring>
+#include <cstdio>
+#include <cstdlib>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -56,7 +56,6 @@
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucERoute.hh"
 #include "XrdOuc/XrdOucExport.hh"
-#include "XrdOuc/XrdOucTrace.hh"
 
 #include "XrdSec/XrdSecEntity.hh"
 #include "XrdSfs/XrdSfsFAttr.hh"
@@ -73,7 +72,7 @@
 
 extern XrdSysError      OfsEroute;
 
-extern XrdOucTrace      OfsTrace;
+extern XrdSysTrace      OfsTrace;
 
 /******************************************************************************/
 /*                    F i l e   S y s t e m   O b j e c t                     */
