@@ -28,8 +28,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <time.h>
+#include <cstdio>
+#include <ctime>
 #include <sys/errno.h>
 #include <sys/types.h>
 
@@ -402,12 +402,15 @@ int XrdOfsHandle::PoscSet(const char *User, int Unum, short Umod)
           else return 0;
       }
 
-// Find the markers in the incomming user
+// Find the markers in the incoming user
 //
    if (!(Col = index(User, ':')) || !(At = index(User, '@')))
       {User = Who; Col = Whc; At = Whh;}
 
-// If we already have a user check if it matches
+// If we already have a user check if it matches. We allow the same user to
+// reconnect to the file even when it is in posc mode to allow for write error
+// recovery. Other users must re-enable posc for the file to show that they are
+// aware that the file is under POSC mode as this is not a recovery.
 //
    if (Posc->User)
       {if (!Unum)
@@ -490,7 +493,7 @@ int XrdOfsHandle::Retire(XrdOfsHanCB *cbP, int hTime)
    int retc;
 
 // The handle can only be held by one reference and only if it's a POSC and
-// defered handling was properly set up.
+// deferred handling was properly set up.
 //
    myMutex.Lock();
    if (!Posc || !allOK)
@@ -779,7 +782,7 @@ void XrdOfsHanXpr::add2Q(int doLK)
 {
    XrdOfsHanXpr *xPP, *xP;
 
-// Place this object on the defered queue
+// Place this object on the deferred queue
 //
    if (doLK) xqCV.Lock();
    xPP = 0; xP = xprQ;
