@@ -32,11 +32,11 @@
 /******************************************************************************/
   
 #include <unistd.h>
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
 #include <signal.h>
 #include <strings.h>
-#include <stdio.h>
+#include <cstdio>
 #include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -78,7 +78,7 @@ XrdOssSys  *XrdOssSS = 0;
   
 XrdSysError OssEroute(0, "oss_");
 
-XrdOucTrace OssTrace(&OssEroute);
+XrdSysTrace OssTrace("oss");
 
 /******************************************************************************/
 /*           S t o r a g e   S y s t e m   I n s t a n t i a t o r            */
@@ -109,6 +109,11 @@ XrdOss *XrdOssGetSS(XrdSysLogger *Logger, const char *config_fn,
    if (urVer.vNum != myOssSys.myVersion->vNum
    &&  !XrdSysPlugin::VerCmp(urVer, *(myOssSys.myVersion))) return 0;
 
+// Set logger for tracing and errors
+//
+   OssTrace.SetLogger(Logger);
+   OssEroute.logger(Logger);
+
 // If no library has been specified, return the default object
 //
    if (!OssLib) {if (myOssSys.Init(Logger, config_fn, envP)) return 0;
@@ -118,7 +123,6 @@ XrdOss *XrdOssGetSS(XrdSysLogger *Logger, const char *config_fn,
 // Create a plugin object. Take into account the proxy library. Eventually,
 // we will need to support other core libraries. But, for now, this will do.
 //
-   OssEroute.logger(Logger);
    if (!(myLib = new XrdOucPinLoader(&OssEroute, myOssSys.myVersion,
                                      "osslib",   OssLib))) return 0;
 // Declare the interface versions
@@ -579,13 +583,17 @@ int XrdOssDir::Readdir(char *buff, int blen)
 //
    if (lclfd)
       {errno = 0;
-       if ((rp = readdir(lclfd)))
-          {strlcpy(buff, rp->d_name, blen);
+       while((rp = readdir(lclfd)))
+            {strlcpy(buff, rp->d_name, blen);
 #ifdef HAVE_FSTATAT
-           if (Stat && fstatat(fd, rp->d_name, Stat, 0)) return -errno;
+             if (Stat && fstatat(fd, rp->d_name, Stat, 0))
+                {if (errno != ENOENT) return -errno;
+                 errno = 0;
+                 continue;
+                }
 #endif
-           return XrdOssOK;
-          }
+             return XrdOssOK;
+            }
        *buff = '\0'; ateof = true;
        return -errno;
       }
@@ -831,7 +839,7 @@ ssize_t XrdOssFile::Read(off_t offset, size_t blen)
 
      if (fd < 0) return (ssize_t)-XRDOSS_E8004;
 
-#if defined(__linux__)
+#if defined(__linux__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
      posix_fadvise(fd, offset, blen, POSIX_FADV_WILLNEED);
 #endif
 
@@ -897,7 +905,7 @@ ssize_t XrdOssFile::ReadV(XrdOucIOVec *readV, int n)
 
 // For platforms that support fadvise, pre-advise what we will be reading
 //
-#if defined(__linux__) && defined(HAVE_ATOMICS)
+#if (defined(__linux__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))) && defined(HAVE_ATOMICS)
    EPNAME("ReadV");
    long long begOff, endOff, begLst = -1, endLst = -1;
    int nPR = n;
@@ -931,7 +939,7 @@ ssize_t XrdOssFile::ReadV(XrdOucIOVec *readV, int n)
         if (rdsz < 0 || rdsz != readV[i].size)
            {totBytes =  (rdsz < 0 ? -errno : -ESPIPE); break;}
         totBytes += rdsz;
-#if defined(__linux__) && defined(HAVE_ATOMICS)
+#if (defined(__linux__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))) && defined(HAVE_ATOMICS)
         if (nPR < n && readV[nPR].size > 0)
            {begOff = XrdOssSS->prPMask &  readV[nPR].offset;
             endOff = XrdOssSS->prPBits | (readV[nPR].offset+readV[nPR].size);
@@ -949,7 +957,7 @@ ssize_t XrdOssFile::ReadV(XrdOucIOVec *readV, int n)
 
 // All done, return bytes read.
 //
-#if defined(__linux__) && defined(HAVE_ATOMICS)
+#if (defined(__linux__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))) && defined(HAVE_ATOMICS)
    if (XrdOssSS->prDepth) AtomicDec((XrdOssSS->prActive));
 #endif
    return totBytes;
@@ -1035,6 +1043,36 @@ int XrdOssFile::Fchmod(mode_t Mode)
     return (fchmod(fd, Mode) ? -errno : XrdOssOK);
 }
   
+/******************************************************************************/
+/*                                  F c t l                                   */
+/******************************************************************************/
+/*
+  Function: Perform control operations on a file.
+
+  Input:    cmd       - The command.
+            alen      - length of arguments.
+            args      - Pointer to arguments.
+            resp      - Pointer to where response should be placed.
+
+  Output:   Returns XrdOssOK upon success and -errno upon failure.
+*/
+
+int XrdOssFile::Fctl(int cmd, int alen, const char *args, char **resp)
+{
+   const struct timeval *utArgs;
+
+   switch(cmd)
+         {case XrdOssDF::Fctl_utimes:
+               if (alen != sizeof(struct timeval)*2 || !args) return -EINVAL;
+               utArgs = (const struct timeval *)args;
+               if (futimes(fd, utArgs)) return -errno;
+               return XrdOssOK;
+               break;
+          default: break;
+         }
+   return -ENOTSUP;
+}
+  
 /******************************************************************************/
 /*                                 F l u s h                                  */
 /******************************************************************************/
@@ -1049,7 +1087,7 @@ void XrdOssFile::Flush()
 {
 // This actually only works in Linux so we punt otherwise
 //
-#if defined(__linux__)
+#if defined(__linux__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
    if (fd>= 0)
       {fdatasync(fd);
        posix_fadvise(fd, 0, 0, POSIX_FADV_DONTNEED);
@@ -1241,8 +1279,8 @@ int XrdOssFile::Open_ufs(const char *path, int Oflag, int Mode,
 
 // Trace the action.
 //
-    TRACE(Open, "fd=" <<myfd <<" flags=" <<std::hex <<Oflag <<" mode="
-                <<std::oct <<Mode <<std::dec <<ftype <<path);
+    TRACE(Open, "fd=" <<myfd <<" flags=" <<Xrd::hex1 <<Oflag <<" mode="
+                <<Xrd::oct1 <<Mode <<ftype <<path);
 
 // All done
 //
