@@ -30,13 +30,13 @@
 
 #include <unistd.h>
 #include <dirent.h>
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
 #include <map>
-#include <stdio.h>
+#include <cstdio>
 #include <string>
 #include <strings.h>
-#include <time.h>
+#include <ctime>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -60,7 +60,7 @@
 
 extern XrdSysError  OssEroute;
 
-extern XrdOucTrace  OssTrace;
+extern XrdSysTrace  OssTrace;
 
 XrdOssCache_Group  *XrdOssCache_Group::fsgroups = 0;
 XrdOssCache_Group  *XrdOssCache_Group::PubGroup = 0;
@@ -460,7 +460,7 @@ void XrdOssCache::Adjust(const char *Path, off_t size, struct stat *buf)
 // Process the result
 //
    if (fsp) Adjust(fsp, size);
-      else {DEBUG("cahe path " <<Path <<" not found.");}
+      else {DEBUG("cache path " <<Path <<" not found.");}
 }
 
 /******************************************************************************/
