@@ -29,17 +29,18 @@
 /******************************************************************************/
 
 #include <unistd.h>
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <stdio.h>
+#include <cstdio>
 #include <strings.h>
-#include <time.h>
+#include <ctime>
 #include <utime.h>
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include "XrdSys/XrdSysHeaders.hh"
+#include "XrdSys/XrdSysPlatform.hh"
 #include "XrdOss/XrdOssApi.hh"
 #include "XrdOss/XrdOssCache.hh"
 #include "XrdOss/XrdOssConfig.hh"
@@ -212,7 +213,7 @@ int XrdOssSys::StatFS(const char *path, unsigned long long &Opt,
 /******************************************************************************/
 
 /*
-  Function: Return free space information based on a cahe group name.
+  Function: Return free space information based on a cache group name.
 
   Input:    Env         - Is the environment for cgi info.
             path        - Is the path name.
@@ -231,7 +232,7 @@ int XrdOssSys::StatLS(XrdOucEnv &env, const char *path, char *buff, int &blen)
    char *cgrp, cgbuff[XrdOssSpace::minSNbsz];
    int retc;
 
-// We provide psuedo support whould be not have a cache
+// We provide pseudo support whould be not have a cache
 //
    if (!XrdOssCache_Group::fsgroups)
       {unsigned long long Opt;
@@ -243,7 +244,7 @@ int XrdOssSys::StatLS(XrdOucEnv &env, const char *path, char *buff, int &blen)
        return XrdOssOK;
       }
 
-// Find the cache group. We provide psuedo support should we not have a cache
+// Find the cache group. We provide pseudo support should we not have a cache
 //
    if (!(cgrp = env.Get(OSS_CGROUP)))
       {if ((retc = getCname(path, &sbuff, cgbuff))) return retc;
@@ -306,6 +307,13 @@ int XrdOssSys::StatPF(const char *path, struct stat *buff, int opts)
 //
    if (stat(path, buff)) return (errno ? -errno : -ENOMSG);
 
+// Check of general stat information is to be returned
+//
+   if (opts %  PF_dStat)
+      {buff->st_rdev = 0;
+       return XrdOssOK;
+      }
+
 // Check if device info is to be returned
 //
    if (opts & PF_dInfo) XrdOssCache::DevInfo(*buff);
