@@ -27,9 +27,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
 #ifndef WIN32
 #include <unistd.h>
 #include <sys/types.h>
@@ -42,6 +42,7 @@
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucProg.hh"
 #include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysPthread.hh"
 
@@ -51,7 +52,7 @@
   
 XrdOucProg::~XrdOucProg()
 {
-   if (ArgBuff) free(ArgBuff);
+   Reset();
    if (myStream) delete myStream;
 }
 
@@ -105,11 +106,10 @@ int XrdOucProg::Feed(const char *data[], const int dlen[])
                   theRC = Run(strmP, argV, argC)
   
 int XrdOucProg::Run(XrdOucStream *Sp, const char *argV[], int argC,
-                                      const char *envV[])
+                                      const char *envV[]) const
 {
-   const int maxArgs = sizeof(Arg)/sizeof(Arg[0])+4;
-   char *myArgs[maxArgs+1];
-   int rc, j = numArgs;
+   char **myArgs;
+   int rc, totArgs = numArgs + argC;
 
 // If we have no program, return an error
 //
@@ -118,26 +118,16 @@ int XrdOucProg::Run(XrdOucStream *Sp, const char *argV[], int argC,
        return -ENOEXEC;
       }
 
-// Copy the arglist to our private area
+// Copy arguments to stack storage (we could avoid this but not with the stack).
 //
-   memcpy((void *)myArgs, (const void *)Arg, lenArgs);
-
-// Append additional arguments as needed
-//
-   for (int i = 0; i < argC && j < maxArgs; i++)
-       if (argV[i]) myArgs[j++] = (char *)argV[i];
-
-// Make sure we don't have too many
-//
-   if (j >= maxArgs) 
-      {if (eDest) eDest->Emsg("Run", E2BIG, "execute", Arg[0]);
-       return -E2BIG;
-      }
-   myArgs[j] = (char *)0;
+   myArgs = (char**)alloca(sizeof(char *) * (totArgs + 1));
+   if (numArgs) memcpy(myArgs, Arg, sizeof(char*)*numArgs);
+   if (argC)    memcpy(&myArgs[numArgs], argV, sizeof(char*)*argC);
+   myArgs[totArgs] = 0;
 
 // If this is a local process then just execute it inline on this thread
 //
-   if (myProc) return (*myProc)(Sp, myArgs, j);
+   if (myProc) return (*myProc)(Sp, myArgs, totArgs);
 
 // Execute the command, possibly setting an environment.
 //
@@ -161,10 +151,33 @@ int XrdOucProg::Run(XrdOucStream *Sp, const char *argV[], int argC,
    return 0;
 }
 
+/******************************************************************************/
+  
+int XrdOucProg::Run(const char *argV[], int argC, const char *envV[]) const
+{
+   XrdOucStream cmd;
+   char *lp;
+   int rc;
+
+// Execute the command
+//
+   rc = Run(&cmd, argV, argC, envV);
+   if (rc) return rc;
+
+// Drain all output
+//
+   while((lp = cmd.GetLine()))
+        if (eDest && *lp) eDest->Emsg("Run", lp);
+
+// All done
+//
+   return RunDone(cmd);
+}
+
 /******************************************************************************/
 
 int XrdOucProg::Run(XrdOucStream *Sp, const char *arg1, const char *arg2,
-                                      const char *arg3, const char *arg4)
+                                      const char *arg3, const char *arg4) const
 {
    int rc;
 
@@ -177,7 +190,7 @@ int XrdOucProg::Run(XrdOucStream *Sp, const char *arg1, const char *arg2,
 /******************************************************************************/
 
 int XrdOucProg::Run(const char *arg1, const char *arg2,
-                    const char *arg3, const char *arg4)
+                    const char *arg3, const char *arg4) const
 {
    XrdOucStream cmd;
    char *lp;
@@ -202,7 +215,7 @@ int XrdOucProg::Run(const char *arg1, const char *arg2,
 
 int XrdOucProg::Run(char *outBuff, int outBsz,
                     const char *arg1, const char *arg2,
-                    const char *arg3, const char *arg4)
+                    const char *arg3, const char *arg4) const
 {
    XrdOucStream cmd;
    char *lp, *tp;
@@ -241,7 +254,7 @@ int XrdOucProg::Run(char *outBuff, int outBsz,
 /*                               R u n D o n e                                */
 /******************************************************************************/
 
-int XrdOucProg::RunDone(XrdOucStream &cmd)
+int XrdOucProg::RunDone(XrdOucStream &cmd) const
 {
    int rc;
 
@@ -283,36 +296,36 @@ int XrdOucProg::RunDone(XrdOucStream &cmd)
 int XrdOucProg::Setup(const char *prog, XrdSysError *errP,
                       int (*Proc)(XrdOucStream *, char **, int))
 {
-   const int maxArgs = sizeof(Arg)/sizeof(Arg[0]);
-   char *pp;
-   int j;
+   static const int maxArgs = 65;
+   char *argV[maxArgs];
+   int rc;
 
 // Prepare to handle the program
 //
-   if (ArgBuff) free(ArgBuff);
-   pp = ArgBuff = strdup(prog);
+   Reset();
    if (!errP) errP = eDest;
+   myProc = 0;
+   ArgBuff = strdup(prog);
   
 // Construct the argv array based on passed command line.
 //
-for (j = 0; j < maxArgs-1 && *pp; j++)
-    {while(*pp == ' ') pp++;
-     if (!(*pp)) break;
-     Arg[j] = pp;
-     while(*pp && *pp != ' ') pp++;
-     if (*pp) {*pp = '\0'; pp++;}
-    }
-
-// Make sure we did not overflow the buffer
-//
-   if (j == maxArgs-1 && *pp) 
-      {if (errP) errP->Emsg("Run", E2BIG, "set up", Arg[0]);
-       free(ArgBuff); ArgBuff = 0;
-       return -E2BIG;
+   rc = XrdOucUtils::argList(ArgBuff, argV, maxArgs);
+   if (rc <= 0)
+      {if (errP)
+          {if (!rc || !argV[0])
+              {const char *pgm =  (Proc ? "proceedure" : "program");
+               errP->Emsg("Run", pgm, "name not specified.");
+              } else errP->Emsg("Run", rc, "set up", argV[0]);
+          }
+       return (rc ? rc : -EINVAL);
       }
-   Arg[j] = (char *)0;
-   numArgs= j;
-   lenArgs = sizeof(Arg[0]) * numArgs;
+
+// Record the arguments including the phamtom null pointer. We must have
+// atleast one, the program or proceedure name.
+//
+   numArgs = rc;
+   Arg = new char*[rc+1];
+   memcpy(Arg, argV, sizeof(char *) * (rc+1));
 
 // If this is a local process then just record its address
 //
@@ -321,9 +334,9 @@ for (j = 0; j < maxArgs-1 && *pp; j++)
 // Make sure the program is really executable
 //
    if (access(Arg[0], X_OK))
-      {int rc = errno;
+      {rc = errno;
        if (errP) errP->Emsg("Run", rc, "set up", Arg[0]);
-       free(ArgBuff); ArgBuff = 0;
+       Reset();
        return rc;
       }
    return 0;
@@ -350,6 +363,19 @@ int XrdOucProg::Start()
 /******************************************************************************/
 /*                       P r i v a t e   M e t h o d s                        */
 /******************************************************************************/
+/******************************************************************************/
+/*                                 R e s e t                                  */
+/******************************************************************************/
+
+void XrdOucProg::Reset()
+{
+
+   if (ArgBuff) {free(ArgBuff); ArgBuff = 0;}
+   if (numArgs) delete [] Arg;
+   Arg = &ArgBuff;
+   numArgs = 0;
+}
+  
 /******************************************************************************/
 /*                               R e s t a r t                                */
 /******************************************************************************/
