@@ -27,19 +27,16 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <ctype.h>
+#include <cctype>
 #include <fcntl.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
+#include <cstdlib>
+#include <cstring>
+#include <cstdio>
 #ifndef WIN32
 #include <poll.h>
 #include <unistd.h>
 #include <strings.h>
-#if !defined(__linux__) && !defined(__CYGWIN__)
-#ifdef __FreeBSD__
-#include <sys/param.h>
-#endif
+#if !defined(__linux__) && !defined(__CYGWIN__) && !defined(__GNU__)
 #include <sys/conf.h>
 #endif
 #include <sys/stat.h>
@@ -506,7 +503,7 @@ int XrdOucStream::Exec(char **parm, int inrd, int efd)
        {if (inrd)
            {if (dup2(Child_in, STDIN_FILENO) < 0)
                {Erx(Exec, errno, "setting up standard in for " <<parm[0]);
-                exit(255);
+                _exit(255);
                } else if (Child_in != Child_out) close(Child_in);
            }
        }
@@ -516,7 +513,7 @@ int XrdOucStream::Exec(char **parm, int inrd, int efd)
     if (Child_out >= 0)
        {if (dup2(Child_out, STDOUT_FILENO) < 0)
            {Erx(Exec, errno, "setting up standard out for " <<parm[0]);
-            exit(255);
+            _exit(255);
            } else if (Child_out != Child_log) close(Child_out);
        }
 
@@ -525,7 +522,7 @@ int XrdOucStream::Exec(char **parm, int inrd, int efd)
     if (Child_log >= 0)
        {if (dup2(Child_log, STDERR_FILENO) < 0)
            {Erx(Exec, errno, "set up standard err for " <<parm[0]);
-            exit(255);
+            _exit(255);
            } else close(Child_log);
        }
 
@@ -544,7 +541,7 @@ int XrdOucStream::Exec(char **parm, int inrd, int efd)
     setpgid(0,0);
     execv(parm[0], parm);
     Erx(Exec, errno, "executing " <<parm[0]);
-    exit(255);
+    _exit(255);
 }
 
 /******************************************************************************/
@@ -728,7 +725,7 @@ char *XrdOucStream::GetMyFirstWord(int lowcase)
         if (var && !strcmp("fi",   var))
            {if (sawif) sawif = skpel = skip2fi = 0;
                else {if (Eroute)
-                        Eroute->Emsg("Stream", "No preceeding 'if' for 'fi'.");
+                        Eroute->Emsg("Stream", "No preceding 'if' for 'fi'.");
                      ecode = EINVAL;
                     }
             continue;
@@ -747,6 +744,11 @@ char *XrdOucStream::GetWord(int lowcase)
 {
      char *wp, *ep;
 
+     // A call means the first token was acceptable and we continuing to
+     // parse, hence the line is echoable.
+     //
+     if (llBok == 1) llBok = 2;
+
      // If we have a token, return it
      //
      xline = 1;
@@ -1075,7 +1077,7 @@ bool XrdOucStream::docont(const char *path, XrdOucTList *tlP)
 // A continue directive in the context of a continuation is illegal
 //
    if ((myInfo && myInfo->fcList) || (flags & XrdOucStream_CONT) != 0)
-      return Echo(EINVAL, "'continue' is a continuation is not allowed.");
+      return Echo(EINVAL, "'continue' in a continuation is not allowed.");
 
 // Check if this file must exist (we also take care of empty paths)
 //
@@ -1096,7 +1098,7 @@ bool XrdOucStream::docont(const char *path, XrdOucTList *tlP)
       }
 
 // For directory continuation, there is much more to do (this can only happen
-// once). Note that we used to allow a limited number of chained fle
+// once). Note that we used to allow a limited number of chained file
 // continuations. No more, but we are still setup to easily do so.
 //
    if ((Stat.st_mode & S_IFMT) == S_IFDIR)
@@ -1196,6 +1198,7 @@ bool XrdOucStream::docontF(const char *path, bool noentok)
 //
    if (Eroute) Eroute->Say("Config continuing with file ", path, " ...");
    bleft = 0;
+   close(cFD);
    return true;
 }
   
@@ -1210,7 +1213,7 @@ char *XrdOucStream::doelse()
 // An else must be preceeded by an if and not by a naked else
 //
    if (!sawif || sawif == 2)
-      {if (Eroute) Eroute->Emsg("Stream", "No preceeding 'if' for 'else'.");
+      {if (Eroute) Eroute->Emsg("Stream", "No preceding 'if' for 'else'.");
        ecode = EINVAL;
        return 0;
       }
