@@ -29,8 +29,8 @@
 /******************************************************************************/
   
 //#include <iostream>
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 #include <strings.h>
 
 #include "XrdNet/XrdNetAddr.hh"
