@@ -28,9 +28,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <ctype.h>
+#include <cctype>
 #include <grp.h>
-#include <stdio.h>
+#include <cstdio>
 
 #ifdef WIN32
 #include <direct.h>
@@ -43,8 +43,11 @@
 #endif
 #include <map>
 #include "XrdNet/XrdNetUtils.hh"
+#include "XrdOuc/XrdOucCRC.hh"
 #include "XrdOuc/XrdOucEnv.hh"
+#include "XrdOuc/XrdOucSHA3.hh"
 #include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucString.hh"
 #include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysError.hh"
@@ -103,6 +106,65 @@ int LookUp(idMap_t &idMap, unsigned int id, char *buff, int blen)
 }
 }
   
+/******************************************************************************/
+/*                               a r g L i s t                                */
+/******************************************************************************/
+
+int XrdOucUtils::argList(char *args, char **argV, int argC)
+{
+   char *aP = args;
+   int j;
+
+// Construct the argv array based on passed command line.
+//
+for (j = 0; j < argC; j++)
+    {while(*aP == ' ') aP++;
+     if (!(*aP)) break;
+
+     if (*aP == '"' || *aP == '\'')
+        {argV[j] = aP+1;
+         aP = index(aP+1, *aP);
+         if (!aP || (*(aP+1) != ' ' && *(aP+1)))
+            {if (!j) argV[0] = 0; return -EINVAL;}
+         *aP++ = '\0';
+        } else {
+         argV[j] = aP;
+         if ((aP = index(aP+1, ' '))) *aP++ = '\0';
+            else {j++; break;}
+        }
+
+    }
+
+// Make sure we did not overflow the vector
+//
+   if (j > argC-1) return -E2BIG;
+
+// End list with a null pointer and return the actual number of arguments
+//
+   argV[j] = 0;
+   return j;
+}
+  
+/******************************************************************************/
+/*                               b i n 2 h e x                                */
+/******************************************************************************/
+  
+char *XrdOucUtils::bin2hex(char *inbuff, int dlen, char *buff, int blen,
+                           bool sep)
+{
+    static char hv[] = "0123456789abcdef";
+    char *outbuff = buff;
+    for (int i = 0; i < dlen && blen > 2; i++) {
+        *outbuff++ = hv[(inbuff[i] >> 4) & 0x0f];
+        *outbuff++ = hv[ inbuff[i]       & 0x0f];
+        blen -= 2;
+        if (sep && blen > 1 && ((i & 0x03) == 0x03 || i+1 == dlen))
+           {*outbuff++ = ' '; blen--;}
+        }
+     *outbuff = '\0';
+     return buff;
+}
+
 /******************************************************************************/
 /*                              e n d s W i t h                               */
 /******************************************************************************/
@@ -151,7 +213,7 @@ char *XrdOucUtils::eText(int rc, char *eBuff, int eBlen)
 // Returning 1 if true (i.e., this machine is one of the named hosts in hostlist 
 // and is running one of the programs pgmlist and named by one of the names in 
 // namelist).
-// Return -1 (negative truth) if an error occured.
+// Return -1 (negative truth) if an error occurred.
 // Otherwise, returns false (0). Some combination of hostlist, pgm, and 
 // namelist, must be specified.
 
@@ -274,6 +336,42 @@ int XrdOucUtils::doIf(XrdSysError *eDest, XrdOucStream &Config,
    return (val != 0);
 }
 
+/******************************************************************************/
+/*                               f i n d P g m                                */
+/******************************************************************************/
+
+bool XrdOucUtils::findPgm(const char *pgm, XrdOucString& path)
+{
+   struct stat Stat;
+
+// Check if only executable bit needs to be checked
+//
+   if (*pgm == '/')
+      {if (stat(pgm, &Stat) || !(Stat.st_mode & S_IXOTH)) return false;
+       path = pgm;
+       return true;
+      }
+
+// Make sure we have the paths to check
+//
+   const char *pEnv = getenv("PATH");
+   if (!pEnv) return false;
+
+// Setup to find th executable
+//
+   XrdOucString prog, pList(pEnv);
+   int from = 0;;
+   prog += '/'; prog += pgm;
+
+// Find it!
+//
+   while((from = pList.tokenize(path, from, ':')) != -1)
+        {path += prog;
+         if (!stat(path.c_str(), &Stat) && Stat.st_mode & S_IXOTH) return true;
+        }
+   return false;
+}
+  
 /******************************************************************************/
 /*                              f m t B y t e s                               */
 /******************************************************************************/
@@ -340,6 +438,63 @@ int XrdOucUtils::genPath(char *buff, int blen, const char *path, const char *psf
     return 0;
 }
 
+/******************************************************************************/
+/*                               g e t F i l e                                */
+/******************************************************************************/
+
+char *XrdOucUtils::getFile(const char *path, int &rc, int maxsz, bool notempty)
+{
+   struct stat Stat;
+   struct fdHelper
+         {int fd = -1;
+              fdHelper() {}
+             ~fdHelper() {if (fd >= 0) close(fd);}
+         } file;
+   char *buff;
+   int   flen;
+
+// Preset RC
+//
+   rc = 0;
+
+// Open the file in read mode
+//
+   if ((file.fd = open(path, O_RDONLY)) < 0) {rc = errno; return 0;}
+
+// Get the size of the file
+//
+   if (fstat(file.fd, &Stat)) {rc = errno; return 0;}
+
+// Check if the size exceeds the maximum allowed
+//
+   if (Stat.st_size > maxsz) {rc = EFBIG; return 0;}
+
+// Make sure the file is not empty is empty files are disallowed
+//
+   if (Stat.st_size == 0 && notempty) {rc = ENODATA; return 0;}
+
+// Allocate a buffer
+//
+   if ((buff = (char *)malloc(Stat.st_size+1)) == 0)
+      {rc = errno; return 0;}
+
+// Read the contents of the file into the buffer
+//
+   if (Stat.st_size)
+      {if ((flen = read(file.fd, buff, Stat.st_size)) < 0)
+          {rc = errno; free(buff); return 0;}
+      } else flen = 0;
+
+// Add null byte. recall the buffer is bigger by one byte
+//
+   buff[flen] = 0;
+
+// Return the size aand the buffer
+//
+   rc = flen;
+   return buff;
+}
+
 /******************************************************************************/
 /*                                g e t G I D                                 */
 /******************************************************************************/
@@ -460,25 +615,88 @@ int XrdOucUtils::GroupName(gid_t gID, char *gName, int gNsz)
    return glen;
 }
 
+/******************************************************************************/
+/*                                i 2 b s t r                                 */
+/******************************************************************************/
+
+const char *XrdOucUtils::i2bstr(char *buff, int blen, int val, bool pad)
+{
+   char zo[2] = {'0', '1'};
+
+   if (blen < 2) return "";
+
+   buff[--blen] = 0;
+   if (!val) buff[blen--] = '0';
+      else while(val && blen >= 0)
+                {buff[blen--] = zo[val & 0x01];
+                 val >>= 1;
+                }
+
+   if (blen >= 0 && pad) while(blen >= 0) buff[blen--] = '0';
+
+   return &buff[blen+1];
+}
+  
 /******************************************************************************/
 /*                                 I d e n t                                  */
 /******************************************************************************/
 
+namespace
+{
+long long genSID(char *&urSID, const char *iHost, int         iPort,
+                               const char *iName, const char *iProg)
+{
+   static const XrdOucSHA3::MDLen mdLen = XrdOucSHA3::SHA3_512;
+   static const uint32_t fpOffs = 2, fpSize = 6;  // 48 bit finger print
+
+   const char *iSite = getenv("XRDSITE");
+   unsigned char mDigest[mdLen];
+   XrdOucString  myID;
+   union {uint64_t mdLL; unsigned char mdUC[8];}; // Works for fpSize only!
+
+// Construct our unique identification
+//
+   if (iSite) myID  = iSite;
+   myID += iHost;
+   myID += iPort;
+   if (iName) myID += iName;
+   myID += iProg;
+
+// Generate a SHA3 digest of this string.
+//
+   memset(mDigest, 0, sizeof(mDigest));
+   XrdOucSHA3::Calc(myID.c_str(), myID.length(), mDigest, mdLen);
+
+// Generate a CRC32C of the same string
+//
+   uint32_t crc32c = XrdOucCRC::Calc32C(myID.c_str(), myID.length());
+
+// We need a 48-bit fingerprint that has a very low probability of collision.
+// We accomplish this by convoluting the CRC32C checksum with the SHA3 checksum.
+//
+   uint64_t fpPos = crc32c % (((uint32_t)mdLen) - fpSize);
+   mdLL = 0;
+   memcpy(mdUC+fpOffs, mDigest+fpPos, fpSize);
+   long long fpVal = static_cast<long long>(ntohll(mdLL));
+
+// Generate the character version of our fingerprint and return the binary one.
+//
+   char fpBuff[64];
+   snprintf(fpBuff, sizeof(fpBuff), "%lld", fpVal);
+   urSID = strdup(fpBuff);
+   return fpVal;
+}
+}
+
 char *XrdOucUtils::Ident(long long  &mySID, char *iBuff, int iBlen,
                          const char *iHost, const char *iProg,
-                         const char *iName, int Port)
+                         const char *iName, int iPort)
 {
-   const char *sP;
-   char sName[64], uName[256];
-   long long urSID;
-   int  myPid;
-
-// Generate our server ID
-//
-   myPid   = static_cast<int>(getpid());
-   urSID   = static_cast<long long>(myPid)<<16ll | Port;
-   sprintf(sName, "%lld", urSID);
-   if (!(sP = getenv("XRDSITE"))) sP = "";
+   static char *theSIN;
+   static long long theSID = genSID(theSIN, iHost, iPort, iName, iProg);
+   const char *sP = getenv("XRDSITE");
+   char uName[256];
+   int myPid   = static_cast<int>(getpid());
 
 // Get our username
 //
@@ -487,13 +705,13 @@ char *XrdOucUtils::Ident(long long  &mySID, char *iBuff, int iBlen,
 
 // Create identification record
 //
-   snprintf(iBuff, iBlen, "%s.%d:%s@%s\n&pgm=%s&inst=%s&port=%d&site=%s",
-                          uName, myPid, sName, iHost, iProg, iName, Port, sP);
+   snprintf(iBuff,iBlen,"%s.%d:%s@%s\n&site=%s&port=%d&inst=%s&pgm=%s",
+            uName, myPid, theSIN, iHost, (sP ? sP : ""), iPort, iName, iProg);
 
-// Return a copy of the sid
+// Return a copy of the sid key
 //
-   h2nll(urSID, mySID);
-   return strdup(sName);
+   h2nll(theSID, mySID);
+   return strdup(theSIN);
 }
   
 /******************************************************************************/
@@ -643,7 +861,7 @@ bool XrdOucUtils::makeHome(XrdSysError &eDest, const char *inst,
    const char *slash = "", *slash2 = "";
    int n, rc;
 
-// Provide backward compatability for instance name qualification
+// Provide backward compatibility for instance name qualification
 //
 
    if (!path || !(n = strlen(path)))
@@ -721,9 +939,25 @@ bool XrdOucUtils::parseLib(XrdSysError &eDest, XrdOucStream &Config,
 {
     char *val, parms[2048];
 
-// Get the path
+// Get the next token
 //
-   if (!(val = Config.GetWord()) || !val[0])
+   val = Config.GetWord();
+
+// We do not support stacking as the caller does not support stacking
+//
+   if (val && !strcmp("++", val))
+      {eDest.Say("Config warning: stacked plugins are not supported in "
+                 "this context; directive ignored!");
+       return true;
+      }
+
+// Now skip over any options
+//
+   while(val && *val && *val == '+') val = Config.GetWord();
+
+// Check if we actually have a path
+//
+   if (!val || !val[0])
       {eDest.Emsg("Config", libName, "not specified"); return false;}
 
 // Record the path
@@ -739,6 +973,7 @@ bool XrdOucUtils::parseLib(XrdSysError &eDest, XrdOucStream &Config,
 
 // Record any parms
 //
+   *parms = 0;
    if (!Config.GetRest(parms, sizeof(parms)))
       {eDest.Emsg("Config", libName, "parameters too long"); return false;}
    if (*parms) *libParm = strdup(parms);
