@@ -19,10 +19,41 @@ using namespace XrdPfc;
 
 XrdVERSIONINFO(XrdOucGetCache, XrdPfc);
 
+Configuration::Configuration() :
+   m_hdfsmode(false),
+   m_allow_xrdpfc_command(false),
+   m_data_space("public"),
+   m_meta_space("public"),
+   m_diskTotalSpace(-1),
+   m_diskUsageLWM(-1),
+   m_diskUsageHWM(-1),
+   m_fileUsageBaseline(-1),
+   m_fileUsageNominal(-1),
+   m_fileUsageMax(-1),
+   m_purgeInterval(300),
+   m_purgeColdFilesAge(-1),
+   m_purgeAgeBasedPeriod(10),
+   m_accHistorySize(20),
+   m_dirStatsMaxDepth(-1),
+   m_dirStatsStoreDepth(0),
+   m_bufferSize(256*1024),
+   m_RamAbsAvailable(0),
+   m_RamKeepStdBlocks(0),
+   m_wqueue_blocks(16),
+   m_wqueue_threads(4),
+   m_prefetch_max_blocks(10),
+   m_hdfsbsize(128*1024*1024),
+   m_flushCnt(2000),
+   m_cs_UVKeep(-1),
+   m_cs_Chk(CSChk_Net),
+   m_cs_ChkTLS(false)
+{}
+
+
 bool Cache::cfg2bytes(const std::string &str, long long &store, long long totalSpace, const char *name)
 {
    char errStr[1024];
-   snprintf(errStr, 1024, "Cache::ConfigParameters() Error parsing parameter %s", name);
+   snprintf(errStr, 1024, "ConfigParameters() Error parsing parameter %s", name);
 
    if (::isalpha(*(str.rbegin())))
    {
@@ -47,7 +78,7 @@ bool Cache::cfg2bytes(const std::string &str, long long &store, long long totalS
 
    if (store < 0 || store > totalSpace)
    {
-     snprintf(errStr, 1024, "Cache::ConfigParameters() Error: parameter %s should be between 0 and total available disk space (%lld) - it is %lld (given as %s)",
+     snprintf(errStr, 1024, "ConfigParameters() Error: parameter %s should be between 0 and total available disk space (%lld) - it is %lld (given as %s)",
               name, totalSpace, store, str.c_str());
      m_log.Emsg(errStr, "");
      return false;
@@ -56,6 +87,94 @@ bool Cache::cfg2bytes(const std::string &str, long long &store, long long totalS
    return true;
 }
 
+/* Function: xcschk
+
+   Purpose:  To parse the directive: cschk <parms>
+
+             parms:  [[no]net] [[no]tls] [[no]cache] [uvkeep <arg>]
+
+             all     Checksum check on cache & net transfers.
+             cache   Checksum check on cache only, 'no' turns it off.
+             net     Checksum check on net transfers 'no' turns it off.
+             tls     use TLS if server doesn't support checksums 'no' turns it off.
+             uvkeep  Maximum amount of time a cached file make be kept if it
+                     contains unverified checksums as n[d|h|m|s], where 'n'
+                     is a non-negative integer. A value of 0 prohibits disk
+                     caching unless the checksum can be verified. You can
+                     also specify "lru" which means the standard purge policy
+                     is to be used.
+
+   Output: true upon success or false upon failure.
+ */
+bool Cache::xcschk(XrdOucStream &Config)
+{
+   const char *val, *val2;
+   struct cschkopts {const char *opname; int opval;} csopts[] =
+   {
+      {"off",     CSChk_None},
+      {"cache",   CSChk_Cache},
+      {"net",     CSChk_Net},
+      {"tls",     CSChk_TLS}
+   };
+   int i, numopts = sizeof(csopts)/sizeof(struct cschkopts);
+   bool isNo;
+
+   if (! (val = Config.GetWord()))
+   {m_log.Emsg("Config", "cschk parameter not specified"); return false; }
+
+   while(val)
+   {
+      if ((isNo = strncmp(val, "no", 2) == 0))
+         val2 = val + 2;
+      else
+         val2 = val;
+      for (i = 0; i < numopts; i++)
+      {
+         if (!strcmp(val2, csopts[i].opname))
+         {
+            if (isNo)
+               m_configuration.m_cs_Chk &= ~csopts[i].opval;
+            else if (csopts[i].opval)
+               m_configuration.m_cs_Chk |= csopts[i].opval;
+            else
+               m_configuration.m_cs_Chk  = csopts[i].opval;
+            break;
+         }
+      }
+      if (i >= numopts)
+      {
+         if (strcmp(val, "uvkeep"))
+         {
+            m_log.Emsg("Config", "invalid cschk option -", val);
+            return false;
+         }
+         if (!(val = Config.GetWord()))
+         {
+            m_log.Emsg("Config", "cschk uvkeep value not specified");
+            return false;
+         }
+         if (!strcmp(val, "lru"))
+            m_configuration.m_cs_UVKeep = -1;
+         else
+         {
+            int uvkeep;
+            if (XrdOuca2x::a2tm(m_log, "uvkeep time", val, &uvkeep, 0))
+               return false;
+            m_configuration.m_cs_UVKeep = uvkeep;
+         }
+      }
+      val = Config.GetWord();
+   }
+   // Decompose into separate TLS state, it is only passed on to psx
+   m_configuration.m_cs_ChkTLS =  m_configuration.m_cs_Chk & CSChk_TLS;
+   m_configuration.m_cs_Chk   &= ~CSChk_TLS;
+
+   m_env->Put("psx.CSNet", m_configuration.is_cschk_net() ? (m_configuration.m_cs_ChkTLS ? "2" : "1") : "0");
+
+   return true;
+}
+
+
 /* Function: xdlib
 
    Purpose:  To parse the directive: decisionlib <path> [<parms>]
@@ -97,7 +216,7 @@ bool Cache::xdlib(XrdOucStream &Config)
    Decision * d = ep(m_log);
    if (! d)
    {
-      TRACE(Error, "Cache::Config() decisionlib was not able to create a decision object");
+      TRACE(Error, "Config() decisionlib was not able to create a decision object");
       return false;
    }
    if (params[0])
@@ -137,6 +256,7 @@ bool Cache::xtrace(XrdOucStream &Config)
          return true;
       }
    }
+   m_log.Emsg("Config", "invalid trace option -", val);
    return false;
 }
 
@@ -158,14 +278,14 @@ bool Cache::Config(const char *config_filename, const char *parameters)
 
    if (! config_filename || ! *config_filename)
    {
-      TRACE(Error, "Cache::Config() configuration file not specified.");
+      TRACE(Error, "Config() configuration file not specified.");
       return false;
    }
 
    int fd;
    if ( (fd = open(config_filename, O_RDONLY, 0)) < 0)
    {
-      TRACE( Error, "Cache::Config() can't open configuration file " << config_filename);
+      TRACE( Error, "Config() can't open configuration file " << config_filename);
       return false;
    }
 
@@ -188,8 +308,11 @@ bool Cache::Config(const char *config_filename, const char *parameters)
       m_configuration.m_wqueue_threads = 1;
    }
 
+   // If network checksum processing is the default, indicate so.
+   if (m_configuration.is_cschk_net()) m_env->Put("psx.CSNet", m_configuration.m_cs_ChkTLS ? "2" : "1");
+
    // Actual parsing of the config file.
-   bool retval = true;
+   bool retval = true, aOK = true;
    char *var;
    while ((var = Config.GetMyFirstWord()))
    {
@@ -197,6 +320,10 @@ bool Cache::Config(const char *config_filename, const char *parameters)
       {
          retval = ofsCfg->Parse(XrdOfsConfigPI::theOssLib);
       }
+      else if (! strcmp(var,"pfc.cschk"))
+      {
+         retval = xcschk(Config);
+      }
       else if (! strcmp(var,"pfc.decisionlib"))
       {
          retval = xdlib(Config);
@@ -216,8 +343,8 @@ bool Cache::Config(const char *config_filename, const char *parameters)
 
       if ( ! retval)
       {
-         TRACE(Error, "Cache::Config() error in parsing");
-         break;
+         TRACE(Error, "Config() error in parsing");
+         aOK = false;
       }
    }
 
@@ -225,14 +352,24 @@ bool Cache::Config(const char *config_filename, const char *parameters)
 
    // Load OSS plugin.
    myEnv.Put("oss.runmode", "pfc");
+   if (m_configuration.is_cschk_cache())
+   {
+      char csi_conf[128];
+      if (snprintf(csi_conf, 128, "space=%s nofill", m_configuration.m_meta_space.c_str()) < 128)
+      {
+         ofsCfg->Push(XrdOfsConfigPI::theOssLib, "libXrdOssCsi.so", csi_conf);
+      } else {
+         TRACE(Error, "Config() buffer too small for libXrdOssCsi params.");
+         return false;
+      }
+   }
    if (ofsCfg->Load(XrdOfsConfigPI::theOssLib, &myEnv))
    {
       ofsCfg->Plugin(m_oss);
    }
    else
    {
-      TRACE(Error, "Cache::Config() Unable to create an OSS object");
-      m_oss = 0;
+      TRACE(Error, "Config() Unable to create an OSS object");
       return false;
    }
 
@@ -241,12 +378,12 @@ bool Cache::Config(const char *config_filename, const char *parameters)
    {
       if (m_oss->StatVS(&sP, m_configuration.m_data_space.c_str(), 1) < 0)
       {
-         m_log.Emsg("Cache::ConfigParameters()", "error obtaining stat info for data space ", m_configuration.m_data_space.c_str());
+         m_log.Emsg("ConfigParameters()", "error obtaining stat info for data space ", m_configuration.m_data_space.c_str());
          return false;
       }
       if (sP.Total < 10ll << 20)
       {
-         m_log.Emsg("Cache::ConfigParameters()", "available data space is less than 10 MB (can be due to a mistake in oss.localroot directive) for space ",
+         m_log.Emsg("ConfigParameters()", "available data space is less than 10 MB (can be due to a mistake in oss.localroot directive) for space ",
                     m_configuration.m_data_space.c_str());
                     return false;
       }
@@ -257,12 +394,11 @@ bool Cache::Config(const char *config_filename, const char *parameters)
           cfg2bytes(tmpc.m_diskUsageHWM, m_configuration.m_diskUsageHWM, sP.Total, "highWatermark"))
       {
          if (m_configuration.m_diskUsageLWM >= m_configuration.m_diskUsageHWM) {
-            printf("GGGG %lld %lld\n", m_configuration.m_diskUsageLWM, m_configuration.m_diskUsageHWM);
-            m_log.Emsg("Cache::ConfigParameters()", "pfc.diskusage should have lowWatermark < highWatermark.");
-            retval = false;
+            m_log.Emsg("ConfigParameters()", "pfc.diskusage should have lowWatermark < highWatermark.");
+            aOK = false;
          }
       }
-      else retval = false;
+      else aOK = false;
 
       if ( ! tmpc.m_fileUsageMax.empty())
       {
@@ -274,11 +410,11 @@ bool Cache::Config(const char *config_filename, const char *parameters)
               m_configuration.m_fileUsageBaseline >= m_configuration.m_fileUsageMax     ||
               m_configuration.m_fileUsageNominal  >= m_configuration.m_fileUsageMax)
           {
-            m_log.Emsg("Cache::ConfigParameters()", "pfc.diskusage files should have baseline < nominal < max.");
-            retval = false;
+            m_log.Emsg("ConfigParameters()", "pfc.diskusage files should have baseline < nominal < max.");
+            aOK = false;
           }
         }
-        else retval = false;
+        else aOK = false;
       }
    }
    // sets flush frequency
@@ -320,12 +456,25 @@ bool Cache::Config(const char *config_filename, const char *parameters)
    char* cenv = getenv("XRDDEBUG");
    if (cenv && ! strcmp(cenv,"1") && m_trace->What < 4) m_trace->What = 4;
 
-   if (retval)
+   if (aOK)
    {
       int  loff = 0;
-      char buff[8192];
-      float rg =  (m_configuration.m_RamAbsAvailable) / float(1024*1024*1024);
+//                         000    001            010
+      const char *csc[] = {"off", "cache nonet", "nocache net notls",
+//                         011
+                           "cache net notls",
+//                         100    101            110
+                           "off", "cache nonet", "nocache net tls",
+//                         111
+                           "cache net tls"};
+      char buff[8192], uvk[32];
+      if (m_configuration.m_cs_UVKeep < 0)
+         strcpy(uvk, "lru");
+      else
+         sprintf(uvk, "%ld", m_configuration.m_cs_UVKeep);
+      float rg = (m_configuration.m_RamAbsAvailable) / float(1024*1024*1024);
       loff = snprintf(buff, sizeof(buff), "Config effective %s pfc configuration:\n"
+                      "       pfc.cschk %s uvkeep %s\n"
                       "       pfc.blocksize %lld\n"
                       "       pfc.prefetch %d\n"
                       "       pfc.ram %.fg\n"
@@ -337,6 +486,7 @@ bool Cache::Config(const char *config_filename, const char *parameters)
                       "       pfc.flush %lld\n"
                       "       pfc.acchistorysize %d\n",
                       config_filename,
+                      csc[int(m_configuration.m_cs_Chk)], uvk,
                       m_configuration.m_bufferSize,
                       m_configuration.m_prefetch_max_blocks,
                       rg,
@@ -351,7 +501,7 @@ bool Cache::Config(const char *config_filename, const char *parameters)
                       m_configuration.m_flushCnt,
                       m_configuration.m_accHistorySize);
 
-      if (m_configuration.are_dirstats_enabled())
+      if (m_configuration.is_dir_stat_reporting_on())
       {
          loff += snprintf(buff + loff, sizeof(buff) - loff,
                           "       pfc.dirstats maxdepth %d ((internal: store_depth %d, size_of_dirlist %d, size_of_globlist %d))\n",
@@ -392,11 +542,26 @@ bool Cache::Config(const char *config_filename, const char *parameters)
 
    m_log.Say("Config Proxy File Cache g-stream has", m_gstream ? "" : " NOT", " been configured via xrootd.monitor directive");
 
-   m_log.Say("------ Proxy File Cache configuration parsing ", retval ? "completed" : "failed");
+   m_log.Say("------ Proxy File Cache configuration parsing ", aOK ? "completed" : "failed");
 
    if (ofsCfg) delete ofsCfg;
 
-   return retval;
+   // XXXX-CKSUM Testing. To be removed after OssPgi is also merged and valildated.
+   // Building of xrdpfc_print fails when this is enabled.
+#ifdef XRDPFC_CKSUM_TEST
+   {
+      int xxx = m_configuration.m_cs_Chk;
+
+      for (m_configuration.m_cs_Chk = CSChk_None; m_configuration.m_cs_Chk <= CSChk_Both; ++m_configuration.m_cs_Chk)
+      {
+         Info::TestCksumStuff();
+      }
+
+      m_configuration.m_cs_Chk = xxx;
+   }
+#endif
+
+   return aOK;
 }
 
 //------------------------------------------------------------------------------
@@ -467,7 +632,7 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
             {
                return false;
             }
-            if (XrdOuca2x::a2i(m_log, "Error getting purgecoldfiles period", cwg.GetWord(), &m_configuration.m_purgeColdFilesPeriod, 1, 1000))
+            if (XrdOuca2x::a2i(m_log, "Error getting purgecoldfiles period", cwg.GetWord(), &m_configuration.m_purgeAgeBasedPeriod, 1, 1000))
             {
                return false;
             }
@@ -487,14 +652,12 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
    }
    else if ( part == "dirstats" )
    {
-      m_configuration.m_dirStats = true;
-
       const char *p = 0;
-      while ((p = cwg.GetWord()))
+      while ((p = cwg.GetWord()) && cwg.HasLast())
       {
          if (strcmp(p, "maxdepth") == 0)
          {
-            if (XrdOuca2x::a2i(m_log, "Error getting maxdepth value", cwg.GetWord(), &m_configuration.m_dirStatsMaxDepth, 0, 5))
+            if (XrdOuca2x::a2i(m_log, "Error getting maxdepth value", cwg.GetWord(), &m_configuration.m_dirStatsMaxDepth, 0, 16))
             {
                return false;
             }
@@ -553,7 +716,8 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
          }
          else
          {
-            m_log.Emsg("Config", "Error: dirstats stanza contains unknown directive", p);
+            m_log.Emsg("Config", "Error: dirstats stanza contains unknown directive '", p, "'");
+            return false;
          }
       }
    }
@@ -565,6 +729,12 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
       {
          return false;
       }
+      if (m_configuration.m_bufferSize & 0xFFF)
+      {
+         m_configuration.m_bufferSize &= ~0x0FFF;
+         m_configuration.m_bufferSize +=  0x1000;
+         m_log.Emsg("Config", "pfc.blocksize must be a multiple of 4 kB. Rounded up.");
+      }
    }
    else if ( part == "prefetch" || part == "nramprefetch" )
    {
@@ -614,12 +784,11 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
          return false;
       }
    }
-   else if ( part == "hdfsmode" || part == "filefragmentmode" )
+   else if ( part == "hdfsmode" )
    {
-      if (part == "filefragmentmode")
-      {
-         m_log.Emsg("Config", "pfc.filefragmentmode is deprecated, please use pfc.hdfsmode instead. Replacing the directive internally.");
-      }
+      m_log.Emsg("Config", "pfc.hdfsmode is currently unsupported.");
+      return false;
+
       m_configuration.m_hdfsmode = true;
 
       const char* params = cwg.GetWord();
@@ -652,7 +821,7 @@ bool Cache::ConfigParameters(std::string part, XrdOucStream& config, TmpConfigur
    }
    else
    {
-      m_log.Emsg("Cache::ConfigParameters() unmatched pfc parameter", part.c_str());
+      m_log.Emsg("ConfigParameters() unmatched pfc parameter", part.c_str());
       return false;
    }
 
