@@ -28,9 +28,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <stdio.h>
+#include <cstdio>
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/resource.h>
@@ -160,7 +160,7 @@ XrdPosixFile::~XrdPosixFile()
           XrdPosixGlobals::Stats.Count(XrdPosixGlobals::Stats.X.CloseErrs);
       }
 
-// Get rid of defered open object
+// Get rid of deferred open object
 //
    if (PrepIO) delete PrepIO;
 
@@ -212,7 +212,7 @@ do{if (doWait)
 
 // Do some debugging
 //
-   DEBUG("DLY destory of "<<ddCount<<" objects; "<<ddNumLost <<" already lost.");
+   DEBUG("DLY destroy of "<<ddCount<<" objects; "<<ddNumLost <<" already lost.");
 
 // Try to delete all the files on the list. If we exceeded the try limit,
 // remove the file from the list and let it sit forever.
@@ -257,7 +257,7 @@ do{if (doWait)
                                          ddCount,  ddNumLost);
             Say->Emsg("DDestroy", buff);
            } else {
-            DEBUG("DLY destory end; "<<ddCount<<" objects deferred and "
+            DEBUG("DLY destroy end; "<<ddCount<<" objects deferred and "
                              <<ddNumLost <<" lost.");
            }
         if (XrdPosixGlobals::theCache && ddNumLost != nowLost)
@@ -295,7 +295,7 @@ void XrdPosixFile::DelayedDestroy(XrdPosixFile *fp)
    fp->numTries = 0;
    ddMutex.UnLock();
 
-   DEBUG("DLY destory "<<(doPost ? "post " : "has ")<<ddCount
+   DEBUG("DLY destroy "<<(doPost ? "post " : "has ")<<ddCount
                        <<" objects; added "<<fp->Origin());
 
    if (doPost) ddSem.Post();
@@ -307,7 +307,7 @@ void XrdPosixFile::DelayedDestroy(XrdPosixFile *fp)
 
 bool XrdPosixFile::Close(XrdCl::XRootDStatus &Status)
 {
-// If this is a defered open, disable any future calls as we are ready to
+// If this is a deferred open, disable any future calls as we are ready to
 // shutdown this beast!
 //
    if (PrepIO) PrepIO->Disable();
@@ -398,7 +398,7 @@ void XrdPosixFile::HandleResponse(XrdCl::XRootDStatus *status,
    XrdPosixCallBack *xeqCB = theCB;
    int rc = fdNum;
 
-// If no errors occured, complete the open
+// If no errors occurred, complete the open
 //
    if (!(status->IsOK()))          rc = XrdPosixMap::Result(*status);
       else if (!Finalize(&Status)) rc = XrdPosixMap::Result(Status);
@@ -451,12 +451,13 @@ int XrdPosixFile::pgRead(char                  *buff,
                          long long              offs,
                          int                    rlen,
                          std::vector<uint32_t> &csvec,
-                         uint64_t               opts)
+                         uint64_t               opts,
+                         int                   *csfix)
 {
 // Do a sync call using the async interface
 //
    pgioCB pgrCB("Posix pgRead CB");
-   pgRead(pgrCB, buff, offs, rlen, csvec, opts);
+   pgRead(pgrCB, buff, offs, rlen, csvec, opts, csfix);
    return pgrCB.Wait4PGIO();
 }
   
@@ -467,7 +468,8 @@ void XrdPosixFile::pgRead(XrdOucCacheIOCB       &iocb,
                           long long              offs,
                           int                    rlen,
                           std::vector<uint32_t> &csvec,
-                          uint64_t               opts)
+                          uint64_t               opts,
+                          int                   *csfix)
 {
    XrdCl::XRootDStatus Status;
    XrdPosixFileRH *rhP;
@@ -479,7 +481,8 @@ void XrdPosixFile::pgRead(XrdOucCacheIOCB       &iocb,
 
 // Set the destination checksum vector
 //
-   rhP->setCSVec(&csvec, (opts & XrdOucCacheIO::forceCS) != 0);
+   if (csfix) *csfix = 0;
+   rhP->setCSVec(&csvec, csfix, (opts & XrdOucCacheIO::forceCS) != 0);
 
 // Issue read
 //
@@ -502,10 +505,15 @@ int XrdPosixFile::pgWrite(char                  *buff,
                           long long              offs,
                           int                    wlen,
                           std::vector<uint32_t> &csvec,
-                          uint64_t               opts)
+                          uint64_t               opts,
+                          int                   *csfix)
 {
    XrdCl::XRootDStatus Status;
 
+// Preset checksum error count
+//
+   if (csfix) *csfix = 0;
+
 // Issue write and return appropriately
 //
    Ref();
@@ -522,7 +530,8 @@ void XrdPosixFile::pgWrite(XrdOucCacheIOCB       &iocb,
                            long long              offs,
                            int                    wlen,
                            std::vector<uint32_t> &csvec,
-                           uint64_t               opts)
+                           uint64_t               opts,
+                           int                   *csfix)
 {
    XrdCl::XRootDStatus Status;
    XrdPosixFileRH *rhP;
@@ -532,6 +541,13 @@ void XrdPosixFile::pgWrite(XrdOucCacheIOCB       &iocb,
 //
    rhP = XrdPosixFileRH::Alloc(&iocb,this,offs,wlen,XrdPosixFileRH::isWrite);
 
+// Set checksum info
+//
+   if (csfix)
+      {*csfix = 0;
+       rhP->setCSVec(0, csfix);
+      }
+
 // Issue write
 //
    Ref();
