@@ -93,6 +93,7 @@ XrdPosixFileRH *XrdPosixFileRH::Alloc(XrdOucCacheIOCB *cbp,
    newCB->theCB   = cbp;
    newCB->theFile = fp;
    newCB->csVec   = 0;
+   newCB->csfix   = 0;
    newCB->offset  = offs;
    newCB->result  = xResult;
    newCB->typeIO  = typeIO;
@@ -137,6 +138,7 @@ void XrdPosixFileRH::HandleResponse(XrdCl::XRootDStatus *status,
                             }
                     csVec = 0;
                    }
+                if (csfix) *csfix = pInfo->GetNbRepair();
                } else {
                 result = 0;
                 if (csVec) {csVec->clear(); csVec = 0;}
