@@ -28,7 +28,7 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <sys/stat.h>
 
 #include "XProtocol/XProtocol.hh"
@@ -44,6 +44,10 @@
 #define ENOSR ENOSPC
 #endif
 
+#ifndef ECHRNG
+#define ECHRNG EINVAL
+#endif
+
 /******************************************************************************/
 /*                        S t a t i c   M e m b e r s                         */
 /******************************************************************************/
@@ -80,10 +84,6 @@ mode_t XrdPosixMap::Flags2Mode(dev_t *rdv, uint32_t flags)
 /* Private:                      m a p C o d e                                */
 /******************************************************************************/
 
-#ifndef ECHRNG
-#define ECHRNG 44
-#endif
-
 int XrdPosixMap::mapCode(int rc)
 {
     switch(rc)
