@@ -28,10 +28,10 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
 #include <iostream>
-#include <stdio.h>
+#include <cstdio>
 #include <sys/time.h>
 #include <sys/param.h>
 #include <sys/resource.h>
@@ -155,7 +155,7 @@ int OpenDefer(XrdPosixFile           *fp,
 //
    fp->PrepIO = new XrdPosixPrepIO(fp, XOflags, XOmode);
 
-// Finalize this file object. A null argument indicates it is defered.
+// Finalize this file object. A null argument indicates it is deferred.
 //
    fp->Finalize(0);
 
@@ -647,7 +647,7 @@ int XrdPosixXrootd::Open(const char *path, int oflags, mode_t mode,
       }
 
 // Finalize the open (this gets the stat info). For async opens, the
-// finalization is defered until the callback happens.
+// finalization is deferred until the callback happens.
 //
    if (cbP) {errno = EINPROGRESS; return -1;}
    if (fp->Finalize(&Status)) return fp->FDNum();
@@ -952,7 +952,7 @@ struct dirent* XrdPosixXrootd::Readdir(DIR *dirp)
    if (!(dp64 = Readdir64(dirp))) return 0;
 
    dp32 = (struct dirent *)dp64;
-   if (dp32->d_name  != dp64->d_name)
+   if ((char*)dp32->d_name != (char*)dp64->d_name)
       {dp32->d_ino    = dp64->d_ino;
 #if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__GNU__) && !(defined(__FreeBSD_kernel__) && defined(__GLIBC__))
        dp32->d_off     = dp64->d_off;
