@@ -32,11 +32,12 @@
 /******************************************************************************/
   
 #include <unistd.h>
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
 #include <signal.h>
+#include <cstdint>
 #include <strings.h>
-#include <stdio.h>
+#include <cstdio>
 #include <sys/file.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -44,6 +45,7 @@
 #ifdef __solaris__
 #include <sys/vnode.h>
 #endif
+#include <vector>
 
 #include "XrdVersion.hh"
 
@@ -53,12 +55,14 @@
 #include "XrdPss/XrdPssUrlInfo.hh"
 #include "XrdPss/XrdPssUtils.hh"
 #include "XrdPosix/XrdPosixConfig.hh"
+#include "XrdPosix/XrdPosixExtra.hh"
 #include "XrdPosix/XrdPosixInfo.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
 
 #include "XrdOss/XrdOssError.hh"
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucExport.hh"
+#include "XrdOuc/XrdOucPgrwUtils.hh"
 #include "XrdSec/XrdSecEntity.hh"
 #include "XrdSecsss/XrdSecsssID.hh"
 #include "XrdSys/XrdSysError.hh"
@@ -69,6 +73,10 @@
 #define O_DIRECT 0
 #endif
 
+#ifndef ENOATTR
+#define ENOATTR ENODATA
+#endif
+
 /******************************************************************************/
 /*                               D e f i n e s                                */
 /******************************************************************************/
@@ -103,6 +111,8 @@ static const char   *osslclCGI = "oss.lcl=1";
 
 static const int     PBsz = 4096;
 
+       int           rpFD = -1;
+
        bool          idMapAll = false;
 
        bool          outProxy = false; // True means outgoing proxy
@@ -111,7 +121,6 @@ static const int     PBsz = 4096;
 
        XrdSysTrace SysTrace("Pss",0);
 }
-
 using namespace XrdProxy;
 
 /******************************************************************************/
@@ -135,7 +144,7 @@ XrdOss *XrdOssGetStorageSystem2(XrdOss       *native_oss,
 // Ignore the parms (we accept none for now) and call the init routine
 //
    envP = envp;
-   return (XrdProxySS.Init(Logger, cFN) ? 0 : (XrdOss *)&XrdProxySS);
+   return (XrdProxySS.Init(Logger, cFN, envP) ? 0 : (XrdOss *)&XrdProxySS);
 }
 }
  
@@ -148,7 +157,7 @@ XrdOss *XrdOssGetStorageSystem2(XrdOss       *native_oss,
   
 XrdPssSys::XrdPssSys() : LocalRoot(0), theN2N(0), DirFlags(0),
                          myVersion(&XrdVERSIONINFOVAR(XrdOssGetStorageSystem2)),
-                         myFeatures(XRDOSS_HASPRXY|XRDOSS_HASNOSF)
+                         myFeatures(XRDOSS_HASPRXY|XRDOSS_HASPGRW|XRDOSS_HASNOSF)
                          {}
 
 /******************************************************************************/
@@ -162,7 +171,7 @@ XrdPssSys::XrdPssSys() : LocalRoot(0), theN2N(0), DirFlags(0),
 
   Output:   Returns zero upon success otherwise (-errno).
 */
-int XrdPssSys::Init(XrdSysLogger *lp, const char *cFN)
+int XrdPssSys::Init(XrdSysLogger *lp, const char *cFN, XrdOucEnv *envP)
 {
    int NoGo;
    const char *tmp;
@@ -175,7 +184,7 @@ int XrdPssSys::Init(XrdSysLogger *lp, const char *cFN)
 
 // Initialize the subsystems
 //
-   tmp = ((NoGo = Configure(cFN)) ? "failed." : "completed.");
+   tmp = ((NoGo = Configure(cFN, envP)) ? "failed." : "completed.");
    eDest.Say("------ Proxy storage system initialization ", tmp);
 
 // All done.
@@ -766,7 +775,17 @@ int XrdPssFile::Open(const char *path, int Oflag, mode_t Mode, XrdOucEnv &Env)
    if (tpcMode)
       {Oflag &= ~O_NOFOLLOW;
        if (!XrdProxy::outProxy || !IS_FWDPATH(path))
-          {if (rwMode) {tpcPath = strdup(path); return XrdOssOK;}
+          {if (rwMode)
+              {tpcPath = strdup(path);
+               if (XrdPssSys::reProxy)
+                  {const char *rPath = Env.Get("tpc.reproxy");
+                   if (!rPath || *rPath != '/') return -ENOATTR;
+                   if (!(rPath = rindex(rPath, '/')) || *(rPath+1) == 0)
+                      return -EFAULT;
+                   rpInfo = new tprInfo(rPath+1);
+                  }
+               return XrdOssOK;
+              }
            ucgiOK = false;
           }
       }
@@ -845,6 +864,120 @@ int XrdPssFile::Close(long long *retsz)
     return (rc == 0 ? XrdOssOK : -errno);
 }
 
+/******************************************************************************/
+/*                                p g R e a d                                 */
+/******************************************************************************/
+
+/*
+  Function:  Read file pages into a buffer and return corresponding checksums.
+
+  Input:   buffer  - pointer to buffer where the bytes are to be placed.
+           offset  - The offset where the read is to start.
+           rdlen   - The number of bytes to read.
+           csvec   - A vector to be filled with the corresponding CRC32C
+                     checksums for each page or page segment, if available.
+           opts    - Options as noted (see inherited class).
+
+   Output: returns Number of bytes that placed in buffer upon success and
+                   -errno upon failure.
+*/
+
+ssize_t XrdPssFile::pgRead(void     *buffer,
+                           off_t     offset,
+                           size_t    rdlen,
+                           uint32_t *csvec,
+                           uint64_t  opts)
+{
+   std::vector<uint32_t> vecCS;
+   uint64_t psxOpts;
+   ssize_t bytes;
+
+// Make sure file is open
+//
+   if (fd < 0) return (ssize_t)-XRDOSS_E8004;
+
+// Set options as needed
+//
+   psxOpts = (csvec ? XrdPosixExtra::forceCS : 0);
+
+// Issue the pgread
+//
+   if ((bytes = XrdPosixExtra::pgRead(fd,buffer,offset,rdlen,vecCS,psxOpts)) < 0)
+      return (ssize_t)-errno;
+
+// Copy out the checksum vector
+//
+   if (vecCS.size() && csvec)
+       memcpy(csvec, vecCS.data(), vecCS.size()*sizeof(uint32_t));
+
+// All done
+//
+   return bytes;
+}
+
+/******************************************************************************/
+/*                               p g W r i t e                                */
+/******************************************************************************/
+/*
+  Function: Write file pages into a file with corresponding checksums.
+
+  Input:  buffer  - pointer to buffer containing the bytes to write.
+          offset  - The offset where the write is to start.
+          wrlen   - The number of bytes to write.
+                    be the last write to the file at or above the offset.
+          csvec   - A vector which contains the corresponding CRC32 checksum
+                    for each page or page segment. If size is 0, then
+                    checksums are calculated. If not zero, the size must
+                    equal the required number of checksums for offset/wrlen.
+          opts    - Options as noted.
+
+  Output: Returns the number of bytes written upon success and -errno
+                  upon failure.
+*/
+
+ssize_t XrdPssFile::pgWrite(void     *buffer,
+                            off_t     offset,
+                            size_t    wrlen,
+                            uint32_t *csvec,
+                            uint64_t  opts)
+{
+   std::vector<uint32_t> vecCS;
+   ssize_t bytes;
+
+// Make sure we have an open file
+//
+   if (fd < 0) return (ssize_t)-XRDOSS_E8004;
+
+// Check if caller wants to verify the checksums before writing
+//
+   if (csvec && (opts & XrdOssDF::Verify))
+      {XrdOucPgrwUtils::dataInfo dInfo((const char*)buffer,csvec,offset,wrlen);
+       off_t bado;
+       int   badc;
+       if (!XrdOucPgrwUtils::csVer(dInfo, bado, badc)) return -EDOM;
+      }
+
+// Check if caller want checksum generated and possibly returned
+//
+   if ((opts & XrdOssDF::doCalc) || csvec == 0)
+      {XrdOucPgrwUtils::csCalc((const char *)buffer, offset, wrlen, vecCS);
+       if (csvec) memcpy(csvec, vecCS.data(), vecCS.size()*sizeof(uint32_t));
+      } else {
+       int n = XrdOucPgrwUtils::csNum(offset, wrlen);
+       vecCS.resize(n);
+       vecCS.assign(n, 0);
+       memcpy(vecCS.data(), csvec, n*sizeof(uint32_t));
+      }
+
+// Issue the pgwrite
+//
+   bytes = XrdPosixExtra::pgWrite(fd, buffer, offset, wrlen, vecCS);
+
+// Return result
+//
+   return (bytes < 0 ? (ssize_t)-errno : bytes);
+}
+
 /******************************************************************************/
 /*                                  r e a d                                   */
 /******************************************************************************/
@@ -978,15 +1111,76 @@ ssize_t XrdPssFile::Write(const void *buff, off_t offset, size_t blen)
 
 int XrdPssFile::Fstat(struct stat *buff)
 {
-    if (fd < 0)
-       {if (!tpcPath) return -XRDOSS_E8004;
-        XrdOucEnv fstatEnv(0, 0, entity);
-        if (XrdProxySS.Stat(tpcPath, buff, 0, &fstatEnv))
-           memset(buff, 0, sizeof(struct stat));
-        return XrdOssOK;
-       }
+// If we have a file descriptor then return a stat for it
+//
+   if (fd >= 0) return (XrdPosixXrootd::Fstat(fd, buff) ? -errno : XrdOssOK);
 
-    return (XrdPosixXrootd::Fstat(fd, buff) ? -errno : XrdOssOK);
+// Otherwise, if this is not a tpc of any kind, return an error
+//
+   if (!tpcPath) return -XRDOSS_E8004;
+
+// If this is a normal tpc then simply issue the stat against the origin
+//
+   if (!rpInfo)
+      {XrdOucEnv fstatEnv(0, 0, entity);
+       return XrdProxySS.Stat(tpcPath, buff, 0, &fstatEnv);
+      }
+
+// This is a reproxy tpc, if we have not yet dertermined the true dest, do so.
+//
+   struct stat Stat;
+
+   if (rpInfo->dstURL == 0
+   ||  !fstatat(rpFD, rpInfo->tprPath, &Stat, AT_SYMLINK_NOFOLLOW))
+      {char lnkbuff[2048]; int lnklen;
+       lnklen = readlinkat(rpFD, rpInfo->tprPath, lnkbuff, sizeof(lnkbuff)-1);
+       if (lnklen <= 0)
+          {int rc = 0;
+           if (lnklen < 0) {if (errno != ENOENT) rc = -errno;}
+               else rc = -EFAULT;
+           if (rc)
+              {unlinkat(rpFD, rpInfo->tprPath, 0);
+               return rc;
+              }
+          } else {
+            unlinkat(rpFD, rpInfo->tprPath, 0);
+            lnkbuff[lnklen] = 0;
+            if (rpInfo->dstURL) free(rpInfo->dstURL);
+            rpInfo->dstURL = strdup(lnkbuff);
+            rpInfo->fSize = 1;
+std::cerr<<"Pss_fstat: "<<tident<<" "<<rpInfo->tprPath<<" maps "
+         <<tpcPath<<" -> "<<lnkbuff<<"\n"<<std::flush;
+          }
+      }
+
+// At this point we may or may not have the final endpoint. An error here could
+// be due to write error recovery, so make allowance for that.
+//
+   if (rpInfo->dstURL)
+      {if (!XrdPosixXrootd::Stat(rpInfo->dstURL, buff))
+          {if (!(rpInfo->fSize = buff->st_size)) rpInfo->fSize = 1;
+           return XrdOssOK;
+          }
+       free(rpInfo->dstURL);
+       rpInfo->dstURL = 0;
+      }
+
+// We don't have the final endpoint. If we ever had it before, then punt.
+//
+   if (rpInfo->fSize)
+      {memset(buff, 0, sizeof(struct stat));
+       buff->st_size = rpInfo->fSize;
+       return XrdOssOK;
+      }
+
+// If we are here then maybe the reproxy option was the wrong config setting.
+// Give stat a try on the origin we'll retry resolution on the next stat.
+//
+   XrdOucEnv fstatEnv(0, 0, entity);
+
+   if (XrdProxySS.Stat(tpcPath, buff, 0, &fstatEnv))
+       memset(buff, 0, sizeof(struct stat));
+   return XrdOssOK;
 }
 
 /******************************************************************************/
