@@ -29,9 +29,9 @@
 /******************************************************************************/
 
 #include <unistd.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <string.h>
+#include <cctype>
+#include <cstdio>
+#include <cstring>
 #include <strings.h>
 #include <sys/param.h>
 #include <sys/types.h>
@@ -50,6 +50,7 @@
 #include "XrdPss/XrdPssUtils.hh"
 
 #include "XrdSys/XrdSysError.hh"
+#include "XrdSys/XrdSysFD.hh"
 #include "XrdSys/XrdSysHeaders.hh"
 #include "XrdSys/XrdSysPlatform.hh"
 #include "XrdSys/XrdSysPthread.hh"
@@ -87,7 +88,7 @@
 
 #define TS_DBG(x,m)    if (!strcmp(x,var)) {SysTrace.What |= m; return 0;}
 
-/*******x**********************************************************************/
+/******************************************************************************/
 /*                               G l o b a l s                                */
 /******************************************************************************/
 
@@ -112,6 +113,7 @@ bool         XrdPssSys::xLfn2Pfn  = false;
 bool         XrdPssSys::dcaCheck  = false;
 bool         XrdPssSys::dcaWorld  = false;
 bool         XrdPssSys::deferID   = false;
+bool         XrdPssSys::reProxy   = false;
 
 namespace XrdProxy
 {
@@ -125,6 +127,8 @@ extern XrdOucEnv       *envP;
 
 extern XrdSecsssID     *idMapper; // -> Auth ID mapper
 
+extern int              rpFD;
+
 extern bool             idMapAll;
 
 extern bool             outProxy; // True means outgoing proxy
@@ -151,7 +155,7 @@ using namespace XrdProxy;
 /*                             C o n f i g u r e                              */
 /******************************************************************************/
   
-int XrdPssSys::Configure(const char *cfn)
+int XrdPssSys::Configure(const char *cfn, XrdOucEnv *envP)
 {
 /*
   Function: Establish configuration at start up time.
@@ -196,6 +200,10 @@ int XrdPssSys::Configure(const char *cfn)
 //
    XrdPosixConfig::SetEnv("ParallelEvtLoop", 3);
 
+// Turn off the fork handler as we always exec after forking.
+//
+   XrdPosixConfig::SetEnv("RunForkHandler", 0);
+
 // Process the configuration file
 //
    if ((NoGo = ConfigProc(cfn))) return NoGo;
@@ -226,6 +234,28 @@ int XrdPssSys::Configure(const char *cfn)
        psxConfig->xLfn2Pfn = false;
       }
 
+// If we have a cache, indicate so in the feature set
+//
+   if(psxConfig->hasCache()) myFeatures |= XRDOSS_HASCACH;
+
+// If we need to reproxy, then open the directory where the reproxy information
+// will ne placed. The path is in the Env.
+//
+   if (reProxy)
+      {char *rPath;
+       if (!envP || !(rPath = envP->Get("tpc.rpdir")))
+          {eDest.Say("Config warning: ignoring 'pss.reproxy'; TPC is not enabled!");
+           reProxy = false;
+           myFeatures &= ~XRDOSS_HASRPXY;
+          } else {
+           rpFD = XrdSysFD_Open(rPath, O_DIRECTORY);
+           if (rpFD < 0)
+              {eDest.Emsg("Config", "to open reproxy directory", rPath);
+               return 1;
+              }
+          }
+      }
+
 // Finalize the configuration
 //
    if (!(psxConfig->ConfigSetup(eDest))) return 1;
@@ -292,7 +322,7 @@ int XrdPssSys::Configure(const char *cfn)
    if (ManList) sprintf(theRdr, "%s%s:%d", outeq, ManList->text, ManList->val);
       else strcpy(theRdr, outeq);
    XrdOucEnv::Export("XRDXROOTD_PROXY",  theRdr);
-   XrdOucEnv::Export("XRDXROOTD_ORIGIN", theRdr); // Backward compatability
+   XrdOucEnv::Export("XRDXROOTD_ORIGIN", theRdr); // Backward compatibility
 
 // Construct the contact URL header
 //
@@ -419,7 +449,7 @@ int XrdPssSys::ConfigProc(const char *Cfn)
             if (ConfigXeq(var+4, Config)) {Config.Echo(); NoGo = 1;}
         }
 
-// Now check if any errors occured during file i/o
+// Now check if any errors occurred during file i/o
 //
    if ((retc = Config.LastError()))
       NoGo = eDest.Emsg("Config", retc, "read config file", Cfn);
@@ -462,6 +492,13 @@ int XrdPssSys::ConfigXeq(char *var, XrdOucStream &Config)
    TS_PSX("setopt",        ParseSet);
    TS_PSX("trace",         ParseTrace);
 
+   if (!strcmp("reproxy", var))
+      {myFeatures |= XRDOSS_HASRPXY;
+       reProxy = true;
+       Config.GetWord(); // Force echo
+       return 0;
+      }
+
    // Copy the variable name as this may change because it points to an
    // internal buffer in Config. The vagaries of effeciency. Then get value.
    //
