@@ -29,12 +29,12 @@
 /******************************************************************************/
 
 #include <alloca.h>
-#include <ctype.h>
+#include <cctype>
 #include <iostream>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <strings.h>
-#include <stdio.h>
+#include <cstdio>
 #include <sys/param.h>
 #include <unistd.h>
 
@@ -309,8 +309,8 @@ if (!(decKey.Data.Opts & XrdSecsssKT::ktEnt::noIPCK))
 
        // Set corresponding uid and gid
        //
-          XrdOucUtils::getUID(myID.name, Entity.uid, &Entity.gid);
-          XrdOucUtils::getGID(myID.grps, Entity.gid);
+          if (myID.name) XrdOucUtils::getUID(myID.name, Entity.uid, &Entity.gid);
+          if (myID.grps) XrdOucUtils::getGID(myID.grps, Entity.gid);
        }
 
 // Calculate the amount of space we will need
@@ -586,7 +586,7 @@ int XrdSecProtocolsss::getCred(XrdOucErrInfo       *einfo,
    if ((dLen = Decode(einfo, decKey, parm->buffer, &prData, parm->size)) <= 0)
       return Fatal(einfo, "getCred", EINVAL, "Unable to decode server response.");
 
-// Extract out the loginid. This messy code is for backwards compatability.
+// Extract out the loginid. This messy code is for backwards compatibility.
 //
    bP = prData.Data; eodP = dLen + (char *)&prData;
    while(bP < eodP)
@@ -682,10 +682,11 @@ XrdSecCredentials *XrdSecProtocolsss::getCredentials(XrdSecParameters *parms,
 // Determine if we should send the keyname (v2 servers only)
 //
    if (v2EndPnt)
-      {strcpy(rrHdr.keyName, encKey.Data.Name);
-       int n = (strlen(rrHdr.keyName) + 8) & ~7;
+      {int k = strlen(encKey.Data.Name), n = (k + 8) & ~7;
+       strcpy(rrHdr.keyName, encKey.Data.Name);
+       if (n - k > 1) memset(rrHdr.keyName + k, 0, n - k);
        rrHdr.knSize = static_cast<uint8_t>(n);
-      }
+      } else rrHdr.knSize = 0;
 
 // Now simply encode the data and return the result
 //
@@ -831,7 +832,7 @@ char *XrdSecProtocolsss::Load_Client(XrdOucErrInfo *erp, const char *parms)
 
 // We want to establish the default location of the keytable. First check
 // the environment passed from the client then the envar. We support two
-// version of the envar for backward compatability due to an early mistake.
+// version of the envar for backward compatibility due to an early mistake.
 //
    if( erp && erp->getEnv() && ( kP = erp->getEnv()->Get( "xrd.sss" ) ) )
      ktFixed = true;
@@ -1019,7 +1020,7 @@ char *XrdSecProtocolsss::Load_Server(XrdOucErrInfo *erp, const char *parms)
    CLDBG("Server keytab='" <<ktServer <<"'");
 
 // Construct client parameter <enccode>.+<lifetime>:<keytab>
-// Note: The plus preceeding the <lifetime> indicates that we are a V2 server.
+// Note: The plus preceding the <lifetime> indicates that we are a V2 server.
 // V1 clients will simply ignore this and treat us as a V1 server.
 //
    sprintf(buff, "%c.+%s%d:%s", CryptObj->Type(),ask4Creds,lifeTime,ktClient);
