@@ -30,10 +30,10 @@
 
 #include <fcntl.h>
 #include <iostream>
-#include <stdio.h>
+#include <cstdio>
 #include <stddef.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -44,6 +44,10 @@
 #include "XrdOuc/XrdOucStream.hh"
 #include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSys/XrdSysE2T.hh"
+
+#ifndef ENODATA
+#define ENODATA ENOATTR
+#endif
   
 /******************************************************************************/
 /*                    S t a t i c   D e f i n i t i o n s                     */
