@@ -31,17 +31,17 @@
 #define __STDC_FORMAT_MACROS 1
 #include <alloca.h>
 #include <unistd.h>
-#include <ctype.h>
-#include <errno.h>
+#include <cctype>
+#include <cerrno>
 #include <fcntl.h>
-#include <inttypes.h>
+#include <cinttypes>
 #include <iostream>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cstdint>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <strings.h>
-#include <time.h>
+#include <ctime>
 #include <vector>
 #include <arpa/inet.h>
 #include <sys/stat.h>
@@ -217,12 +217,13 @@ struct TokenHdr
       {char     id[4];
        char     ver;
        char     opr;
+       char     rsvd[2]; // Reserved bytes (note struct is 8 bytes long)
 
        static const char SndAI = 'S';
        static const char IsTkn = 'T';
 
        void     Fill(char opc) {strcpy(id, "ztn"); ver = ztnVersion;
-                                opr = opc;
+                                opr = opc; rsvd[0] = rsvd[1] = 0;
                                }
       };
 
@@ -454,7 +455,7 @@ XrdSecCredentials *XrdSecProtocolztn::readToken(XrdOucErrInfo *erp,
 // Make sure token is not too big
 //
    if (Stat.st_size > maxTSize) return readFail(erp, path, EMSGSIZE);
-   buff = (char *)alloca(Stat.st_size);
+   buff = (char *)alloca(Stat.st_size+1);
 
 // Open the token file
 //
@@ -470,6 +471,10 @@ XrdSecCredentials *XrdSecProtocolztn::readToken(XrdOucErrInfo *erp,
       }
    close(tokFD);
 
+// Make sure the token ends with a null byte
+//
+   buff[Stat.st_size] = 0;
+
 // Strip the token
 //
    if (!(bTok = Strip(buff, sz)))
@@ -483,7 +488,7 @@ XrdSecCredentials *XrdSecProtocolztn::readToken(XrdOucErrInfo *erp,
 
 // Return response
 //
-   return retToken(erp, buff, Stat.st_size);
+   return retToken(erp, bTok, sz);
 }
   
 /******************************************************************************/
@@ -569,6 +574,7 @@ int XrdSecProtocolztn::Authenticate(XrdSecCredentials *cred,
                                     XrdSecParameters **parms,
                                     XrdOucErrInfo     *erp)
 {
+   static const int pfxLen = sizeof(TokenHdr) + sizeof(uint16_t);
    TokenResp *tResp;
 
 // Check if we have any credentials or if no credentials really needed.
@@ -604,11 +610,19 @@ int XrdSecProtocolztn::Authenticate(XrdSecCredentials *cred,
 
 // Make sure the response is consistent
 //
+   const char *isBad = 0;
    int tLen = ntohs(tResp->len);
-   if (tResp->hdr.ver != ztnVersion || tLen < 1
-   ||  (int(sizeof(TokenResp) + tLen)) > cred->size
-   || !(tResp->tkn[0]) || *(tResp->tkn+(tLen-1)))
-      {Fatal(erp, "'ztn' token response malformed", EINVAL, false);
+
+        if (tResp->hdr.ver != ztnVersion) isBad = "version mismatch";
+   else if (tLen < 1)                     isBad = "token length < 1";
+   else if (pfxLen + tLen > cred->size)   isBad = "respdata > credsize";
+   else if (!(tResp->tkn[0]))             isBad = "null token";
+   else if (*(tResp->tkn+(tLen-1)))       isBad = "missing null byte";
+
+   if (isBad)
+      {char eText[80];
+       snprintf(eText, sizeof(eText), "'ztn' token malformed; %s", isBad);
+       Fatal(erp, eText, EINVAL, false);
        return -1;
       }
 
