@@ -31,7 +31,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <stdlib.h>
+#include <cstdlib>
 
 #include "XrdVersion.hh"
 #include "XrdOss/XrdOss.hh"
