@@ -26,20 +26,20 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 #include <unistd.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <errno.h>
-#include <time.h>
+#include <ctime>
 
 #include "XrdSut/XrdSutAux.hh"
 #include "XrdSut/XrdSutPFEntry.hh"
 #include "XrdSut/XrdSutPFile.hh"
 #include "XrdSut/XrdSutTrace.hh"
+#include "XrdSys/XrdSysE2T.hh"
 
 //_________________________________________________________________
 XrdSutPFEntInd::XrdSutPFEntInd(const char *n, kXR_int32 no,
@@ -1694,26 +1694,46 @@ kXR_int32 XrdSutPFile::Browse(void *oout)
                   }
                   char smt[20] = {0};
                   XrdSutTimeString(ent.mtime,smt);
+		  std::string buffer;
                   char buf[2048] = {0};
                   memset(buf,0,2048);
                   sprintf(buf,"// #%d mod:%s",i+1,smt);
-                  sprintf(buf,"%s name:%s",buf,ind.name);
-                  fprintf(out,"%s\n",buf);
+                  buffer = buf;
+                  sprintf(buf," name:%s",ind.name);
+                  buffer += buf;
+                  fprintf(out,"%s\n",buffer.c_str());
+
+		  buffer.clear();
                   sprintf(buf,"//    buf");
+		  buffer = buf;
+
                   if (ent.cnt == 1) {
                      if (ent.buf1.len && ent.buf1.buf)
-                        sprintf(buf,"%s: %.*s",buf,ent.buf1.len,ent.buf1.buf);
+		     {
+                        sprintf(buf,": %.*s",ent.buf1.len,ent.buf1.buf);
+			buffer += buf;
+		     }
                      if (ent.buf2.len && ent.buf2.buf)
-                        sprintf(buf,"%s: %.*s",buf,ent.buf2.len,ent.buf2.buf);
+		     {
+                        sprintf(buf,": %.*s",ent.buf2.len,ent.buf2.buf);
+			buffer += buf;
+		     }
                      if (ent.buf3.len && ent.buf3.buf)
-                        sprintf(buf,"%s: %.*s",buf,ent.buf3.len,ent.buf3.buf);
+		     {
+                        sprintf(buf,": %.*s",ent.buf3.len,ent.buf3.buf);
+			buffer += buf;
+		     }
                      if (ent.buf4.len && ent.buf4.buf)
-                        sprintf(buf,"%s: %.*s",buf,ent.buf4.len,ent.buf4.buf);
+		     {
+                        sprintf(buf,": %.*s",ent.buf4.len,ent.buf4.buf);
+			buffer += buf;
+		     }
                   } else {
-                     sprintf(buf,"%s:%d:%d:%d:%d",buf,
+                     sprintf(buf,":%d:%d:%d:%d",
                                  ent.buf1.len,ent.buf2.len,ent.buf3.len,
                                  ent.buf4.len);
-                     sprintf(buf,"%s (protected)",buf);
+		     buffer += buf;
+		     buffer += " (protected)";
                   }
                   fprintf(out,"%s\n",buf);
                }
@@ -1941,7 +1961,7 @@ kXR_int32 XrdSutPFile::UpdateHashTable(bool force)
 {
    // Update hash table reflecting the index of the file
    // If force is .true. the table is recreated even if no recent 
-   // change in the index has occured.
+   // change in the index has occurred.
    // Returns the number of entries in the table.
 
    // The file must be open
@@ -2193,7 +2213,7 @@ kXR_int32 XrdSutPFile::Err(kXR_int32 code, const char *loc,
    fError = code;
 
    // Build string following the error code
-   char *errbuf = strerror(errno);
+   const char *errbuf = XrdSysE2T(errno);
    switch (code) {
       case kPFErrBadInputs:
          snprintf(buf,XrdSutMAXBUF,
