@@ -28,8 +28,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
   
 #include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysFD.hh"
@@ -38,10 +38,26 @@
 #include "XrdSys/XrdSysPlatform.hh"
 #include "XrdSys/XrdSysPthread.hh"
 
+/******************************************************************************/
+/*                            L o c a l   D a t a                             */
+/******************************************************************************/
+
+namespace
+{
+// Status code to name array corresponding to:
+// enum Status   {isClear = 0, isCBMode, isDead};
+//
+   const char *statName[] = {"isClear", "isCBMode", "isDead"};
+}
+  
 /******************************************************************************/
 /*                         L o c a l   D e f i n e s                          */
 /******************************************************************************/
 
+#define STATUS statName[(int)chStat]
+
+#define STATUSOF(x) statName[(int)(x->chStat)]
+
 #define SINGLETON(dlvar, theitem)\
                theitem ->dlvar .next == theitem
   
@@ -69,7 +85,7 @@
 
 #define DO_TRACE(x,fd,y) \
                 {PollerInit::traceMTX.Lock(); \
-                 cerr <<"IOE fd " <<fd <<' ' <<#x <<": " <<y <<endl; \
+                 cerr <<"IOE fd "<<fd<<' '<<#x <<": "<<y<<'\n'<<flush; \
                  PollerInit::traceMTX.UnLock();}
 
 #define TRACING PollerInit::doTrace
@@ -285,6 +301,10 @@ void XrdSys::IOEvents::Channel::Delete()
    Poller *myPoller;
    bool isLocked = true;
 
+// Do some tracing
+//
+   IF_TRACE(Delete,chFD,"status="<<STATUS);
+
 // Lock ourselves during the delete process. If the channel is disassociated
 // or the real poller is set to the error poller then this channel is clean
 // and can be deleted (i.e. the channel ran through Detach()).
@@ -312,6 +332,7 @@ void XrdSys::IOEvents::Channel::Delete()
            chMutex.UnLock();
           } else {
            XrdSysSemaphore cbDone(0);
+           IF_TRACE(Delete,chFD,"waiting for callback");
            chStat = isDead;
            chCBA  = (void *)&cbDone;
            chMutex.UnLock();
@@ -320,6 +341,7 @@ void XrdSys::IOEvents::Channel::Delete()
       }
 // It is now safe to release the storage
 //
+   IF_TRACE(Delete,chFD,"chan="<<hex<<(void *)this<<dec);
    delete this;
 }
   
@@ -860,7 +882,7 @@ int XrdSys::IOEvents::Poller::GetRequest()
    do {rlen = read(reqFD, pipeBuff, pipeBlen);} 
       while(rlen < 0 && errno == EINTR);
    if (rlen <= 0)
-      {cerr <<"Poll: " <<XrdSysE2T(errno) <<" reading from request pipe" <<endl;
+      {cerr <<"Poll: "<<XrdSysE2T(errno)<<" reading from request pipe\n"<<flush;
        return 0;
       }
 
@@ -1066,6 +1088,11 @@ bool XrdSys::IOEvents::Poller::TmoAdd(XrdSys::IOEvents::Channel *cP, int tmoSet)
    Channel *ncP;
    bool setRTO, setWTO;
 
+// Do some tracing
+//
+   IF_TRACE(TmoAdd,cP->chFD,"chan="<<hex<<(void*)cP<<dec
+            <<" inTOQ="<<BOOLNAME(cP->inTOQ)<<" status="<<STATUSOF(cP));
+
 // Remove element from timeout queue if it is there
 //
    if (cP->inTOQ)
@@ -1125,6 +1152,11 @@ bool XrdSys::IOEvents::Poller::TmoAdd(XrdSys::IOEvents::Channel *cP, int tmoSet)
 void XrdSys::IOEvents::Poller::TmoDel(XrdSys::IOEvents::Channel *cP)
 {
 
+// Do some tracing
+//
+   IF_TRACE(TmoDel,cP->chFD,"chan="<<hex<<(void*)cP<<dec
+            <<" inTOQ="<<BOOLNAME(cP->inTOQ)<<" status="<<STATUSOF(cP));
+
 // Get the timeout queue lock and remove the channel from the queue
 //
    toMutex.Lock();
