@@ -28,9 +28,9 @@
 /******************************************************************************/
 
 #include <stddef.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <unistd.h>
-#include <stdio.h>
+#include <cstdio>
 
 #include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysLogger.hh"
@@ -107,7 +107,7 @@ bool XrdSysLogging::Configure(XrdSysLogger &logr, Parms &parms)
 // Allocate a log buffer
 //
    int bsz = (parms.bufsz < 0 ? 65536 : parms.bufsz);
-   rc = posix_memalign(reinterpret_cast<void**>(&buffOrg), getpagesize(), bsz);
+   rc = posix_memalign((void **)&buffOrg, getpagesize(), bsz);
    if (rc != 0 || !buffOrg) return EMsg(logr, "Unable to allocate log buffer!\n");
 
    buffBeg = buffOrg + buffOvhd;
