@@ -33,10 +33,10 @@
 /*                                                                            */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <cerrno>
 
 #include "XrdVoms.hh"
 #include "XrdVomsFun.hh"
@@ -438,6 +438,7 @@ int XrdVomsFun::VOMSInit(const char *cfg)
    //         dbg                    To force verbose mode
    //
    EPNAME("Init");
+   vomsdata vomsInit; // This forces libssl initialization at load time
 
    XrdOucString oos(cfg);
 
