@@ -28,11 +28,11 @@
 /******************************************************************************/
  
 #include <unistd.h>
-#include <ctype.h>
+#include <cctype>
 #include <fcntl.h>
 #include <string>
-#include <string.h>
-#include <stdio.h>
+#include <cstring>
+#include <cstdio>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -49,6 +49,7 @@
 #include "XrdSfs/XrdSfsFlags.hh"
 #include "XrdSfs/XrdSfsInterface.hh"
 #include "XrdNet/XrdNetOpts.hh"
+#include "XrdNet/XrdNetPMarkCfg.hh"
 #include "XrdNet/XrdNetSocket.hh"
 #include "XrdOuc/XrdOuca2x.hh"
 #include "XrdOuc/XrdOucEnv.hh"
@@ -226,6 +227,17 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
    if (rdf && Config(rdf)) return 0;
    if (pi->DebugON) XrdXrootdTrace.What = TRACE_ALL;
 
+// Initialize the packet marking framework if configured. We do that here as
+// nothing else following this code can fail but we can so be consistent.
+//
+   bool bad = false;
+   PMark = XrdNetPMarkCfg::Config(&eDest, pi->Sched, &XrdXrootdTrace, bad);
+   if (PMark)
+      {if (pi->theEnv) pi->theEnv->PutPtr("XrdNetPMark*", PMark);
+       xrootdEnv.PutPtr("XrdNetPMark*", PMark);
+      }
+      else if (bad) return 0;
+
 // Check if we are exporting a generic object name
 //
    if (XPList.Opts() & XROOTDXP_NOSLASH)
@@ -320,7 +332,7 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 //
    if (!(asyncFlags & asDebug) && as_aioOK)
       {if (fsFeatures & XrdSfs::hasNAIO) as_aioOK = 0;
-          else if (asyncFlags && asNoCache && fsFeatures & XrdSfs::hasCACH)
+          else if (asyncFlags & asNoCache && fsFeatures & XrdSfs::hasCACH)
                   as_aioOK = 0;
        if (!as_aioOK) eDest.Say("Config asynchronous I/O has been disabled!");
       }
@@ -377,6 +389,14 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
    if (fsFeatures & XrdSfs::hasPRXY) myRole |= kXR_attrProxy;
    myRole |= tlsFlags;
 
+// Turn off client redirects if we are neither a redirector nor a proxy server
+//
+   if (CL_Redir && !isRedir && !isProxy)
+      {CL_Redir = false;
+       eDest.Say("Config warning: 'redirect client' ignored; "
+                 "not a redirector nor a proxy server");
+      }
+
 // Check if we are redirecting anything
 //
    if ((xp = RPList.Next()))
@@ -438,7 +458,7 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
    if (!(AdminSock = XrdNetSocket::Create(&eDest, adminp, "admin", pi->AdmMode))
    ||  !XrdXrootdAdmin::Init(&eDest, AdminSock)) return 0;
 
-// Indicate whether or not we support extened attributes
+// Indicate whether or not we support extended attributes
 //
   {XrdOucEnv     myEnv;
    XrdOucErrInfo eInfo("", &myEnv);
@@ -491,6 +511,7 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 /******************************************************************************/
   
 #define TS_Xeq(x,m) (!strcmp(x,var)) GoNo = m(Config)
+#define TS_Zeq(x,m) (!strcmp(x,var)) GoNo = m(&eDest, Config)
 
 int XrdXrootdProtocol::Config(const char *ConfigFN)
 {
@@ -519,6 +540,7 @@ int XrdXrootdProtocol::Config(const char *ConfigFN)
 
          if (ismine)
             {     if TS_Xeq("async",         xasync);
+             else if TS_Xeq("bindif",        xbif);
              else if TS_Xeq("chksum",        xcksum);
              else if TS_Xeq("diglib",        xdig);
              else if TS_Xeq("export",        xexp);
@@ -528,6 +550,7 @@ int XrdXrootdProtocol::Config(const char *ConfigFN)
              else if TS_Xeq("log",           xlog);
              else if TS_Xeq("mongstream",    xmongs);
              else if TS_Xeq("monitor",       xmon);
+             else if TS_Zeq("pmark",         XrdNetPMarkCfg::Parse);
              else if TS_Xeq("prep",          xprep);
              else if TS_Xeq("redirect",      xred);
              else if TS_Xeq("seclib",        xsecl);
@@ -890,6 +913,82 @@ int XrdXrootdProtocol::xasync(XrdOucStream &Config)
    return 0;
 }
 
+/******************************************************************************/
+/*                                  x b i f                                   */
+/******************************************************************************/
+  
+/* Function: xbif
+
+   Purpose:  To parse the directive: bindif <trg>
+
+   <trg>:    <host>:<port>[%<prvhost>:<port>]] [<trg>]
+*/
+
+namespace XrdXrootd
+{
+char *bifResp[2] = {0,0};
+int   bifRLen[2] = {0,0};
+}
+
+int XrdXrootdProtocol::xbif(XrdOucStream &Config)
+{
+   static const int brSize = sizeof(XrdProto::bifReqs);
+   using XrdXrootd::bifResp;
+   using XrdXrootd::bifRLen;
+
+   XrdOucString bSpec[2];
+   char *bHost[2], *val, buff[512];
+   int   bPort[2], thePort;
+
+// Cleanup any previous bif specification
+//
+   if (bifResp[1])
+      {if (bifResp[1] != bifResp[0]) free(bifResp[1]);
+       bifResp[1] = 0; bifRLen[1] = 0;
+      }
+   if (bifResp[0])
+      {free(bifResp[0]);
+       bifResp[0] = 0; bifRLen[0] = 0;
+      }
+
+// Process all of the options
+//
+   while((val = Config.GetWord()) && *val)
+        {if (!xred_php(val, bHost, bPort, "bindif", true)) return 1;
+         for (int i = 0; i < 2 && bHost[i] != 0; i++)
+             {thePort = (bPort[i] ? bPort[i] : XrdXrootdPort);
+              snprintf(buff, sizeof(buff), "%s%s:%d",
+                      (bSpec[i].length() ? "," : ""),  bHost[i], thePort);
+              bSpec[i] += buff;
+             }
+        }
+
+// Generate the "b" record for each type of interface
+//
+   for (int i = 0; i < 2 && bSpec[i].length(); i++)
+       {int n = brSize + bSpec[i].length() + 1;
+        n = (n + 7) & ~7;
+        XrdProto::bifReqs *bifRec = (XrdProto::bifReqs *)malloc(n);
+        memset(bifRec, 0, n);
+        bifRec->theTag = 'B';
+        bifRec->bifILen = htons(static_cast<kXR_unt16>(n-brSize));
+        strcpy(((char *)bifRec)+brSize, bSpec[i].c_str());
+        bifResp[i] = (char *)bifRec;
+        bifRLen[i] = n;
+       }
+
+// Now complete the definition
+//
+   if (bifResp[0] && bifResp[1] == 0)
+      {bifResp[1] = bifResp[0];
+       bifRLen[1] = bifRLen[0];
+      }
+
+// All done
+//
+   return 0;
+}
+
 /******************************************************************************/
 /*                                x c k s u m                                 */
 /******************************************************************************/
@@ -1219,7 +1318,7 @@ int XrdXrootdProtocol::xfso(XrdOucStream &Config)
          else if (!strcmp(val, "nobypass")) bypass = 0;
          else if (!strcmp(val, "redirect"))
                  {val = Config.GetWord();
-                  if (!xred_php(val, hP, rPort)) return 1;
+                  if (!xred_php(val, hP, rPort, "redirect")) return 1;
                   for (int i = 0; i < 2; i++)
                       {if (!hP[i]) rHost[i][0] = 0;
                           else {strlcpy(rHost[i], hP[i], rHLen);
@@ -1412,7 +1511,8 @@ int XrdXrootdProtocol::xprep(XrdOucStream &Config)
 /* Function: xred
 
    Purpose:  To parse the directive: redirect <host>:<port>[%<prvhost>:<port>]
-                                              {<funcs>|[?]<path>}
+                                              {<funcs>|[?]<path>} |
+                                              client <domlist>
 
              <funcs>   are one or more of the following functions that will
                        be immediately redirected to <host>:<port>. Each function
@@ -1426,6 +1526,8 @@ int XrdXrootdProtocol::xprep(XrdOucStream &Config)
                        then the redirect applies to any operation on the path
                        that results in an ENOENT error.
 
+             <domlist> {private | local | .<domain>} [<domlist>]
+
   Output: 0 upon success or !0 upon failure.
 */
 
@@ -1454,7 +1556,7 @@ int XrdXrootdProtocol::xred(XrdOucStream &Config)
 // Get the host and port
 //
    val = Config.GetWord();
-   if (!xred_php(val, hP, rPort)) return 1;
+   if (!xred_php(val, hP, rPort, "redirect")) return 1;
 
 // Copy out he values as the target variable will be lost
 //
@@ -1470,6 +1572,10 @@ int XrdXrootdProtocol::xred(XrdOucStream &Config)
     if (!(val = Config.GetWord()))
        {eDest.Emsg("config", "redirect option not specified"); return 1;}
 
+// Handle the client option
+//
+   if (!strcmp("client", val)) return xred_clnt(Config, hP, rPort);
+
     if (*val == '/' || (isQ = ((*val == '?') || !strcmp(val,"enoent"))))
        {if (isQ)
            {RQLxist = 1;
@@ -1485,7 +1591,7 @@ int XrdXrootdProtocol::xred(XrdOucStream &Config)
         for (k = static_cast<int>(RD_open1); k < RD_Num; k++)
             if (xred_xok(k, hP, rPort)) break;
         if (k >= RD_Num)
-           {eDest.Emsg("Config", "too many diffrent path redirects"); return 1;}
+           {eDest.Emsg("Config", "too many different path redirects"); return 1;}
         xred_set(RD_func(k), hP, rPort);
         do {if (isQ) RQList.Insert(val, k, 0);
                else  RPList.Insert(val, k, 0);
@@ -1518,14 +1624,67 @@ int XrdXrootdProtocol::xred(XrdOucStream &Config)
    return 0;
 }
 
-bool XrdXrootdProtocol::xred_php(char *val, char *hP[2], int rPort[2])
+/******************************************************************************/
+
+int XrdXrootdProtocol::xred_clnt(XrdOucStream &Config,char *hP[2],int rPort[2])
 {
+   static const int maxDom = sizeof(RouteClient.Domain)/sizeof(char*);
+   char *val;
+
+// Reset values
+//
+   if (CL_Redir)
+      {for (int i = 0; i < RouteClient.DomCnt; i++)
+           {if (RouteClient.Domain[i]) free(RouteClient.Domain[i]);}
+      }
+   for (int i = 0; i < maxDom; i++) RouteClient.Domain[i] = 0;
+   RouteClient.DomCnt = 0;
+   RouteClient.pvtIP  = false;
+   RouteClient.lclDom = false;
+   CL_Redir = true;
+
+// Process arguments
+//
+   if (!(val = Config.GetWord()))
+      {eDest.Emsg("Config", "redirect client argument not specified.");
+       return 1;
+      }
+
+   while(val)
+        {     if (!strcmp("private", val)) RouteClient.pvtIP  = true;
+         else if (!strcmp("local",   val)) RouteClient.lclDom = true;
+         else if (*val == '.')
+                 {if (RouteClient.DomCnt >= maxDom)
+                     {eDest.Emsg("Config",
+                                 "Too many redirect client domains specified.");
+                      return 1;
+                     }
+                  RouteClient.Domain[RouteClient.DomCnt++] = strdup(val);
+                 }
+         else {eDest.Emsg("Config", "Invalid redirect client domain -", val);
+               return 1;
+              }
+         val = Config.GetWord();
+        }
+
+// Set the host parameters
+//
+   xred_set(RD_client, hP, rPort);
+   return 0;
+}
+
+/******************************************************************************/
+
+bool XrdXrootdProtocol::xred_php(char *val, char *hP[2], int rPort[2],
+                                 const char *what, bool optport)
+{
+   XrdNetAddr testAddr;
    char *pp;
 
 // Make sure we have a value
 //
    if (!val || !(*val))
-      {eDest.Emsg("config", "redirect option not specified"); return false;}
+      {eDest.Emsg("config", what, "argument not specified"); return false;}
 
 // Check if we have two hosts here
 //
@@ -1536,7 +1695,7 @@ bool XrdXrootdProtocol::xred_php(char *val, char *hP[2], int rPort[2])
 // Verify corectness here
 //
    if (!(*val) || (hP[1] && !*hP[1]))
-      {eDest.Emsg("Config", "malformed redirect host specification");
+      {eDest.Emsg("Config", "malformed", what, "host specification");
        return false;
       }
 
@@ -1545,12 +1704,26 @@ bool XrdXrootdProtocol::xred_php(char *val, char *hP[2], int rPort[2])
    for (int i = 0; i < 2; i++)
        {if (!(val = hP[i])) break;
         if (!val || !val[0] || val[0] == ':')
-           {eDest.Emsg("Config", "redirect host not specified"); return false;}
-        if (!(pp = rindex(val, ':')))
-           {eDest.Emsg("Config", "redirect port not specified"); return false;}
-        if (!(rPort[i] = atoi(pp+1)))
-           {eDest.Emsg("Config", "redirect port is invalid");    return false;}
-        *pp = '\0';
+           {eDest.Emsg("Config", what, "host not specified"); return false;}
+        if ((pp = rindex(val, ':')))
+           {if ((rPort[i] = XrdOuca2x::a2p(eDest, "tcp", pp+1, false)) <= 0)
+               return false;
+            *pp = '\0';
+           } else {
+            if (optport) rPort[i] = 0;
+               else {eDest.Emsg("Config", what, "port not specified");
+                     return false;
+                    }
+           }
+        const char *eText = testAddr.Set(val, 0);
+        if (eText)
+           {if (XrdNetAddrInfo::isHostName(val) && !strncmp(eText,"Dynamic",7))
+               eDest.Say("Config warning: ", eText, " as ", val);
+               else {eDest.Say("Config failure: ", what, " target ", val,
+                               " is invalid; ", eText);
+                     return false;
+                    }
+            }
        }
 
 // All done
