@@ -28,10 +28,10 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
+#include <cerrno>
 #include <cstdio>
-#include <stdlib.h>
-#include <time.h>
+#include <cstdlib>
+#include <ctime>
 #include <unistd.h>
 #include <sys/types.h>
 
@@ -96,7 +96,7 @@ char               XrdXrootdMonitor::monCLOCK   = 0;
 /*                               G l o b a l s                                */
 /******************************************************************************/
   
-extern          XrdOucTrace       *XrdXrootdTrace;
+extern          XrdSysTrace        XrdXrootdTrace;
 
 namespace XrdXrootdMonInfo
 {
