@@ -27,10 +27,11 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 #include <arpa/inet.h>
 #include <sys/uio.h>
+#include <unistd.h>
 
 #include "XProtocol/XProtocol.hh"
 #include "Xrd/XrdBuffer.hh"
@@ -179,7 +180,19 @@ class IOVec
 {
 public:
 struct iovec *Alloc(int &num)
-                   {if (num > IOV_MAX) num = IOV_MAX;
+                   {static int iovmax = -1;
+                    if (iovmax == -1) {
+#ifdef _SC_IOV_MAX
+                       iovmax = sysconf(_SC_IOV_MAX);
+                       if (iovmax == -1)
+#endif
+#ifdef IOV_MAX
+                          iovmax = IOV_MAX;
+#else
+                          iovmax = 1024;
+#endif
+                    }
+                    if (num > iovmax) num = iovmax;
                     theIOV = new struct iovec[num];
                     return theIOV;
                    }
