@@ -27,8 +27,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <ctype.h>
-#include <stdio.h>
+#include <cctype>
+#include <cstdio>
 #include <sys/uio.h>
 
 #include "XrdSfs/XrdSfsInterface.hh"
@@ -526,6 +526,7 @@ do{if (isFresh)
 // Update offset and advance to next frame
 //
    IO.Offset += Quantum;
+   isFresh = true;
 
   } while(pgwCtl->Advance());
 
